//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// Elements.h
// Contains all visual elements drawn on the screen
//
// Trader logic is handled elsewhere
//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

#define TEL_BGLEFT					1800
#define TEL_BGRIGHT					1750

#define TEL_MONEY					1700	
#define TEL_WAVE					1701
#define TEL_TIME					1702

#define TEL_EXITTEXT				1703
#define TEL_EXIT					1704
#define TEL_AMMO					1705
#define TEL_ARMOR					1706
#define TEL_HELP					1707
#define TEL_LOG						1708

#define TEL_SELLSUB					1709
#define TEL_HEDLEFT					1710
#define TEL_HEDRIGHT				1711

#define TEL_ARMORIND				1713
#define TEL_ARMORTXT				1712

#define TM_ARMOR					0
#define TM_AMMO						1

// Padding between list elements
#define TEL_PADDING					4.0

// Width of the price tag
#define PRICETAG					89.0

// Trader elements
namespace TElem
{
	using TraderPics;
	using TraderEnums;
	using TraderUI : TraderState, DoHandshake;
	
	private int dynInitialized;			// Initialized dynamics?
	private int initialized;			// Initialized statics?
	private int ResX, ResY;
	private int CenX, CenY;
	
	private str SecondString;
	
	// Regions for interactive elements
	private struct Rect					BUT_EXIT, BUT_AMMO, BUT_ARMOR;
	private struct Rect					REC_SELL, REC_BUY;
	
	// Hover state of our elements
	// If these are changed, we should re-draw the elements
	private bool 								HOV_EXIT, HOV_AMMO, HOV_ARMOR;
	
	// --==--==--==--==--==--==--==--==--==--==--==--==--==--==
	// Initialize STATIC trader elements
	//
	// This is called when the trader opens, or does
	// a DRAMATIC change between pages / states
	//
	// Closing the trader will also clear these messages
	// --==--==--==--==--==--==--==--==--==--==--==--==--==--==
	void TE_Init(bool clear = false)
	{
		ResX = ScreenWidth() * 1.0;
		ResY = ScreenHeight() * 1.0;
		
		int counter;
		str text = StrEval(clear, "a", "");
		
		initialized = !clear;
		
		CenX = FixedMul(ResX, 0.5);
		CenY = FixedMul(ResY, 0.5);
		
		// BG behind left and right items
		counter = ZTHudSegment(0.0, 0.0, ResY, TEL_BGLEFT, SideBGTop, SideBGMid, SideBGMid, FOREVER, clear);
		ZTImgMessage(SideTabRed.Tex, Half(SideBGTop.W), 95.0, counter, text, ALIGN_CENTER, ALIGN_BOTTOM, FOREVER);
		SetRect(REC_SELL, 0.0, 0.0, SideBGMid.W, ResY);
		
		counter = ZTHudSegment(ResX - SideBGMid.W, 0.0, ResY, TEL_BGRIGHT, SideBGTop, SideBGMid, SideBGMid, FOREVER, clear);
		ZTImgMessage(SideTabRed.Tex, ResX - Half(SideBGTop.W), 95.0, counter, text, ALIGN_CENTER, ALIGN_BOTTOM, FOREVER);
		SetRect(REC_BUY, ResX - SideBGMid.W, 0.0, SideBGMid.W, ResY);
		
		// Draw the text at the top left
		int NameX = Half(SideBGTop.W);
		int NameY = 95.0 - Half(SideTabRed.H);
		
		SetFont("TFNDESC");
		ZTHudMessage(NameX, NameY - 4.0, TEL_SELLSUB, StrEval(clear, "\cUMake a quick buck!", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
	
		// Top headers
		int HX = Half(SideBGTop.W);
		int HY = 36.0;
		SetFont("TFNTEXIT");
		ZTHudMessage(HX, HY, TEL_HEDLEFT, StrEval(clear, "SELL", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		
		HX = ResX - HX;
		ZTHudMessage(HX, HY, TEL_HEDRIGHT, StrEval(clear, "BUY", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		
		// Money background
		ZTImgMessage(MoneyBack.Tex, CenX, 117.0, TEL_MONEY, text, ALIGN_CENTER, ALIGN_TOP, FOREVER);
		
		// Wave should never change mid-trader, so it's "permanent"
		// We show the wave we're GOING INTO
		SetFont("TFNTAB");
		str WaveText = StrParam(s:"Wave ", d:CheckInventory("HUDDummy_Wave"), s:" / 10");
		ZTHudMessage(CenX, 33.0, TEL_WAVE, StrEval(clear, WaveText, ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		
		// -- Update some inventory-related things
		UpdateCash(CheckInventory("Money"));
		UpdateTime(60);
		
		// Set locations for center elements
		int Y = (ResY-26.0)-ExitBut.H;
		SetRect(BUT_EXIT, CenX-Half(ExitBut.W), Y, ExitBut.W, ExitBut.H);
		
		Y -= 9.0 + (ArmorBut.H);
		SetRect(BUT_ARMOR, BUT_EXIT.X, Y, ArmorBut.W, ArmorBut.H);
		SetRect(BUT_AMMO, (BUT_EXIT.X + BUT_EXIT.W) - AmmoBut.W, Y, AmmoBut.W, AmmoBut.H);
		
		// Help!
		SetFont("TFNDESC");
		
		str HelpText = StrParam(s:"\cU[Strafe Left]: ", s:StrEval(TraderState != TS_PERKS, "Exit", "Previous Menu"));
		HelpText = StrParam(s:"\cU[Fire]: Select Option\n", s:HelpText);
		
		ZTHudMessage(CenX, BUT_AMMO.Y - 8.0, TEL_HELP, StrEval(clear, HelpText, ""), ALIGN_CENTER, ALIGN_BOTTOM, FOREVER);
		
		Tooltips::TT_Init();
	}
	
	// --==--==--==--==--==--==--==--==--==--==--==--==--==--==
	// Draw dynamic elements, these are interactive
	// --==--==--==--==--==--==--==--==--==--==--==--==--==--==
	int TraderSeconds, TraderCash, TraderArmor;
	
	// If these are true, then we need to update these
	bool DirtyCash, DirtySeconds, DirtyArmor;
	
	void TE_Dynamics(bool clear)
	{
		int myCash = CheckInventory("Money");
		if (TraderCash != myCash)
		{
			DirtyCash = true;
			ModeGuns::MoneyUpdated();
			ModeUpgrade::MoneyUpdated();
		}
		
		if (TraderArmor != CheckInventory("Armor"))
			DirtyArmor = true;

		// Cash (Dirty only)
		if (DirtyCash || !dynInitialized)
		{
			DirtyCash = false;
			TraderCash = CheckInventory("Money");
			SetFont("TFNPRIC");
			ZTHudMessage(CenX, 117.0 + Half(MoneyBack.H), TEL_MONEY-1, StrEval(clear, StrParam(s:"$",d:TraderCash), ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		}

		// Time (Dirty only)
		if (DirtySeconds || !dynInitialized)
		{
			DirtySeconds = false;
			SetFont("TFNTIME");
			ZTHudMessage(CenX, 71.0, TEL_TIME, StrEval(clear, SecondString, ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		}
		
		// Armor
		if (DirtyArmor || !dynInitialized)
		{
			int ARM = CheckInventory("Armor");
			TraderArmor = ARM;
			DirtyArmor = false;
			
			// Armor indicator
			int armX = (BUT_ARMOR.X - Half(BUT_ARMOR.W)) + 16.0;
			int armY = BUT_ARMOR.Y + Half(BUT_ARMOR.H);
			
			// Which picture to use?
			str armPic = "TRADARMS";
			if (ARM >= 100)
				armPic = "TRADARMG";
			else if (ARM >= 50)
				armPic = "TRADARMY";
			
			ZTImgMessage(armPic, armX - 1.0, armY, TEL_ARMORIND, StrEval(clear, "a", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
			
			// Armor Text
			SetFont("TFNDESC");
			ZTHudMessage(armX, armY - 4.0, TEL_ARMORTXT, StrEval(clear, StrParam(d:CheckInventory("Armor")), ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		}
		
		// Draw the exit button (Hover state change only)
		if (Mousing(BUT_EXIT) != HOV_EXIT || !dynInitialized)
		{
			HOV_EXIT = Mousing(BUT_EXIT);
			
			if (HOV_EXIT)
				LocalAmbientSound("trader/perkover", 127);
			
			ZTImgMessage(StrEval(HOV_EXIT, ExitBut.Tex, ExitButOver.Tex), BUT_EXIT.X, BUT_EXIT.Y, TEL_EXIT, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
			
			// Exit text
			SetFont("TFNTEXIT");
			ZTHudMessage(BUT_EXIT.X + Half(BUT_EXIT.W), (BUT_EXIT.Y + Half(BUT_EXIT.H)) - 5.0, TEL_EXITTEXT, StrEval(clear, "EXIT MENU", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		}
		
		// Ammo button (Hover state change only)
		bool MA = Mousing(BUT_AMMO);
		bool MAA = Mousing(BUT_ARMOR);
		
		if (MA != HOV_AMMO || MAA != HOV_ARMOR)
		{
			if (MA)
			{
				LocalAmbientSound("trader/perkover", 127);
				Tooltips::TT_Set(TTM_AMMO);
			}
			else if (MAA)
			{
				LocalAmbientSound("trader/perkover", 127);
				Tooltips::TT_Set(TTM_ARMOR);
			}
			else
				Tooltips::TT_Set();
		}
		
		if (MA != HOV_AMMO || !dynInitialized)
		{	
			HOV_AMMO = MA;
			
			ZTImgMessage(StrEval(HOV_AMMO, AmmoBut.Tex, AmmoButOver.Tex), BUT_AMMO.X, BUT_AMMO.Y, TEL_AMMO, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
		}
		
		// Armor button (Hover state change only)
		if (MAA != HOV_ARMOR || !dynInitialized)
		{
			HOV_ARMOR = MAA;
			
			ZTImgMessage(StrEval(HOV_ARMOR, ArmorBut.Tex, ArmorButOver.Tex), BUT_ARMOR.X, BUT_ARMOR.Y, TEL_ARMOR, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
		}
		
		// The weapons in our inventory are ALWAYS drawn
		
		// TRADER STATE
		switch (TraderState)
		{
			default:
			case TS_PERKS:
				ModePerks::M_Tic();
			break;
			
			case TS_UPGRADES:
				ModeUpgrade::M_Tic();
			break;
			
			case TS_GUNS:
				ModeGuns::M_Tic();
			break;
		}
		
		SellList::M_Tic();
		
		Tooltips::TT_Draw();
		
		if (clear)
			TraderLog("");
		
		dynInitialized = !clear;
	}
	
	// Called every tic
	void TE_Tic()
	{
		int secTick = 99;
		
		if (!initialized) { TE_Init(false); }
		
		Scrollbars::Bar_Tic();
		
		// Update seconds appropriately
		secTick ++;
		if (secTick >= 5)
		{
			secTick = 0;
			UpdateTime(HUDInfo.TraderTime);
		}
		
		TE_Dynamics(false);
	}
	
	// Called when the menu closes
	void TE_Cleanup(bool swapped = false, bool FromUpgrade = false)
	{
		dynInitialized = false;
		initialized = false;
		
		TE_Init(true);
		TE_Dynamics(true);
		
		BlankMessage(LID_CURSOR);
		
		if (!swapped || FromUpgrade)
			SellList::M_Cleanup();
			
		ModeGuns::M_Cleanup();
		ModePerks::M_Cleanup();
		ModeUpgrade::M_Cleanup();
		Tooltips::TT_Cleanup();
		// ModeUpgrades.M_Cleanup();
	}
	
	// Right clicked something
	void TE_MouseRightClick()
	{
		SellList::M_MouseRightClick();
	}
	
	// Clicked something
	void TE_MouseClick()
	{
		// Clicked exit!
		if (Mousing(BUT_EXIT))
		{
			NamedRequestScriptPuke("TraderAbort");
			return;
		}
		
		// Clicked armor
		if (Mousing(BUT_ARMOR))
		{
			LocalAmbientSound("trader/perkclick", 127);
			
			if (CheckInventory("Money") < GetArmorCost())
			{
				TraderLog("\cIYou can't afford that!");
				return;
			}
			else if (CheckInventory("Armor") >= 100)
			{
				TraderLog("\cIArmor is already full!");
				return;
			}
			ACS_NamedExecuteAlways("BuyAmmoArmor", 0, TM_ARMOR);
			return;
		}
		
		// Clicked ammo
		if (Mousing(BUT_AMMO))
		{
			LocalAmbientSound("trader/perkclick", 127);
			
			if (CheckInventory("Money") < GetRefillCost())
			{
				TraderLog("\cIYou can't afford that!");
				return;
			}
			
			ACS_NamedExecuteAlways("BuyAmmoArmor", 0, TM_AMMO);
			return;
		}
		
		Scrollbars::Bar_Click();
		
		SellList::M_MouseClick();
		
		switch (TraderState)
		{
			case TS_GUNS:
				ModeGuns::M_MouseClick();
			break;
			
			default:
			case TS_PERKS:
				ModePerks::M_MouseClick();
			break;
			
			case TS_UPGRADES:
				ModeUpgrade::M_MouseClick();
			break;
		}
	}
	
	// Released mouse
	// Used for scrollbars, mainly
	void TE_MouseRelease()
	{
		Scrollbars::Bar_Release();
		
		switch (TraderState)
		{
			case TS_GUNS:
				ModeGuns::M_MouseRelease();
			break;
			
			default:
			case TS_PERKS:
				ModePerks::M_MouseRelease();
			break;
		}
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	
	void TraderLog(str Text)
	{
		SetFont("TFNPRIC");
		ZTHudMessage(CenX, BUT_AMMO.Y - 64.0, TEL_LOG, Text, ALIGN_CENTER, ALIGN_BOTTOM, 1.0);
	}
	
	script "TraderAbort" (void) NET
	{
		LocalAmbientSound("trader/exit", 127);
		ACS_ExecuteAlways(SCR_TRADEROPEN, 0, 1);
	}
	
	// Buy ammo / armor
	script "BuyAmmoArmor" (int mode) NET
	{
		int Price;
		
		// Armor
		if (mode == TM_ARMOR)
		{
			Price = GetArmorCost();
			GiveInventory("GreenArmor", 1);
			TakeInventory("Money", Price);
			DoHandshake(HS_BUYARMOR);
		}
		
		else if (mode == TM_AMMO)
		{
			Price = GetRefillCost();
			TakeInventory("Money", Price);
			GiveInventory("Backpack", 1);
			DoHandshake(HS_BUYAMMO);
		}
	}
	
	// Server -> Client
	void M_BoughtAmmo()
	{
		LocalAmbientSound("trader/ammo", 127);
		TraderLog("You bought ammo!");
	}
	
	void M_BoughtArmor()
	{
		LocalAmbientSound("trader/armor", 127);
		TraderLog("You feel protected!");
	}
	
	void UpdateCash(int val)
	{
		if (TraderCash == val)
			return;
			
		TraderCash = val;
		DirtyCash = true;
	}
	
	void UpdateTime(int val)
	{
		if (TraderSeconds == val)
			return;
			
		TraderSeconds = val;
		
		SecondString = TimeString(traderSeconds);
		
		DirtySeconds = true;
	}
	
	str GetMinutes(int val)
	{
		str returner = "";
		int minutes;
		
		// Split the trader text into minutes
		minutes = Shave(FixedDiv(val*1.0,60.0));
		if (minutes < 10.0)
			returner = StrParam(s:"0",f:minutes);
		else
			returner = StrParam(f:minutes);
			
		return returner;
	}

	str GetSeconds(int val)
	{
		int seconds;
		
		str returner = "";
		// Split the trader text into seconds
		seconds = val % 60;
		if (seconds < 10)
			returner = StrParam(s:"0",d:seconds);
		else
			returner = StrParam(d:seconds);
				
		return returner;
	}
}
