// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// W E A P O N   M O D E
// We're currently viewing a list of weapons to buy
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

// Maximum of guns PER CATEGORY that we can parse
// If you have 99 guns for a single perk, I'm impressed
#define 			MAX_FILTER					99;

// This is the same HID as perks, this is intentional
// Make sure perks are cleaned up BEFORE we draw guns
#define				TEL_FILTERNAME				1601
#define				TEL_GUNS					1600
#define				TEL_GUNBAR					1400

// Text is drawn -1 from this
#define				TEL_GOBACK					1603

// Number of elements drawn in a single tab
// This is subtracted per iteration
#define				GUNTAB_ELEMS				3

// TODO: CALCULATING GUN PRICE WHEN WE DRAW THE TAB MIGHT NOT BE A GOOD IDEA
// STORE IT WHEN WE GENERATE THE FILTER LIST

struct FilteredGunT
{
	int ID, Price;
	bool CashDirty, Affordable;
};

namespace TElem::ModeGuns
{
	using Scrollbars:barHeld, DrawScrollbar, ScrollbarDirty, ScrollIndex, BarNeeded;
	using TraderUI:FilterPerk, Handshaking, DoHandshake, SwapTraderState;
	
	// Stored before we do a handshake
	private int HandHash;
	
	private int modeReady;
	private int maxGuns;
	
	// Stored for later, half height of the tab
	private int TabHalfHeight;
	
	// Which gun we're mousing over
	private int MousedGun, MousedGunOld;
	
	// Hovering over the back button
	private bool HoverBack;
	
	// Contains a list of shop weapon INDEXES
	// This is called on initialization, keeps us looping as much
	private FilteredGunT filteredGuns[MAX_FILTER];
	private int filteredCount = 0;
	
	// Contains the actual NAME of the perk we're filtering by
	private str filterName = "";
	
	// Regions
	private struct Rect			REC_GUNAREA, REC_GUNSCROLL, REC_GUNBACK;
	private ScrollbarT			REC_GUNBAR;
	
	void ResetFilter()
	{
		filteredCount = 0;
		for (int l=0; l<MAX_FILTER; l++)
		{
			filteredGuns[l].ID = -1;
			filteredGuns[l].Price = 0;
			filteredGuns[l].Affordable = false;
		}
	}
	
	// Draw the actual perk elements
	void M_Draw(bool clear)
	{
		int HID = TEL_GUNS;
		
		// Do we even need a bar?
		bool UseBar = BarNeeded(filteredCount, maxGuns);
		
		int counter = 0;
		int ListBegin = 0;
		if (UseBar)
			ListBegin = ScrollIndex(filteredCount, maxGuns, REC_GUNBAR);
		
		MousedGun = -1;
		
		bool SBDirty = UseBar && ScrollbarDirty(REC_GUNBAR);
		
		for (int l=ListBegin; l<ListBegin+maxGuns; l++)
		{
			// This is above the REAL amount
			if (l >= filteredCount)
			{
				BlankMessage(HID);
				BlankMessage(HID-1);
				BlankMessage(HID-2);
				BlankMessage(HID-3);
				
				HID -= GUNTAB_ELEMS;
				continue;
			}
			
			int Y = GunTabY(counter);

			// Mouse is over this specific perk
			if (!barHeld && Between(GetMouseX(), REC_GUNAREA.X, REC_GUNSCROLL.X) && Between(GetMouseY(), Y, Y+BuyGun.H))
				MousedGun = l;
				
			if (SBDirty || MousedGunOld != MousedGun || !modeReady || filteredGuns[l].CashDirty)
				DrawGunTab(l, Y, HID, clear);
				
			if (MousedGunOld != MousedGun && MousedGun == l)
				TElem::Tooltips::TT_Set(TTM_WEAPON, filteredGuns[l].ID, filteredGuns[l].Price);
				
			filteredGuns[l].CashDirty = false;
			
			counter ++;
			HID -= GUNTAB_ELEMS;
		}
		
		if (!modeReady || SBDirty || clear)
			DrawScrollbar(REC_GUNBAR, TEL_GUNBAR, clear, UseBar);
			
		// Draw the name at the top
		if (!modeReady || clear)
		{
			int NameX = ResX - Half(SideBGTop.W);
			int NameY = 95.0 - Half(SideTabRed.H);
			
			SetFont("TFNNAME");
			ZTHudMessage(NameX, NameY - 4.0, TEL_FILTERNAME, StrEval(clear, filterName, ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		}
		
		// Exit button
		if ((!barHeld && HoverBack != Mousing(REC_GUNBACK)) || !modeReady || clear)
		{
			if (!barHeld)
				HoverBack = Mousing(REC_GUNBACK);
				
			ZTImgMessage(StrEval(HoverBack, GoBack.Tex, GoBackOver.Tex), REC_GUNBACK.X, REC_GUNBACK.Y, TEL_GOBACK, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
			
			SetFont("TYPEWRIT");
			int GBX = (REC_GUNBACK.X + REC_GUNBACK.W) - 12.0;
			int GBY = REC_GUNBACK.Y + Half(REC_GUNBACK.H);
			ZTHudMessage(GBX, GBY, TEL_GOBACK-1, StrEval(clear, "P E R K S", ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
		}
		
		// Stopped mousing over something, hide tooltip
		if (MousedGun == -1 && MousedGunOld != -1)
			TElem::Tooltips::TT_Set();
		
		MousedGunOld = MousedGun;
		
		modeReady = !clear;
	}
	
	// Initialize starting variables
	void M_Init(bool clear)
	{
		ResetFilter();
		
		if (clear)
			return;
			
		TabHalfHeight = Half(BuyGun.H);
		
		DetectBuyable();
		
		// Rectangles for some things
		SetRect(REC_GUNAREA, ResX-256.0, 98.0, (BuyGun.W-PRICETAG)-ScrollBack.W, (ResY - 98.0) - GoBack.H);
		SetRect(REC_GUNSCROLL, ResX-ScrollBack.W, REC_GUNAREA.Y, ScrollBack.W, REC_GUNAREA.H);
		
		// Go back
		SetRect(REC_GUNBACK, ResX-GoBack.W, ResY-GoBack.H, GoBack.W, GoBack.H);
		
		SetScrollbar(REC_GUNBAR, REC_GUNSCROLL);
		
		// Calculate how many guns we can hold based on our resolution
		maxGuns = ShaveInt(FixedDiv(REC_GUNAREA.H, (BuyGun.H + TEL_PADDING)));
		
		// Set up the filter name
		int PI = PerkByClass(FilterPerk);
		filterName = ZT2Perks[PI].Name;
	}
	
	// Called every tic
	void M_Tic()
	{
		if (!modeReady) { M_Init(false); }
		M_Draw(false);
	}
	
	// Cleaned up
	void M_Cleanup()
	{
		modeReady = false;
		M_Init(true);
		M_Draw(true);
	}
	
	// Release!
	void M_MouseRelease() {}
	
	// Click!
	void M_MouseClick()
	{
		// Clicked back!
		if (Mousing(REC_GUNBACK))
		{
			LocalAmbientSound("trader/back", 127);
			SwapTraderState(TS_PERKS);
			return;
		}
		
		if (MousedGun == -1)
			return;
			
		LocalAmbientSound("trader/wepclick", 127);
			
		// We clicked a weapon, let's try to buy it!
		if (CheckInventory("Money") < GetGunPrice(MousedGun))
		{
			TElem::TraderLog("\cKYou don't have enough funds!");
			return;
		}
		
		// Actually attempt to purchase the gun
		Handshaking = true;
		HandHash = GetGunHash(MousedGun);
		NamedRequestScriptPuke("Guns_Purchase", GetGunHash(MousedGun));
	}
	
	// Successfully bought a gun!
	void M_Handshook()
	{
		int WI = WeaponByHash(HandHash);
		LocalAmbientSound("trader/wepbuy", 127);
		TraderLog(StrParam(s:"\cDYou've purchased the \cJ", s:WeaponList[WI].DisplayName, s:"\cD."));
		DetectBuyable(true);
		TElem::SellList::DetectGuns(true);
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - 
	
	
	str GetGunName(int ind) { return WeaponList[ filteredGuns[ind].ID ].DisplayName; }
	
	// We gained or lost money
	// Update the "can buy" status of our filtered guns
	void MoneyUpdated()
	{
		int Dosh = CheckInventory("Money");
		bool greasy = false;
		
		for (int l=0; l<filteredCount; l++)
		{
			// It was too expensive, but now it's not
			if (Dosh >= filteredGuns[l].Price && !filteredGuns[l].Affordable)
			{
				greasy = true;
				filteredGuns[l].Affordable = true;
				filteredGuns[l].CashDirty = true;
			}
			
			// It was affortable, but now it's not
			else if (Dosh < filteredGuns[l].price && filteredGuns[l].Affordable)
			{
				greasy = true;
				filteredGuns[l].Affordable = false;
				filteredGuns[l].CashDirty = true;
			}
		}
	}
	
	void DetectBuyable(bool handshook = false)
	{
		if (handshook)
		{
			modeReady = false;
			ResetFilter();
		}
			
		// Loop through all weapons and find those that match our category
		for (int l=0; l<WeaponCount; l++)
		{
			// Local checks were done here, but we should always have one
			// list of weapons in both singleplayer and net games
			if (!CheckInventory(WeaponList[l].Class) && WeaponList[l].Perk == FilterPerk && !WeaponList[l].Hidden)
			{
				filteredGuns[filteredCount].ID = l;
				filteredGuns[filteredCount].Price = FinalPrice(l);
				filteredGuns[filteredCount].Affordable = (CheckInventory("Money") >= filteredGuns[filteredCount].Price);
				filteredCount ++;
			}
		}
	}
	
	// Gun hash, important for sync
	int GetGunHash(int ind)
	{
		return WeaponList[ filteredGuns[ind].ID ].Hash;
	}
	
	// FINAL PRICE of this index, filters in discounts and things
	int GetGunPrice(int ind)
	{
		return filteredGuns[ind].Price;
	}
	
	// Get Y position of a certain gun button
	int GunTabY(int num)
	{
		return REC_GUNAREA.Y + (num * (BuyGun.H + TEL_PADDING));
	}
	
	// Draw the actual gun tab
	void DrawGunTab(int l, int Y, int HID, bool clear)
	{
		int RealNum = filteredGuns[l].ID;
		int TabHalf = Y + TabHalfHeight;
		int GPrice = GetGunPrice(l);
		bool poor = (CheckInventory("Money") < GPrice);
		
		// Gun Tab
		str TexA = StrEval(poor, BuyGun.Tex, BadGun.Tex);
		str TexB = StrEval(poor, BuyGunOver.Tex, BadGunOver.Tex);
		
		ZTImgMessage(StrEval(MousedGun == l, TexA, TexB), REC_GUNAREA.X - PRICETAG, Y, HID, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
		
		// Gun Name
		SetFont("TFNTAB");
		int PNX = REC_GUNBAR.Bounds.X - 12.0;
		
		str col = StrEval(poor, "\cJ", "\cU");
		
		ZTHudMessage(PNX, TabHalf, HID-1, StrParam(s:col, s:GetGunName(l)), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
		
		// Gun PRICE
		SetFont("TFNPRIC");
		int PriceX = REC_GUNAREA.X - 12.0;
		ZTHudMessage(PriceX, TabHalf, HID-2, StrEval(clear, StrParam(s:"$", d:GPrice), ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
	}
	
	script "Guns_Purchase" (int gunHash) NET
	{
		int WI = WeaponByHash(gunHash);
		if (WI == -1)
			terminate;
			
		// Do we have enough money?
		int price = WeaponList[WI].Price;
		
		if (CheckInventory("Money") < FinalPrice(WI))
			terminate;
			
		GiveInventory(WeaponList[WI].Class, 1);
		TakeInventory("Money", price);
		
		DoHandshake(HS_BUYGUN);
	}
}
