// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// P E R K   S E L E C T   M O D E
// We're currently viewing a list of perk CATEGORIES
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

#define				TEL_PERKHEADER				1601
#define				TEL_PERKS					1600
#define				TEL_PERKBAR					1550

namespace TElem::ModePerks
{
	using Scrollbars:barHeld, DrawScrollbar, ScrollbarDirty, ScrollIndex, BarNeeded;
	using TraderEnums;
	using TraderUI : SwapTraderState;
	
	private int modeReady;
	private int MousedPerk, MousedPerkOld;
	
	// Max amount of perks we can hold on the screen at once
	private int maxPerks;
	
	// Regions
	private struct Rect					REC_PERKAREA;
	private struct Rect					REC_PERKSCROLL;
	private ScrollbarT					REC_PERKBAR;
	
	// Draw the actual perk elements
	void M_Draw(bool clear)
	{
		int HID = TEL_PERKS;
		
		// Do we even need a bar?
		bool UseBar = BarNeeded(ZT2PerkCount, maxPerks);
		
		MousedPerk = -1;
		
		bool SBDirty = UseBar && ScrollbarDirty(REC_PERKBAR);
		
		// For all the perks we can hold..
		int counter = 0;
		int ListBegin = 0;
		if (UseBar)
			ListBegin = ScrollIndex(ZT2PerkCount, maxPerks, REC_PERKBAR);
		
		for (int l=ListBegin; l<ListBegin+maxPerks; l++)
		{
			// This is above the REAL amount
			if (l >= ZT2PerkCount)
			{
				BlankMessage(HID);
				BlankMessage(HID-1);
				
				HID -= 2;
				continue;
			}
			
			int Y = PerkTabY(counter);

			// Mouse is over this specific perk
			if (!barHeld && Between(GetMouseX(), REC_PERKAREA.X, REC_PERKSCROLL.X) && Between(GetMouseY(), Y, Y+PerkTab.H))
				MousedPerk = l;
				
			if (SBDirty || MousedPerkOld != MousedPerk || !modeReady)
			{
				if (MousedPerk == l)
					TElem::Tooltips::TT_Set(TTM_PERK, l);
					
				// Perk Tab
				ZTImgMessage(StrEval(MousedPerk == l, PerkTab.Tex, PerkTabOver.Tex), REC_PERKAREA.X, Y, HID, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
				
				// Perk Name - IN THE LIST
				SetFont("TFNPRIC");
				int PNX = REC_PERKAREA.X + REC_PERKAREA.W + 2.0;
				ZTHudMessage(PNX, Y + Half(PerkTab.H), HID-1, StrEval(clear, ZT2Perks[l].Name, ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
			}
			
			counter ++;
			HID -= 2;
		}
		
		// Draw the text at the top
		if (!modeReady || clear)
		{
			int NameX = ResX - Half(SideBGTop.W);
			int NameY = 95.0 - Half(SideTabRed.H);
			
			SetFont("TFNDESC");
			ZTHudMessage(NameX, NameY - 4.0, TEL_PERKHEADER, StrEval(clear, "\cUSelect a Perk Category!", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		}
		
		if (!modeReady || SBDirty || clear)
			DrawScrollbar(REC_PERKBAR, TEL_PERKBAR, clear, UseBar);
		
		// Stopped mousing over something, hide tooltip
		if (MousedPerk == -1 && MousedPerkOld != -1)
			TElem::Tooltips::TT_Set();
		
		MousedPerkOld = MousedPerk;
		
		modeReady = !clear;
	}
	
	// Initialize starting variables
	void M_Init(bool clear)
	{
		SetRect(REC_PERKAREA, ResX-264.0, 98.0, PerkTab.W-ScrollBack.W, ResY - 98.0);
		SetRect(REC_PERKSCROLL, ResX-ScrollBack.W, REC_PERKAREA.Y, ScrollBack.W, REC_PERKAREA.H);
		
		SetScrollbar(REC_PERKBAR, REC_PERKSCROLL);
		
		// Calculate how many perks we can hold based on our resolution
		maxPerks = ShaveInt(FixedDiv(REC_PERKAREA.H, (PerkTab.H + TEL_PADDING)));
	}
	
	// Called every tic
	void M_Tic()
	{
		if (!modeReady) { M_Init(false); }
		M_Draw(false);
	}
	
	// Cleaned up
	void M_Cleanup()
	{
		MousedPerk = -1;
		MousedPerkOld = -1;
		modeReady = false;
		M_Init(true);
		M_Draw(true);
	}
	
	void M_MouseRelease() {}
	
	// Click!
	void M_MouseClick()
	{
		if (MousedPerk >= 0)
		{
			LocalAmbientSound("trader/perkclick", 127);
			str PerkItem = ZT2Perks[MousedPerk].Item;
			TraderUI::FilterPerk = PerkItem;
			SwapTraderState(TS_GUNS);
		}
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - 
	
	// Get Y position of a certain perk button
	int PerkTabY(int num)
	{
		return REC_PERKAREA.Y + (num * (PerkTab.H + TEL_PADDING));
	}
}
