// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// U P G R A D E   M O D E
// We're currently viewing a list of upgrades
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

#define UPTAB_ELEMS 3

namespace TElem::ModeUpgrade
{
	using TraderUI:UpgradeGun, Handshaking, DoHandshake, SwapTraderState;
	
	private FilteredGunT filteredUpgrades[MAX_FILTER];
	private int filteredCount = 0;
	
	// Is this mode ready?
	private int modeReady;
	
	// Which gun we're mousing over
	private int MousedGun, MousedGunOld;
	
	// Money changed!
	private int MoneyDirty;
	
	// Upgrade we last purchased
	private int HandHash, GunHash;
	
	// Hovering over the back button
	private bool HoverBack;
	
	// Regions
	private struct Rect			REC_GUNAREA, REC_GUNBACK;
	
	void ResetFilter()
	{
		filteredCount = 0;
		for (int l=0; l<MAX_FILTER; l++)
		{
			filteredUpgrades[l].Price = 0;
			filteredUpgrades[l].Affordable = false;
		}
	}
	
	// Draw things!
	void M_Draw(bool clear)
	{
		int HID = TEL_GUNS;
		
		MousedGun = -1;
		
		// Draw the text at the top
		if (!modeReady || clear)
		{
			int NameX = ResX - Half(SideBGTop.W);
			int NameY = 95.0 - Half(SideTabRed.H);
			
			SetFont("TFNDESC");
			ZTHudMessage(NameX, NameY - 4.0, TEL_PERKHEADER, StrEval(clear, "\cUIncrease your weapon efficiency!", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		}
		
		for (int l=0; l<WeaponList[UpgradeGun].UpgradeCount; l++)
		{
			int Y = GunTabY(l);
			
			if (Between(GetMouseX(), REC_GUNAREA.X, REC_GUNAREA.X+REC_GUNAREA.W) && Between(GetMouseY(), Y, Y+BuyGun.H))
				MousedGun = l;
				
			if (MousedGunOld != MousedGun || !modeReady || filteredUpgrades[l].CashDirty)
				DrawGunTab(l, Y, HID, clear);
				
			if (MousedGunOld != MousedGun && MousedGun == l)
				TElem::Tooltips::TT_Set(TTM_UPGRADE, l, GetUpgradePrice(l));
				
			filteredUpgrades[l].CashDirty = false;
			
			HID -= UPTAB_ELEMS;
		}
		
		// Exit button
		if (HoverBack != Mousing(REC_GUNBACK) || !modeReady || clear)
		{
			HoverBack = Mousing(REC_GUNBACK);
				
			ZTImgMessage(StrEval(HoverBack, GoBack.Tex, GoBackOver.Tex), REC_GUNBACK.X, REC_GUNBACK.Y, TEL_GOBACK, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
			
			SetFont("TYPEWRIT");
			int GBX = (REC_GUNBACK.X + REC_GUNBACK.W) - 12.0;
			int GBY = REC_GUNBACK.Y + Half(REC_GUNBACK.H);
			ZTHudMessage(GBX, GBY, TEL_GOBACK-1, StrEval(clear, "P E R K S", ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
		}
		
		// Stopped mousing over something, hide tooltip
		if (MousedGun == -1 && MousedGunOld != -1)
			TElem::Tooltips::TT_Set();
			
		MousedGunOld = MousedGun;
		
		modeReady = !clear;
	}
	
	// Initialize the menu
	void M_Init(bool clear)
	{
		ResetFilter();
		
		if (clear)
			return;
			
		DetectBuyable();
			
		// Rectangles for some things
		SetRect(REC_GUNAREA, ResX-256.0, 98.0, BuyGun.W-PRICETAG, (ResY - 98.0) - GoBack.H);
		
		// Go back
		SetRect(REC_GUNBACK, ResX-GoBack.W, ResY-GoBack.H, GoBack.W, GoBack.H);
	}
	
	// Called every tic
	void M_Tic()
	{
		if (!modeReady) { M_Init(false); }
		M_Draw(false);
	}
	
	// Cleaned up
	void M_Cleanup()
	{
		modeReady = false;
		M_Init(true);
		M_Draw(true);
	}
	
	// Clicked on something!
	void M_MouseClick()
	{
		// Clicked back!
		if (Mousing(REC_GUNBACK))
		{
			LocalAmbientSound("trader/back", 127);
			SwapTraderState(TS_PERKS, true);
			return;
		}
		
		if (MousedGun == -1)
			return;
			
		// We already have it
		if (HasUpgrade(UpgradeGun, MousedGun))
		{
			TElem::TraderLog("\cKYou already have that upgrade.");
			return;
		}
			
		// Do we even have enough money to buy it?
		int Price = WeaponList[UpgradeGun].Upgrades[MousedGun].Price;
		if (CheckInventory("Money") < Price)
		{
			TElem::TraderLog("\cKYou don't have enough funds!");
			return;
		}
			
		// Actually attempt to purchase the gun
		GunHash = WeaponList[UpgradeGun].Hash;
		HandHash = GetUpgradeHash(MousedGun);

		Handshaking = true;
		NamedRequestScriptPuke("Upgrade_Purchase", WeaponList[UpgradeGun].Hash, HandHash);
	}
	
	// Successfully bought a gun!
	void M_Handshook()
	{
		int WI = WeaponByHash(GunHash);
		int UI = UpgradeByHash(WI, HandHash);
		LocalAmbientSound("trader/upgrade", 127);
		TraderLog(StrParam(s:"\cDYou've purchased the \cJ", s:WeaponList[WI].Upgrades[UI].Name, s:"\cD upgrade!"));
		DetectBuyable(true);
	}
	
	void DetectBuyable(bool handshook = false)
	{
		if (handshook)
			modeReady = false;
			
		filteredCount = WeaponList[UpgradeGun].UpgradeCount;
		for (int l=0; l<filteredCount; l++)
		{
			filteredUpgrades[l].CashDirty = true;
			filteredUpgrades[l].Price = WeaponList[UpgradeGun].Upgrades[l].Price;
			filteredUpgrades[l].Affordable = (CheckInventory("Money") >= WeaponList[UpgradeGun].Upgrades[l].Price);
		}
	}
	
	// - - - - - - - - - - - - - - - - - - -
	
	// We gained or lost money
	// Update the "can buy" status of our filtered guns
	void MoneyUpdated()
	{
		int Dosh = CheckInventory("Money");

		for (int l=0; l<filteredCount; l++)
		{
			// It was too expensive, but now it's not
			if (Dosh >= filteredUpgrades[l].Price && !filteredUpgrades[l].Affordable)
			{
				filteredUpgrades[l].Affordable = true;
				filteredUpgrades[l].CashDirty = true;
			}
			
			// It was affortable, but now it's not
			else if (Dosh < filteredUpgrades[l].price && filteredUpgrades[l].Affordable)
			{
				filteredUpgrades[l].Affordable = false;
				filteredUpgrades[l].CashDirty = true;
			}
		}
	}
	
	// Get Y position of a certain gun button
	int GunTabY(int num)
	{
		return REC_GUNAREA.Y + (num * (BuyGun.H + TEL_PADDING));
	}
	
	// Gun hash, important for sync
	int GetUpgradeHash(int ind)
	{
		return WeaponList[UpgradeGun].Upgrades[ind].Hash;
	}
	
	// FINAL PRICE of this index, filters in discounts and things
	int GetUpgradePrice(int ind)
	{
		return WeaponList[UpgradeGun].Upgrades[ind].Price;
	}
	
	int DrawGunTab(int l, int Y, int HID, bool clear)
	{
		int CNT = HID;
		int GX = REC_GUNAREA.X - PRICETAG;
		int GY = Y;
		int TabHalf = Y + Half(BuyGun.H);
		
		bool poor = !filteredUpgrades[l].Affordable;
		
		// Gun Tab
		str TexA = StrEval(poor, BuyGun.Tex, BadGun.Tex);
		str TexB = StrEval(poor, BuyGunOver.Tex, BadGunOver.Tex);
		
		// Do we have this upgrade?
		bool HasIt = HasUpgrade(UpgradeGun, l);
		if (HasIt)
		{
			TexA = "TRADBUY5";
			TexB = "TRADBUY6";
		}
		
		ZTImgMessage(StrEval(MousedGun == l, TexA, TexB), GX, GY, CNT, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
		CNT --;
		
		// Name
		SetFont("TFNTAB");
		int PNX = (REC_GUNAREA.X + REC_GUNAREA.W) - 8.0;
		
		str col = StrEval(HasIt, StrEval(poor, "\cJ", "\cU"), "\cD");
		
		ZTHudMessage(PNX, TabHalf, CNT, StrEval(clear, StrParam(s:col, s:WeaponList[UpgradeGun].Upgrades[l].Name), ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
		CNT --;
		
		// Gun PRICE
		int GPrice = GetUpgradePrice(l);
		SetFont("TFNPRIC");
		int PriceX = REC_GUNAREA.X - 12.0;
		
		str FinalPriceText = StrEval(HasIt, StrParam(s:"$", d:GPrice), "");
		
		ZTHudMessage(PriceX, TabHalf, CNT, StrEval(clear, FinalPriceText, ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
		CNT --;
		
		return CNT;
	}
	
	script "Upgrade_Purchase" (int gunHash, int upHash) NET
	{
		// Find the weapon we're upgrading
		int WI = WeaponByHash(gunHash);
		if (WI == -1)
			terminate;
			
		// Find the desired upgrade
		int UI = UpgradeByHash(WI, upHash);
		if (UI == -1)
			terminate;
			
		// Do we have enough money?
		int price = WeaponList[WI].Upgrades[UI].Price;
		
		// TODO: CALCULATE FINAL UPGRADE PRICE
		if (CheckInventory("Money") < price)
			terminate;
			
		str FinalItem = StrParam(s:WeaponList[WI].Class, s:" ", s:WeaponList[WI].Upgrades[UI].Item);
		InvEnsure(FinalItem, 1);
		TakeInventory("Money", price);
		
		DoHandshake(HS_UPGRADEGUN);
	}
}