// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// S C R O L L B A R S
// Handy for list things
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

struct ScrollBarT
{
	struct Rect Bounds;
	int Pct;
	int BarY, LastBarY;
};

// Set up a scrollbar struct
void SetScrollbar(ScrollBarT& Bar, struct Rect& Boundaries)
{
	Bar.BarY = Boundaries.Y;
	Bar.LastBarY = Boundaries.Y;
	Bar.Bounds.X = Boundaries.X;
	Bar.Bounds.Y = Boundaries.Y;
	Bar.Bounds.W = Boundaries.W;
	Bar.Bounds.H = Boundaries.H;
} 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

namespace TElem::Scrollbars
{
	using ModePerks;
	using ModeGuns;
	using ModeUpgrade;
	using SellList;
	
	// We're currently holding a scrollbar down
	bool barHeld;
	private ScrollBarT NullBar;
	private ScrollBarT& StoredBar = NullBar;
	
	ScrollBarT& GetBar()
	{
		// Check based on the center of the screen
		// Hacky but it will save some ACS calls
		int MX = GetMouseX();
		
		if (MX >= CenX)
		{
			switch (TraderState)
			{
				default:
				case TS_PERKS:
					return REC_PERKBAR;
				break;
				
				case TS_GUNS:
					return REC_GUNBAR;
				break;
			}
		}
		else
			return REC_SELLBAR;
	}
	
	// Is a scrollbar even required?
	bool BarNeeded(int total, int maximum)
	{
		return (total > maximum);
	}
	
	// First index to start drawing items from
	// This should extend to the end of the list, but
	// any items past the list should not be drawn
	int ScrollIndex(int total, int maximum, ScrollBarT& Bar)
	{
		int LMax = total - maximum;
		return FixedMul(LMax, Bar.pct);
	}
	
	bool ScrollbarDirty(ScrollBarT& Bar)
	{
		if (Bar.LastBarY != Bar.BarY)
		{
			Bar.LastBarY = Bar.BarY;
			return true;
		}
		
		return false;
	}
	
	// Draw a scrollbar
	void DrawScrollbar(ScrollBarT& Bar, int HID, bool remove = false, bool useBar = true)
	{
		HID = ZTHudSegment(Bar.Bounds.X, Bar.Bounds.Y, Bar.Bounds.H, HID, ScrollBack, ScrollMid, ScrollLow, FOREVER, remove);
		
		if (useBar || remove)
			ZTImgMessage(ScrollLine.Tex, Bar.Bounds.X + Bar.Bounds.W, Bar.BarY, HID, StrEval(remove, "a", ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
	}
	
	// Every tic
	void Bar_Tic()
	{
		if (barHeld)
		{
			// Calculate a percentage
			int MY = clamp(GetMouseY(), StoredBar.Bounds.Y, StoredBar.Bounds.Y + StoredBar.Bounds.H);
			int pct = FixedDiv(MY - StoredBar.Bounds.Y, StoredBar.Bounds.H);
			
			StoredBar.Pct = Pct;
			StoredBar.BarY = MY;
		}
	}

	// Clicked
	void Bar_Click()
	{
		ScrollBarT& Bar = GetBar();
		
		if (Mousing(Bar.Bounds))
		{
			StoredBar = Bar;
			barHeld = true;
		}
	}
	
	// Released
	void Bar_Release()
	{
		ScrollBarT& Bar = GetBar();
		barHeld = false;
		StoredBar = NullBar;
	}
}
