// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// S E L L   L I S T
// Controls the weapons we currently own
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

// This is the same HID as perks, this is intentional
// Make sure perks are cleaned up BEFORE we draw guns
#define				TEL_SELLS					1300
#define				TEL_SELLBAR					1100

#define 			SELLTAB_ELEMS				4

namespace TElem::SellList
{
	using TraderUI : Handshaking, DoHandshake, TraderState, SwapTraderState, UpgradeGun;
	using Scrollbars:barHeld, DrawScrollbar, ScrollbarDirty, ScrollIndex, BarNeeded;
	
	// Reference to the hash that we previously tried selling
	private int HandHash;
	
	private int modeReady;
	private int maxGuns;
	
	// Stored for later, half height of the tab
	private int TabHalfHeight;
	
	// Which gun we're mousing over
	private int MousedSell, MousedSellOld;
	
	// Contains a list of shop weapon INDEXES
	// This is called on initialization, keeps us looping as much
	private int myGuns[MAX_WEPS];
	private int myCount = 0;
	
	// Regions
	private struct Rect			REC_SELLAREA, REC_SELLSCROLL;
	private ScrollbarT			REC_SELLBAR;
	
	void ResetMine()
	{
		myCount = 0;
		for (int l=0; l<MAX_WEPS; l++)
			myGuns[l] = -1;
	}
	
	// Draw the actual perk elements
	void M_Draw(bool clear)
	{
		int HID = TEL_SELLS;
		
		// Do we even need a bar?
		bool UseBar = BarNeeded(myCount, maxGuns);
		
		int counter = 0;
		int ListBegin = 0;
		if (UseBar)
			ListBegin = ScrollIndex(myCount, maxGuns, REC_SELLBAR);
		
		MousedSell = -1;
		
		bool SBDirty = UseBar && ScrollbarDirty(REC_SELLBAR);
		
		for (int l=ListBegin; l<ListBegin+maxGuns; l++)
		{
			// This is above the REAL amount
			if (l >= myCount)
			{
				BlankMessage(HID);
				BlankMessage(HID-1);
				BlankMessage(HID-2);
				BlankMessage(HID-3);
				BlankMessage(HID-4);
				
				HID -= SELLTAB_ELEMS;
				continue;
			}
			
			int Y = SellTabY(counter);

			// Mouse is over this specific perk
			if (TraderState != TS_UPGRADES && !barHeld && Between(GetMouseX(), REC_SELLAREA.X, REC_SELLAREA.X+REC_SELLAREA.W) && Between(GetMouseY(), Y, Y+SellGun.H))
				MousedSell = l;
				
			if (SBDirty || MousedSellOld != MousedSell || !modeReady)
				DrawSellTab(l, Y, HID, clear);
				
			if (MousedSellOld != MousedSell && MousedSell == l)
				TElem::Tooltips::TT_Set(TTM_WEAPON, myGuns[l], FinalSellPrice(myGuns[l]), true);
			
			counter ++;
			HID -= SELLTAB_ELEMS;
		}
		
		if (!modeReady || SBDirty || clear)
			DrawScrollbar(REC_SELLBAR, TEL_SELLBAR, clear, UseBar);
		
		// Stopped mousing over something, hide tooltip
		if (MousedSell == -1 && MousedSellOld != -1)
			TElem::Tooltips::TT_Set();
		
		MousedSellOld = MousedSell;
		
		modeReady = !clear;
	}
	
	// Initialize starting variables
	void M_Init(bool clear)
	{
		ResetMine();
		
		if (clear)
			return;
			
		TabHalfHeight = Half(SellGun.H);
		
		DetectGuns();
		
		// Rectangles for some things
		SetRect(REC_SELLAREA, ScrollBack.W, 98.0, SellGun.W-PRICETAG, ResY - 98.0);
		SetRect(REC_SELLSCROLL, 0.0, REC_SELLAREA.Y, ScrollBack.W, REC_SELLAREA.H);
		
		SetScrollbar(REC_SELLBAR, REC_SELLSCROLL);
		
		// Calculate how many guns we can hold based on our resolution
		maxGuns = ShaveInt(FixedDiv(REC_SELLAREA.H, (SellGun.H + TEL_PADDING)));
	}
	
	// Called every tic
	void M_Tic()
	{
		if (!modeReady) { M_Init(false); }
		M_Draw(false);
	}
	
	// Cleaned up
	void M_Cleanup()
	{
		modeReady = false;
		M_Init(true);
		M_Draw(true);
	}
	
	// Release!
	void M_MouseRelease() {}
	
	// Click!
	void M_MouseClick()
	{
		if (MousedSell == -1 || TraderState == TS_UPGRADES)
			return;
			
		LocalAmbientSound("trader/wepclick", 127);
			
		if (WeaponList[ myGuns[MousedSell] ].undroppable)
		{
			TraderLog("\cKYou cannot sell that item.");
			return;
		}
			
		Handshaking = true;
		HandHash = GetGunHash(MousedSell);
		NamedRequestScriptPuke("Guns_Sell", HandHash);
	}
	
	// Right clicked!
	void M_MouseRightClick()
	{
		if (MousedSell == -1)
			return;
			
		// Only if we're NOT in upgrade mode
		if (TraderState == TS_UPGRADES)
			return;
			
		// Can this weapon even be upgraded?
		int WI = myGuns[MousedSell];
		if (WeaponList[WI].UpgradeCount <= 0)
			return;
			
		LocalAmbientSound("trader/upgradeswap", 127);
		modeReady = false;
		M_Draw(false);
		UpgradeGun = myGuns[MousedSell];
		SwapTraderState(TS_UPGRADES, true);
	}
	
	// Executed successfully
	void M_Handshook()
	{
		int WI = WeaponByHash(HandHash);
		LocalAmbientSound("trader/wepsell", 127);
		TraderLog(StrParam(s:"\cDYou sold your \cJ", s:WeaponList[WI].DisplayName, s:"\cD."));
		DetectGuns(true);
		TElem::ModeGuns::DetectBuyable(true);
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - 
	
	void DetectGuns(bool reset = false)
	{
		if (reset)
		{
			ResetMine();
			modeReady = false;
		}
			
		// Loop through all weapons and find those that match our category
		for (int l=0; l<WeaponCount; l++)
		{
			if (CheckInventory(WeaponList[l].Class))
				AddSellGun(l);
		}
	}
	
	void AddSellGun(int l)
	{
		myGuns[myCount] = l;
		myCount ++;
	}
	
	str GetGunName(int ind) { return WeaponList[ myGuns[ind] ].DisplayName; }
	
	// Gun hash, important for sync
	int GetGunHash(int ind)
	{
		return WeaponList[ myGuns[ind] ].Hash;
	}
	
	// FINAL PRICE of this index, filters in discounts and things
	int GetGunPrice(int ind)
	{
		return FinalSellPrice(myGuns[ind]);
	}
	
	// Get Y position of a certain gun button
	int SellTabY(int num)
	{
		return REC_SELLAREA.Y + (num * (SellGun.H + TEL_PADDING));
	}
	
	// Draw the actual gun tab
	void DrawSellTab(int l, int Y, int HID, bool clear)
	{
		int RealNum = myGuns[l];
		int TabHalf = Y + TabHalfHeight;
		
		// Gun Tab
		str MainTex = StrEval(MousedSell == l, SellGun.Tex, SellGunOver.Tex);
		if (TraderState == TS_UPGRADES)
		{
			if (UpgradeGun == myGuns[l])
				MainTex = "TRADSLI4";
			else
				MainTex = "TRADSLI3";
		}
			
		ZTImgMessage(MainTex, REC_SELLAREA.X, Y, HID, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
		
		// Gun Name
		SetFont("TFNTAB");
		int PNX = REC_SELLAREA.X + 12.0;
		str col = "\cJ";
		if (TraderState == TS_UPGRADES && UpgradeGun != myGuns[l])
			col = "\cM";
		
		ZTHudMessage(PNX, TabHalf, HID-1, StrParam(s:col, s:GetGunName(l)), ALIGN_LEFT, ALIGN_CENTER, FOREVER);
		
		// Gun PRICE
		bool Sellable = !WeaponList[ myGuns[l] ].undroppable;
		SetFont("TFNPRIC");
		int PriceX = (REC_SELLAREA.X + REC_SELLAREA.W) + 8.0;
		str PriceString = StrEval(Sellable, "\cM---", StrParam(s:col, s:"$", d:GetGunPrice(l)));
		
		if (TraderState == TS_UPGRADES)
			PriceString = "";
		
		ZTHudMessage(PriceX, TabHalf, HID-2, StrEval(clear, PriceString, ""), ALIGN_LEFT, ALIGN_CENTER, FOREVER);
		
		// Can it be upgraded?
		str UpText = StrEval(WeaponList[ myGuns[l] ].UpgradeCount > 0, "", "a");
		UpText = StrEval(TraderState == TS_UPGRADES, UpText, "");
		
		ZTImgMessage("TRADUPGR", REC_SELLAREA.X + SellGun.W + 4.0, Y, HID-3, StrEval(clear, UpText, ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
	}
	
	// Actually sell a gun!
	script "Guns_Sell" (int gunHash) NET
	{
		int WI = WeaponByHash(gunHash);
		if (WI == -1)
			terminate;
			
		// Do we have enough money?
		int price = FinalSellPrice(WI);
		
		if (!CheckInventory(WeaponList[WI].Class))
			terminate;
			
		GiveInventory("Money", price);
		TakeInventory(WeaponList[WI].Class, 1);
			
		DoHandshake(HS_SELLGUN);
	}
}
