// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// T O O L T I P
// Controls various tooltip functions for the trader
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

#define				TTM_WEAPON					0
#define				TTM_CENTER					1
#define				TTM_PERK					2
#define				TTM_ARMOR					3
#define				TTM_AMMO					4
#define				TTM_UPGRADE					5	
#define				TT_ARMOR					0
#define				TT_AMMO						1

#define				TT_HEIGHT					256.0

#define				TEL_TOOLTIP					1000
#define				TEL_TOOLTIPINFO				900

#define WRAP_WIDTH 			300
#define WRAP_GLYPHSIZE		6

namespace TElem::Tooltips
{
	using TraderUI : UpgradeGun;
	
	private bool TTDirty;
	private bool ShowTooltip;
	private int TTMode = -1;
	private int TTArg = -1;
	private int TTPrice = 0;
	private bool TTSelling;
	
	// Used for guns
	private str WrappedText = "";
	private int WrappedLines = 0;
	
	// This is the REGION for the tooltip
	private struct Rect				REC_TOOLTIP;
	
	// Stores how many HIDs our LAST tooltip segment used
	private int StoredCounter = 9999;
	
	// Store the height of the box we're drawing
	private int StoredHeight = 0.0;
	
	void TT_Draw()
	{
		// Not showing the tooltip and also not dirty
		if (!ShowTooltip && !TTDirty)
			return;
			
		int MX = GetMouseX();
		int MY = GetMouseY();
		int TTX = MX - (REC_TOOLTIP.W + 24.0);
		int TTY = MY;
		
		if (TTMode == TTM_AMMO || (MX <= CenX && TTMode != TTM_ARMOR))
			TTX = MX + 32.0;
		
		StoredHeight = TT_GetHeight();
		
		// Would this exceed our screen height?
		// If so, clamp it to the bottom of the screen
		if (TTY + StoredHeight >= ResY)
		{
			TTY = ResY - StoredHeight;
		}
		
		int counter = ZTHudSegment(TTX, TTY, StoredHeight, TEL_TOOLTIP, TooltipTop, TooltipMid, TooltipLow, FOREVER, !ShowTooltip);
			
		// If our segment SHRINKS then we'll have segment bits which
		// aren't accounted for
		// This removes them and keeps them from lingering on the screen
		if (counter > StoredCounter)
		{
			for (int l=counter; l>=StoredCounter; l--)
				BlankMessage(l);
		}
		
		// 2 AM and I'm fixing bugs, we'll just blank out some extra messages
		// Should fix upgrade -> perk lingering message bug
		for (int m=0; m<6; m++)
			BlankMessage(TEL_TOOLTIPINFO-m);
			
		// Which mode are we in?
		switch (TTMode)
		{
			// Perk mode
			case TTM_PERK:
				TTDraw_Perk(TEL_TOOLTIPINFO, TTX, TTY, !ShowTooltip);
			break;
			
			// Gun mode
			case TTM_WEAPON:
				TTDraw_Gun(TEL_TOOLTIPINFO, TTX, TTY, !ShowTooltip);
			break;
			
			case TTM_ARMOR:
			case TTM_AMMO:
				TTDraw_ArmorAmmo(TEL_TOOLTIPINFO, TTX, TTY, !ShowTooltip);
			break;
			
			case TTM_UPGRADE:
				TTDraw_Upgrade(TEL_TOOLTIPINFO, TTX, TTY, !ShowTooltip);
			break;
		}
			
		StoredCounter = counter;
		TTDirty = false;
	}
	
	void TT_Init()
	{
		SetRect(REC_TOOLTIP, 0.0, 0.0, TooltipTop.W, TT_HEIGHT);
	}
	
	void TT_Cleanup(int TempMode = 0)
	{
		TTMode = TempMode;
		
		ShowTooltip = false;
		TTDirty = true;
		TT_Draw();
		
		TTMode = -1;
		TTArg = -1;
		TTPrice = 0;
	}
	
	void TT_Set(int NewMode = -1, int NewArg = 0, int PriceArg = 0, bool selling = false)
	{
		// Reset
		if (NewMode == -1)
		{
			if (ShowTooltip)
			{
				ShowTooltip = false;
				TTDirty = true;
				
				TT_Draw();
				
				WrappedText = "";
				TTPrice = -1;
				TTMode = -1;
				TTArg = -1;
			}
			return;
		}
		
		if (TTMode == NewMode && TTArg == NewArg)
			return;
			
		TTSelling = selling;
		TTPrice = PriceArg;
		TTMode = NewMode;
		TTArg = NewArg;
		
		// Mouse over a weapon
		if (TTMode == TTM_WEAPON)
		{
			WrappedText = WrapUp(TTArg);
			LocalAmbientSound("trader/wepover", 127);
		}
		else if (TTMode == TTM_UPGRADE)
		{
			WrappedText = WrapUp(TTArg, true);
			LocalAmbientSound("trader/wepover", 127);
		}
		else if (TTMode == TTM_PERK)
			LocalAmbientSound("trader/perkover", 127);
		
		ShowTooltip = true;
		TTDirty = true;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - 
	
	// Auto-wrap and add some extra things
	str WrapUp(int WI, bool up = false)
	{
		str Chunk = StrEval(up, WeaponList[TTArg].Description, WeaponList[UpgradeGun].Upgrades[TTArg].Description);
		bool StoredDash;
		int LN = StrLen(Chunk);
		int LineWidth = 0;
		str Wrapped = "";
		WrappedLines = 1;
		StoredDash = false;
		
		for (int l=0; l<LN; l++)
		{
			str CHR = StrMid(Chunk, l, 1);
			if (CHR == "\t")
				continue;
				
			bool NL = (CHR == "\n");
			
			if (NL || (LineWidth >= WRAP_WIDTH && CHR == " "))
			{
				LineWidth = 0;
				WrappedLines ++;
				StoredDash = false;
				Wrapped = StrParam(s:Wrapped, s:"\n");
			}
			
			if (!NL)
			{
				if (LineWidth == 0 && CHR == "-")
					StoredDash = true;
					
				Wrapped = StrParam(s:Wrapped, s:CHR);
				LineWidth += WRAP_GLYPHSIZE;
			}
		}
		
		int extraLines = 0;
		str ExtraJunk = "";
			
		if (!up && StrLen(WeaponList[WI].fireModeItem) > 0)
		{
			if (extraLines == 0)
			{
				// Add double spaces ONLY if our last line didn't start with a -
				ExtraJunk = "";
				if (!StoredDash)
				{
					ExtraJunk = "\n";
					extraLines ++;
				}
			}
			
			ExtraJunk = StrParam(s:ExtraJunk, s:"\n- Alt-fire to toggle between fire modes.");
			extraLines ++;
		}
		
		// Show upgrade help!
		if (!up && TTSelling && WeaponList[WI].UpgradeCount > 0)
		{
			ExtraJunk = StrParam(s:ExtraJunk, s:"\n\n\cKPress \cF[Aim] \cKto upgrade this gun!");
			extraLines += 2;
		}
		
		WrappedLines += extraLines;
		return StrParam(s:Wrapped, s:ExtraJunk);
	}
	
	int TT_GetHeight()
	{
		int baseHeight;
		
		if (TTMode == TTM_PERK)
			return 230.0;
			
		if (TTMode == TTM_UPGRADE)
		{
			baseHeight = 340.0 + (WrappedLines * 16.0);
			
			return baseHeight;
		}
		
		if (TTMode == TTM_WEAPON)
		{
			baseHeight = 256.0 + (WrappedLines * 16.0);
			
			// 2 lines for our help info
			baseHeight += 32.0;
			
			return baseHeight;
		}
			
		return REC_TOOLTIP.H;
	}
	
	void TTDraw_Perk(int HID, int X, int Y, bool clear)
	{
		int TTCen = X + Half(REC_TOOLTIP.W);
		
		
		// -- NAME
		int NX = TTCen;
		int NY = Y + 142.0;
		SetFont("TFNNAME");
		ZTHudMessage(NX, NY, HID, StrEval(clear, ZT2Perks[TTArg].Name, ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
		
		// -- DESCRIPTION
		NY += 42.0;
		str PerkInfo = ZT2Perks[TTArg].Description;
		SetFont("TFNDESC");
		ZTHudMessage(NX, NY, HID, StrEval(clear, PerkInfo, ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
		
		// -- ICON
		NY = Y + 38.0;
		str PerkIcon = ZT2Perks[TTArg].IconLarge;
		ZTImgMessage(PerkIcon, NX, NY, HID, StrEval(clear, "a", ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
	}
	
	void TTDraw_Gun(int HID, int X, int Y, bool clear)
	{
		int TTCen = X + Half(REC_TOOLTIP.W);
		
		// -- NAME
		int NX = TTCen;
		int NY = (Y + 168.0) - 5.0;
		str WepName = WeaponList[TTArg].DisplayName;
		SetFont("TFNNAME");
		ZTHudMessage(NX, NY, HID, StrEval(clear, WepName, ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		HID --;
		
		// -- PICTURE
		NY = Y + 88.0;
		str WepPic = WeaponList[TTArg].Pic;
		ZTImgMessage(WepPic, NX, NY, HID, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
		HID --;
		
		// -- MONEY
		NY = Y + 190.0;
		str WepPrice = StrParam(s:"$", d:TTPrice);
		SetFont("TFNPRIC");
		ZTHudMessage(NX, NY, HID, StrEval(clear, WepPrice, ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		HID --;
		
		// -- DESCRIPTION
		NY = Y + 220.0;
		str WepInfo = WrappedText;
		
		SetFont("TFNDESC");
		ZTHudMessage(NX, NY, HID, StrEval(clear, WepInfo, ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
		
		// -- HELPFUL INFO
		NY = (Y +StoredHeight) - 16.0;
		str HelpString = StrParam(s:"\cUMagazine Capacity: ", d:WeaponList[TTArg].ammoCapA, s:"\n");
		
		// Headshot damage is over 100, add some orange
		HelpString = StrParam(s:HelpString, s:StrEval(WeaponList[TTArg].headshotMult > 1.0, "\cU", "\cK"));
		
		int FinalMult = ShaveInt(FixedMul(WeaponList[TTArg].headshotMult, 100.0));
		HelpString = StrParam(s:HelpString, s:"Headshot Damage: ", d:FinalMult, s:"%"); 

		ZTHudMessage(NX, NY, HID, StrEval(clear, HelpString, ""), ALIGN_CENTER, ALIGN_BOTTOM, FOREVER);
		HID --;
	}
	
	void TTDraw_ArmorAmmo(int HID, int X, int Y, bool clear)
	{
		int TTCen = X + Half(REC_TOOLTIP.W);
		
		str APic = StrEval(TTMode == TTM_ARMOR, "ICONAMMO", "ICONARMR");
		
		// Price
		// TODO: CALCULATE THIS WHEN THE TRADER MENU OPENS, NOT HOVER
		int PRC = StrEval(TTMode == TTM_ARMOR, GetRefillCost(), GetArmorCost());
		str col = StrEval(CheckInventory("Money") >= PRC, "\cU", "\cJ");

		// Pic
		int NX = TTCen;
		int NY = Y + 80.0;
		ZTImgMessage(APic, NX, NY - 4.0, HID, StrEval(clear, "a", ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		HID --;
		
		// Name
		NY += 60.0;
		str NM = StrEval(TTMode == TTM_ARMOR, "Refill Ammo", "Buy Armor");
		SetFont("TFNNAME");
		ZTHudMessage(NX, NY, HID, StrEval(clear, StrParam(s:col, s:NM), ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		HID --;
		
		// Draw price
		NY += 25.0;
		SetFont("TFNPRIC");
		ZTHudMessage(NX, NY, HID, StrEval(clear, StrParam(s:col, s:"$", d:PRC), ""), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
		HID --;
		
		// Description
		NY += 30.0;
		str DSC = StrEval(TTMode == TTM_ARMOR, "Lock and load, refill and prepare for another\nround of heart-stopping mayhem!", "Zip up that combat vest, you'll need it!");
		
		SetFont("TFNDESC");
		ZTHudMessage(NX, NY, HID, StrEval(clear, DSC, ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
	}
	
	void TTDraw_Upgrade(int HID, int X, int Y, bool clear)
	{
		int TTCen = X + Half(REC_TOOLTIP.W);
		int NX = TTCen;
		
		// -- PICTURE
		int NY = Y + 12.0;
		ZTImgMessage(WeaponList[UpgradeGun].Upgrades[TTArg].Pic, NX, NY + 12.0, HID, StrEval(clear, "a", ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
		
		// -- BORDER
		NY = Y + 12.0 + 222.0;
		ZTImgMessage("TRADBORD", NX, NY + 12.0, HID, StrEval(clear, "a", ""), ALIGN_CENTER, ALIGN_BOTTOM, FOREVER);
		HID --;
		
		// -- NAME
		NY = Y + 12.0 + 238.0;
		SetFont("TFNNAME");
		ZTHudMessage(NX, NY, HID, StrEval(clear, WeaponList[UpgradeGun].Upgrades[TTArg].Name, ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
		
		// -- PRICE
		NY += 35.0;
		SetFont("TFNPRIC");
		ZTHudMessage(NX, NY, HID, StrEval(clear, StrParam(s:"$", d:WeaponList[UpgradeGun].Upgrades[TTArg].Price), ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
		
		// -- DESCRIPTION
		NY += 30.0;
		SetFont("TFNDESC");
		ZTHudMessage(NX, NY, HID, StrEval(clear, WrappedText, ""), ALIGN_CENTER, ALIGN_TOP, FOREVER);
		HID --;
	}
}
