//--------------------------------------------------
//
// A M M O   R E L A T E D   F U N C T I O N S
//
//--------------------------------------------------

#define MAX_BONUSES				32
#define MAX_MAGAZINES			99

// Perk gets a bonus for a specific ammo type
struct AmmoBonus {
	str Perk;
	int Amount, PerkedAmount;
};

// Contains ammo info about a certain weapon
struct WepAmmo {
	str AmmoClass, ReserveClass;
	int MagSize;
	int bonuses_c;
	struct AmmoBonus bonuses[MAX_BONUSES];
};

// These are all of the stored magazines
// Hacky dynamic array
struct StoredMags {
	int count;
	struct WepAmmo ammos[MAX_MAGAZINES];
};
struct StoredMags magList;

// - - - - - - - - - - - - -

// Create a new magazine type
void NewMagazine(str class, str resclass, int size)
{
	magList.ammos[magList.count].AmmoClass = class;
	magList.ammos[magList.count].ReserveClass = resclass;
	magList.ammos[magList.count].MagSize = size;
	magList.count ++;
};

// Add a perk bonus to a magazine type
void MagBonus(str perkclass, str mag, int count, int bigcount)
{
	// Search all magazines to find the one that matches this class
	for (int l=0; l<magList.count; l++)
	{
		if (!StrICmp(magList.ammos[l].AmmoClass, mag))
		{
			int b = magList.ammos[l].bonuses_c;
			magList.ammos[l].bonuses[b].Perk = perkclass;
			magList.ammos[l].bonuses[b].Amount = count;
			magList.ammos[l].bonuses[b].PerkedAmount = bigcount;
			magList.ammos[l].bonuses_c ++;
		}
	}
}

// Find the magazine INDEX for a certain type
int MagIndex(str type)
{
	// Find the magazine that matches this class
	for (int l=0; l<magList.count; l++)
	{
		if (!StrICmp(magList.ammos[l].AmmoClass, type))
			return l;
	}
	
	return -1;
}

// Find the maximum magazine capacity for an ammo type
int MagCapacity(str mag)
{
	int MI, goal;

	MI = MagIndex(mag);
	if (MI == -1)
		return 0;
		
	goal = magList.ammos[MI].MagSize;
	
	// Check for perk bonuses
	for (int l=0; l<magList.ammos[MI].bonuses_c; l++)
	{
		if (CheckInventory(magList.ammos[MI].bonuses[l].Perk))
		{
			// Level 10 or above, use EXTENDED magazine
			if (CheckInventory("PerkLevel") >= 10)
				goal = magList.ammos[MI].bonuses[l].PerkedAmount;
			else
				goal = magList.ammos[MI].bonuses[l].Amount;
		}
	}
	
	return goal;
}

// Perform actual ammo calculation
void ReloadMagic(str magazine, str reserve, int shovel)
{
	int MC = MagCapacity(magazine);
	if (MC <= 0)
		return;
		
	int count;
	
	while (CheckInventory(reserve) && CheckInventory(magazine) < MC)
	{
		GiveInventory(magazine, 1);
		TakeInventory(reserve, 1);
		
		count ++;
		
		if (shovel)
			break;
	}
	
	// PrintBold(s:"You gave ", d:count, s:" ammo");
}

// Called externally, grabs the weapon you're holding and controls ammo
script "ReloadAmmo" (int shovel)
{
	str wep = GetWeapon();
	int WI = WeaponByType(wep);
	
	if (WI == -1)
		terminate;
	
	str RA = GetReserveClass(WI);
		
	ReloadMagic(WeaponList[WI].ammoClassA, RA, shovel);
}

// Called externally
// Are we allowed to reload this weapon?
script "ReloadAllowed" (void)
{
	str wep = GetWeapon();
	int WI = WeaponByType(wep);
	
	if (WI == -1)
	{
		SetResultValue(0);
		terminate;
	}
		
	int AA = CheckInventory(WeaponList[WI].ammoClassA);
	int AB = CheckInventory( GetReserveClass(WI) );
	
	if (AB > 0 && AA < MagCapacity(WeaponList[WI].ammoClassA))
		SetResultValue(1);
	else
		SetResultValue(0);
}