//----------------------------------------------
// VERY USEFUL UTILITY FUNCTIONS
//----------------------------------------------

#define SCR_RESSENDSERVER 477

#define PLAYER_BASE_TID 1000
#define p_BaseTID 1000

// BENCHMARK
// This forces a script to run a ton of times, so we can ACSProfile it
#define BENCHMARK_ITERATIONS					20000
#define BENCHMARK_BATCHSIZE						1000
script "Benchmark" ()
{
	int iterations = 0;
	str ScriptName = GetCVarString("cl_zt2benchmark");
	
	if (StrLen(ScriptName) <= 0)
	{
		SetFont("TFNDESC");
		PrintBold(s:"\cJBenchmark script not set, use \cKcl_zt2benchmark \cJto set one.");
		terminate;
	}
	
	SetFont("TFNDESC");
	PrintBold(s:"\cJPerforming benchmark on \cK", s:ScriptName, s:"\cJ...");
	
	while (iterations < BENCHMARK_ITERATIONS)
	{
		for (int l=0; l<BENCHMARK_BATCHSIZE; l++)
		{
			ACS_NamedExecuteAlways(ScriptName, 0);
			iterations ++;
		}
		
		delay(1);
	}
	
	SetFont("TFNDESC");
	PrintBold(s:"Script ", s:ScriptName, s:" run for ", d:BENCHMARK_ITERATIONS, s:" iterations.\nUse the ACSProfile command to check results.");
}

bool SandboxMode() { return map_sandbox; }

#define MAPNOT_Y			0.650

// We're alive
bool Living() { return (ClassifyActor(0) & ACTOR_ALIVE); }

// Ensure we're the proper client
bool WrongClient()
{
	return (PlayerNumber() != ConsolePlayerNumber());
}

int min (int a, int b)
{
	if (a < b)
		return a;

	return b;
}

int max (int a, int b)
{
	if (a > b)
		return a;

	return b;
}

// Create a "fake" hash
// This turns a string into a unique int based on characters
// Fake hashes should be the SAME on client and server
int FakeHash(str hasher)
{
	int hash = 0;
	for (int l=0; l<StrLen(hasher); l++)
		hash += GetChar(hasher, l) + (3*l);
		
	return hash;
}

// Half a fixed
int Half(int num)
{
	return FixedMul(num, 0.5);
}

bool Between(int num, int a, int b)
{
	return (num >= a && num <= b);
}

// Fullscreen HUD
void FullHUD()
{
	int ResX = ScreenWidth();
	int ResY = ScreenHeight();
	
	SetHudSize(ResX, ResY, false);
}

void RestoreHUD()
{
	SetHudSize(0, 0, true);
}

// Freeze / Unfreeze
void FreezeMe(bool frozen, int num = 0)
{
	SetActorProperty(num,APROP_Invulnerable,frozen);
	SetPlayerProperty(num,frozen,4);
	SetPlayerProperty(num,frozen,1);
	SetPlayerProperty(num,frozen,7);
}

// Server-sided
bool IsServer()
{
	if (ConsolePlayerNumber() < 0 || PlayerNumber() < 0)
		return true;
		
	return !IsClient();
}

// Client-side
bool IsClient()
{
	return IsNetworkGame() && ConsolePlayerNumber() >= 0 && !SinglePlayer();
}

// Client-side ONLY
// Used primarily for resolution
bool IsClientOnly()
{
	return ConsolePlayerNumber() >= 0 || SinglePlayer();
}

script "IsClient" ()
{
	SetResultValue( IsClient() );
}

str StrPad(str string, str padder, int len)
{
	while (StrLen(string) < len)
		string = StrParam(s:padder, s:string);
		
	return string;
}

// Return a padded time string from a number
str TimeString(int sec)
{
	// Split the trader text into minutes
	int minutes = ShaveInt(FixedDiv(sec*1.0,60.0));
	str minuteText = StrPad( StrParam(d:minutes), "0", 2 );
		
	// Split the trader text into seconds
	int seconds = sec - (60*minutes);
	str secondText = StrPad( StrParam(d:seconds), "0", 2 );

	return StrParam(s:minuteText,s:":",s:secondText);
}

// Something opened on the map!
void MapNotification(str message)
{
    SetFont("BIGFONT");
	
	AmbientSound("map/notify", 127);
	
	// Plain message
    HudMessageBold(s:"\cD",s:message;HUDMSG_FADEOUT,HID_MAPEVENT,CR_WHITE,0.5,MAPNOT_Y,2.0,1.0);
    HudMessageBold(s:"\cJ",s:message;HUDMSG_FADEOUT,HID_MAPEVENT-1,CR_WHITE,0.5,MAPNOT_Y,0.05,0.25);
}

// Teleport fog has a target when someone ACTUALLY teleports
// If they join the game, they won't
script "TeleportCheck" (void)
{
	int x = GetActorX(0);
	int y = GetActorY(0);
	int z = GetActorZ(0);

	if (SetActivatorToTarget(0))
		Spawn("TeleportSparker", x, y, z, 0, 0);
}

// Evaluate a string
str StrEval(int cond, str a, str b)
{
	if (cond)
		return b;

	return a;
}

// Ensure that we have X amount of an inventory item
void InvEnsure(str item, int amt)
{
	int cnt = CheckInventory(item);
	if (amt > cnt)
		GiveInventory(item, amt-cnt);
	else if (cnt > amt)
		TakeInventory(item, cnt - amt);
}

int PlayerTID(int num) { return PLAYER_BASE_TID + num; }

int Shave(int num)
{
	return (num>>16)<<16;
}

int ShaveInt(int num)
{
	return (num>>16);
}

int clamp(int a, int mn, int mx)
{
	if (a >= mx)
		return mx;
	else if (a <= mn)
		return mn;
	else
		return a;
}

// Does this player exist?
int PlayerExists(int TID)
{
	if (ThingCountName("ZTPlayer", TID) > 0)
		return true;
		
	return false;
}

// Blank out a HUD ID
void BlankMessage(int MID)
{
	HudMessage(s:""; HUDMSG_PLAIN, MID, CR_WHITE, 0.0, 0.0, 1.0);
}

// Distance betwene a point and a TID
int distancePoint (int XV, int YV, int ZV, int tid2)
{
	int x, y, z, d;
	x = XV - GetActorX(tid2) >> 16; // Convert fixed point to integer
	y = YV - GetActorY(tid2) >> 16;
	z = ZV - GetActorZ(tid2) >> 16;
	d = sqrt( x*x + y*y + z*z );
	return d;
}

// Distance between two TIDs
function int distance (int tid1, int tid2)
{
	int x, y, z, d;
	x = GetActorX(tid1) - GetActorX(tid2) >> 16; // Convert fixed point to integer
	y = GetActorY(tid1) - GetActorY(tid2) >> 16;
	z = GetActorZ(tid1) - GetActorZ(tid2) >> 16;
	d = sqrt( x*x + y*y + z*z );
	return d;
}

int abs(int x)
{
    if (x < 0)
        return -x;

    return x;
}

// Shows a message on the screen without being scaled
void HudMessageUnscaled (int xoffset, int yoffset, str text, int alignx, int aligny, int id)
{
	int vidres_x = GetCVar("vid_defwidth");
	int vidres_y = GetCVar("vid_defheight");
	int vidres_xFixed = vidres_x * 1.0;
	int vidres_yFixed = vidres_y * 1.0;
	int vidres_xCenter = FixedDiv(vidres_xFixed, 2.0);
	int vidres_yCenter = FixedDiv(vidres_yFixed, 2.0);
	int alignvar_x;
	int alignvar_y;

	int finaloffset_x;
	int finaloffset_y;

	if (yoffset < 0) { finaloffset_y = vidres_yFixed + yoffset; }
	else { finaloffset_y = yoffset; }

	if (xoffset < 0) { finaloffset_x = vidres_xFixed + xoffset; }
	else { finaloffset_x = xoffset; }

	switch (alignx)
	{
		case 0:
		alignvar_x = ALIGN_LEFT;
		break;

		case 1:
		alignvar_x = ALIGN_CENTER;
		break;

		case 2:
		alignvar_x = ALIGN_RIGHT;
		break;
	}

	switch (aligny)
	{
		case 0:
		alignvar_y = ALIGN_LEFT;
		break;

		case 1:
		alignvar_y = ALIGN_CENTER_Y;
		break;

		case 2:
		alignvar_y = ALIGN_RIGHT;
		break;
	}

	SetHUDSize(vidres_x, vidres_y, false);
	HudMessage(s:text; HUDMSG_PLAIN | HUDMSG_FADEOUT, 725+id, CR_WHITE, finaloffset_x + alignvar_x, finaloffset_y + alignvar_y, 2.0);
}

// Shows a message on the screen without being scaled
void HudMessageUnscaledSlot (int xoffset, int yoffset, str text, int alignx, int aligny, int id, int showtime)
{
	int vidres_x = GetCVar("vid_defwidth");
	int vidres_y = GetCVar("vid_defheight");
	int vidres_xFixed = vidres_x * 1.0;
	int vidres_yFixed = vidres_y * 1.0;
	int vidres_xCenter = FixedDiv(vidres_xFixed, 2.0);
	int vidres_yCenter = FixedDiv(vidres_yFixed, 2.0);
	int alignvar_x;
	int alignvar_y;

	int finaloffset_x;
	int finaloffset_y;

	if (yoffset < 0) { finaloffset_y = vidres_yFixed + yoffset; }
	else { finaloffset_y = yoffset; }

	if (xoffset < 0) { finaloffset_x = vidres_xFixed + xoffset; }
	else { finaloffset_x = xoffset; }

	switch (alignx)
	{
		case 0:
		alignvar_x = ALIGN_LEFT;
		break;

		case 1:
		alignvar_x = ALIGN_CENTER;
		break;

		case 2:
		alignvar_x = ALIGN_RIGHT;
		break;
	}

	switch (aligny)
	{
		case 0:
		alignvar_y = ALIGN_LEFT;
		break;

		case 1:
		alignvar_y = ALIGN_CENTER_Y;
		break;

		case 2:
		alignvar_y = ALIGN_RIGHT;
		break;
	}

	SetHUDSize(vidres_x, vidres_y, false);
	HudMessage(s:text; HUDMSG_PLAIN | HUDMSG_FADEOUT, id, CR_UNTRANSLATED, finaloffset_x + alignvar_x, finaloffset_y + alignvar_y, showtime, 0.5);
}

// -- REDUX OF THE PREVIOUS SCRIPTS --
void HMUSRedux (int xoffset, int yoffset, str text, int alignx, int aligny, int id, int showtime, int side)
{
	HMUSReduxExtended(xoffset, yoffset, text, alignx, aligny, id, showtime, side, 0.25, 0);
}

void HMUSReduxExtended (int xoffset, int yoffset, str text, int alignx, int aligny, int id, int showtime, int side, int fadeTime, int bold)
{
	int alignvar_x, alignvar_y;
	
	switch (alignx)
	{
		case 0:
		alignvar_x = ALIGN_LEFT;
		break;

		case 1:
		alignvar_x = ALIGN_CENTER;
		break;

		case 2:
		alignvar_x = ALIGN_RIGHT;
		break;
	}

	switch (aligny)
	{
		case 0:
		alignvar_y = ALIGN_LEFT;
		break;

		case 1:
		alignvar_y = ALIGN_CENTER_Y;
		break;

		case 2:
		alignvar_y = ALIGN_RIGHT;
		break;
	}
	
	ZTHudMessage(xoffset, yoffset, id, text, alignvar_x, alignvar_y, showtime, fadeTime);
}

// Check if a button was pressed ONCE, not held
bool ButtonPressed(int input, int BT = -1, int OBT = -1)
{
	if (BT == -1)
		BT = GetPlayerInput(-1, INPUT_BUTTONS);
	if (OBT == -1)
		OBT = GetPlayerInput(-1, INPUT_OLDBUTTONS);
		
	if( (BT & input) && !(OBT & input) )
		return true;
		
	return false;
}

// Check if a button was released
bool ButtonReleased(int input, int BT = -1, int OBT = -1)
{
	if (BT == -1)
		BT = GetPlayerInput(-1, INPUT_BUTTONS);
	if (OBT == -1)
		OBT = GetPlayerInput(-1, INPUT_OLDBUTTONS);
		
	if( !(BT & input) && (OBT & input) )
		return true;
		
	return false;
}
