//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Z O M B I E   T O R R E N T   2   P E R K   M E N U
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

#define PM_SHOW				0
#define PM_CLOSE			1

script "ChangePerkKey" (void) NET
{
	if (CheckInventory("MenuActive"))
		terminate;
		
	ACS_NamedExecute("PerkMenuControl", 0, PM_SHOW);
}

void PerkMenuControl(int mode) 
{ 
	if (GameType() == GAME_TITLE_MAP)
		return;
		
	ACS_NamedExecuteAlways("PerkMenuControl",0,mode); 
}

script "PerkMenuControl" (int mode)
{
	switch (mode)
	{
		case PM_SHOW:
		
			// Bots IMMEDIATELY select a perk
			if (PlayerIsBot(PlayerNumber()))
			{
				SelectPerk(0);
				terminate;
			}
		
			FreezeMe(true);
			InvEnsure("MenuActive", 1);
			InvEnsure("TraderDarken", 1);
			InvEnsure("ShowPerkMenu", 1);
			InvEnsure("MenuWeapon", 1);
			delay(1);
			SetWeapon("MenuWeapon");
			ACS_NamedExecuteAlways("PerkSelection",0);
		break;
		
		case PM_CLOSE:
			FreezeMe(false);
			InvEnsure("MenuActive", 0);
			InvEnsure("TraderDarken", 0);
			InvEnsure("ShowPerkMenu", 0);
			InvEnsure("MenuWeapon", 0);
		break;
	}
}

// - - - - - - - - - - - - - - - - - 

#define PS_PADDING				6.0
#define BONUS_DURATION			1

enum BonusState
{
	BS_10,
	BS_25,
};

namespace PerkSelectMenu
{
	using PerkSelectPics;
	
	// Position for certain things
	private int ResX, ResY, CenX, CenY, MenuX, MenuY;
	private int HoveredPerkOld, HoveredPerk;
	private int DirtyMenu;
	
	// Which perk are we?
	private int OurPerk;
	
	// For showing bonuses
	private str BonusString;
	private int BonusState;
	private int BonusTimer;
	
	// Rectangle for the list
	private struct Rect				REC_PERKLIST, REC_PERKMAIN, REC_PERKINFO, REC_PERKBONUS;
	
	script "PerkSelection" (void) CLIENTSIDE
	{
		// Not the right client who opened the menu, destroy
		if (WrongClient())
			terminate;
			
		OpenPerkMenu();
		
		// Show it as long as we have the item
		while (CheckInventory("ShowPerkMenu") > 0 && (ClassifyActor(0) & ACTOR_ALIVE))
		{
			UpdateCursor();
			DrawCursor("TRADCURS");
			
			// Before we DRAW the menu, let's control our mouse
			ControlPerkMouse();
			
			// Now draw it
			ControlPerkMenu();
			delay(1);
		}
		
		ClosePerkMenu();
	}

	// - - - - - - - - - - - - - - - - - 
	
	// Menu was opened
	void OpenPerkMenu()
	{
		LocalAmbientSound("perkselect/open", 127);
		
		OurPerk = FindPerkIndex();
		
		ResX = ScreenWidth() * 1.0;
		ResY = ScreenHeight() * 1.0;
		CenX = Half(ResX);
		CenY = Half(ResY);
		
		// Figure out the starting positions for our actual menu
		MenuX = CenX - Half(PerkSelectBG.W);
		MenuY = CenY - Half(PerkSelectBG.H);
		
		// Actual list that contains our perks
		SetRect(REC_PERKLIST, MenuX+5.0, MenuY+36.0, 207.0, 427.0);
		
		// Main area for our perk information
		SetRect(REC_PERKMAIN, MenuX+216.0, MenuY+36.0, 307.0, 164.0);
		
		// Perk description area
		SetRect(REC_PERKINFO, MenuX+216.0, MenuY+203.0, 307.0, 46.0);
		
		// Perk bonus area
		SetRect(REC_PERKBONUS, MenuX+216.0, MenuY+250.0, 307.0, 207.0);
		
		// Show a white fadeout in front of it
		ZTImgMessage("PSELFG", MenuX, MenuY, PEL_FOREGROUND, "a", ALIGN_LEFT, ALIGN_TOP, 0.05, 0.50);
		
		// Which item we're hovering over
		HoveredPerkOld = -1;
		HoveredPerk = -1;
		
		// Instantly update the menu
		DirtyMenu = true;
	}
	
	// Menu was closed, cleanup
	void ClosePerkMenu()
	{
		ControlPerkMenu(true);
		DrawCursor("");
	}

	// List item was moused over, do something!
	void PerkItemHover(int index)
	{
		HoveredPerkOld = index;
		
		// Hovered over an ACTUAL item
		if (index >= 0)
		{
			LocalAmbientSound("trader/wepover", 127);
			BonusState = BS_10;
			BonusTimer = 0;
			CompileBonuses();
		}
	}
	
	// Get the list Y for an item index
	int PerkListY(int ind)
	{
		return REC_PERKLIST.Y + ((PSItem.H + PS_PADDING) * ind);
	}
	
	// Constantly update the perk menu
	void ControlPerkMenu(bool clear = false)
	{
		int HID = PEL_LIST;
		bool listDirty = false;
		
		// Draw the background?
		if (DirtyMenu || clear)
			ZTImgMessage(PerkSelectBG.Tex, MenuX, MenuY, PEL_BACKGROUND, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
		
		HoveredPerk = -1;
		// Decide which perk we're hovering over before we do anything else
		for (int l=0; l<ZT2PerkCount; l++)
		{
			int PY = PerkListY(l);
			if (MouseInRect(REC_PERKLIST.X, PY, REC_PERKLIST.X+REC_PERKLIST.W, PY+PSItem.H))
				HoveredPerk = l;
		}
		
		// Control timer after we've determined what we're hovering over
		ControlBonusTimer();
		
		// We moused over a BRAND NEW perk
		if (HoveredPerkOld != HoveredPerk)
		{
			listDirty = true;
			PerkItemHover(HoveredPerk);
		}
			
		// Loop through all perks and actually DRAW THEM
		for (l=0; l<ZT2PerkCount; l++)
		{
			int X = REC_PERKLIST.X;
			int Y = PerkListY(l);
			
			// Start drawing if our hover state changed, or the entire menu is dirty
			if (listDirty || DirtyMenu || clear)
			{
				// Background first
				str BG = StrEval(OurPerk == l, StrEval(HoveredPerk == l, PSItem.Tex, PSItemOver.Tex), StrEval(HoveredPerk == l, PSOItem.Tex, PSOItemOver.Tex));
				ZTImgMessage(BG, X, Y, HID, StrEval(clear, "a", ""), ALIGN_LEFT, ALIGN_TOP, FOREVER);
				HID --;
				
				// Perk level next!
				int PLev = CheckInventory("PerkLevel");
				
				if (l != OurPerk)
					PLev = CheckInventory(ZT2Perks[l].ItemLevel);
				
				int ListHalfY = Y + Half(PSItem.H);
				SetFont("TFNPRIC");
				ZTHudMessage(X + 8.0, ListHalfY, HID, StrEval(clear, StrParam(d:PLev), ""), ALIGN_LEFT, ALIGN_CENTER, FOREVER);
				HID --;
				
				// Perk name!
				SetFont("TFNTAB");
				ZTHudMessage((X+PSItem.W) - 8.0, ListHalfY, HID, StrEval(clear, ZT2Perks[l].Name, ""), ALIGN_RIGHT, ALIGN_CENTER, FOREVER);
				HID --;
				
				// Are we mousing over this item?
				// Show some particular on-screen magic
				if (HoveredPerk == l && !clear)
				{
					int TempX, TempY;
					
					// Icon
					TempX = REC_PERKMAIN.X + Half(REC_PERKMAIN.W);
					TempY = REC_PERKMAIN.Y + 55.0;
					
					ZTImgMessage(ZT2Perks[l].IconLarge, TempX, TempY, PEL_INFO_ICON, "a", ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					
					// Name
					TempY += 65.0;
					SetFont("TFNNAME");
					ZTHudMessage(TempX, TempY, PEL_INFO_TITLE, ZT2Perks[l].Name, ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					
					// Level
					TempY += 26.0;
					SetFont("TFNPRIC");
					ZTHudMessage(TempX, TempY, PEL_INFO_LEVEL, StrParam(s:"Level ", d:PLev), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					
					// Description!
					TempY = REC_PERKINFO.Y + Half(REC_PERKINFO.H);
					SetFont("TFNDESC");
					ZTHudMessage(TempX, TempY, PEL_INFO_DESC, ZT2Perks[l].Description, ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					
					// Perk bonuses
					TempY = REC_PERKBONUS.Y + 14.0;
					SetFont("TFNPRIC");
					ZTHudMessage(TempX, TempY, PEL_INFO_BONUSTOP, "Perk Bonuses:", ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					
					// Actual perk bonuses
					TempY = REC_PERKBONUS.Y + 35.0;
					
					SetFont("TFNDESC");
					ZTHudMessage(REC_PERKBONUS.X + 8.0, TempY, PEL_INFO_BONUSMID, BonusString, ALIGN_LEFT, ALIGN_TOP, FOREVER);
				}
			}
		}
		
		// We weren't hovering over anything, so just clear out the on-screen junk
		if (HoveredPerk == -1 || clear)
		{
			BlankMessage(PEL_INFO_ICON);
			BlankMessage(PEL_INFO_TITLE);
			BlankMessage(PEL_INFO_LEVEL);
			BlankMessage(PEL_INFO_DESC);
			BlankMessage(PEL_INFO_BONUSMID);
			BlankMessage(PEL_INFO_BONUSTOP);
		}
		
		DirtyMenu = false;
	}
	
	// Compile a list of bonuses for a certain perk
	void CompileBonuses()
	{
		BonusString = StrParam(s:ZT2Perks[HoveredPerk].Bonuses[0], s:"\n\n");
		
		switch (BonusState)
		{
			// Level 10
			case BS_10:
				BonusString = StrParam(s:BonusString, s:"\cILevel 10:\n", s:ZT2Perks[HoveredPerk].Bonuses[1]);
			break;
			
			// Level 25
			case BS_25:
				BonusString = StrParam(s:BonusString, s:"\cKLevel 25:\n", s:ZT2Perks[HoveredPerk].Bonuses[2]);
			break;
		}
	}
	
	// Timer for alternating bonuses
	void ControlBonusTimer()
	{
		if (HoveredPerk == -1)
			return;
			
		BonusTimer ++;
		if (BonusTimer >= 35*BONUS_DURATION)
		{
			BonusTimer = 0;
			
			if (BonusState == BS_10)
				BonusState = BS_25;
			else
				BonusState = BS_10;
				
			CompileBonuses();
				
			DirtyMenu = true;
		}
	}
	
	// Handle our mouse and mouse inputs
	void ControlPerkMouse()
	{
		if (ButtonPressed(BT_ATTACK) && HoveredPerk != -1)
			NamedRequestScriptPuke("PerkMenuSelect",HoveredPerk);
	}
	
	// CLIENT -> SERVER
	// ATTEMPT TO SELECT A PERK
	script "PerkMenuSelect" (int perkIndex) NET
	{
		PerkMenuControl(PM_CLOSE);
		
		int PI = FindPerkIndex();
		if (perkIndex == PI)
		{
			ACS_NamedExecuteAlways("PerkMenuHandshake", 0, 0, 1);
			terminate;
		}
		
		SelectPerk(perkIndex);
		ACS_NamedExecuteAlways("PerkMenuHandshake", 0, perkIndex);
	}
	
	// SERVER -> CLIENT
	// Let them know we successfully selected a perk!
	script "PerkMenuHandshake" (int perkIndex, int samePerk) CLIENTSIDE
	{
		if (WrongClient())
			terminate;
			
		// Selected the same perk
		if (samePerk)
			terminate;
			
		LocalAmbientSound("perkselect/swap",128);
		
		// Show the perk message on the screen
		int ResX = ScreenWidth() * 1.0;
		int ResY = ScreenHeight() * 1.0;
		int CenX = Half(ResX);
		
		int TempY = ResY - 180.0;
		int HID = HID_YOUARE;
		
		// How long should we initially show these for?
		int showTime = 1.0;
		int insTime = 2.0;
		int totalTime = showTime + insTime;
		
		// BLOOD!!!
		ZTImgMessage("PSBLOD", CenX, TempY+32.0, HID, "a", ALIGN_LEFT, ALIGN_TOP, totalTime, 1.0);
		HID --;
		
		// YOU ARE
		SetFont("TFNUR");
		ZTHudMessage(CenX, TempY, HID, "\cJYou are now a", ALIGN_CENTER, ALIGN_CENTER, totalTime, 1.0);
		HID --;
		ZTHudMessage(CenX, TempY, HID, "\cIYou are now a", ALIGN_CENTER, ALIGN_CENTER, 0.05, 0.5);
		HID --;
		
		// PERK NAME
		TempY += 32.0;
		SetFont("TFNUARE");
		ZTHudMessage(CenX, TempY, HID, StrParam(s:"\cJ", s:ZT2Perks[perkIndex].Name), ALIGN_CENTER, ALIGN_CENTER, totalTime, 1.0);
		HID --;
		ZTHudMessage(CenX, TempY, HID, StrParam(s:"\cI", s:ZT2Perks[perkIndex].Name), ALIGN_CENTER, ALIGN_CENTER, 0.05, 0.5);
		HID --;
		
		delay(ShaveInt(35*showTime));
		
		// INSTRUCTIONS
		TempY += 30.0;
		SetFont("TFNDESC");
		ZTHudMessage(CenX, TempY, HID, StrParam(s:"\cJ", s:ZT2Perks[perkIndex].YouAre), ALIGN_CENTER, ALIGN_CENTER, insTime, 1.0);
		HID --;
		ZTHudMessage(CenX, TempY, HID, StrParam(s:"\cI", s:ZT2Perks[perkIndex].YouAre), ALIGN_CENTER, ALIGN_CENTER, 0.05, 0.5);
		HID --;
	}
}