//-----------------------------------------------
//
// S U R V I V A L   S C R I P T
// Fakes survival, respawns players at end of wave
//
//-----------------------------------------------

// Has the game actually started yet?
enum GameState
{
	STATE_LOBBY,
	STATE_COUNTDOWN,
	STATE_WAVE,
	STATE_TRADER,
	STATE_DIED
};

int gameState;

// Can the game be joined?
bool GameJoinable()
{
	if (gameState == STATE_LOBBY || gameState == STATE_TRADER || gameState == STATE_COUNTDOWN)
		return true;
		
	return false;
}

// Stored perk information for certain players
struct StorageData
{
	str PerkClass;
	int Cash;
	int XP[PERK_COUNT];
	int XPGoal[PERK_COUNT];
	int XPLevel[PERK_COUNT];
	bool valid;
	bool joined;						// Whether or not the player has joined even ONCE
	bool pickedPerk;					// Whether we successfully picked our perk or not
	
	// Has this player set their INITIAL perk information?
	// This is the ammo capacity for XPGoal, etc.
	bool wasSetup;
};

struct StorageData StoredInfo[MAXPLAYERS];

// Store perks when we die
void StorePerks(void)
{
	int PN = PlayerNumber();
	StoredInfo[PN].valid = true;
	
	int PI = FindPerkIndex();
	StoredInfo[PN].PerkClass = ZT2Perks[PI].Item;
	
	// Save ALL of our perk data
	for (int l=0; l<ZT2PerkCount; l++)
	{
		StoredInfo[PN].XP[l] = CheckInventory(ZT2Perks[l].ItemXP);
		StoredInfo[PN].XPGoal[l] = CheckInventory(ZT2Perks[l].ItemXPGoal);
		StoredInfo[PN].XPLevel[l] = CheckInventory(ZT2Perks[l].ItemLevel);
	}
	
	StoredInfo[PN].XP[PI] = CheckInventory("PerkXP");
	StoredInfo[PN].XPGoal[PI] = CheckInventory("PerkXPGoal");
	StoredInfo[PN].XPLevel[PI] = CheckInventory("PerkLevel");
	StoredInfo[PN].Cash = CheckInventory("Money");
}

void RestorePerks(void)
{
	int PN = PlayerNumber();
	
	// Invalid data
	if (!StoredInfo[PN].valid)
		return;
		
	GiveInventory(StoredInfo[PN].PerkClass, 1);
	
	TakeInventory("Money", 9999);
	GiveInventory("Money", StoredInfo[PN].Cash);
	
	int PI = FindPerkIndex();
	
	for (int l=0; l<ZT2PerkCount; l++)
	{
		GiveInventory(ZT2Perks[l].ItemXP, StoredInfo[PN].XP[l]);
		GiveInventory(ZT2Perks[l].ItemXPGoal, StoredInfo[PN].XPGoal[l]);
		GiveInventory(ZT2Perks[l].ItemLevel, StoredInfo[PN].XPLevel[l]);
		
		if (l == PI)
		{
			GiveInventory("PerkXP", StoredInfo[PN].XP[l]);
			GiveInventory("PerkXPGoal", StoredInfo[PN].XPGoal[l]);
			GiveInventory("PerkLevel", StoredInfo[PN].XPLevel[l]);
		}
	}
	
	SetAmmoCapacity("PerkXP",CheckInventory("PerkXPGoal"));
}

void ClearPerks(int PN)
{
	StoredInfo[PN].PerkClass = "";
	StoredInfo[PN].pickedPerk = false;
	StoredInfo[PN].valid = false;
	StoredInfo[PN].joined = false;
	StoredInfo[PN].Cash = 0;
	
	for (int l=0; l<PERK_COUNT; l++)
	{
		StoredInfo[PN].XP[l] = 0;
		StoredInfo[PN].XPGoal[l] = 0;
		StoredInfo[PN].XPLevel[l] = 0;
	}
}

// - - - - - - - - - - - - - - - - -

// SERVER
// Voice should be setup on a delay
// ( HandleVoiceTimer also controls this )
script "FindMyVoice" (void)
{
	delay(35*1);
	FindMyVoice();
}

// Setup some required player things
void SetupPlayer()
{
	ACS_NamedExecuteAlways("ResSendClient",0,0,0,0);
	ACS_NamedExecuteAlways("HealthRegeneration",0,0,0,0);
	ACS_NamedExecuteAlways("LastWeaponHandler",0,0,0,0);
	
	// Reload keys
	ACS_NamedExecuteAlways("ZoomReloadKeys",0,0,0,0);
	ACS_NamedExecuteAlways("LandHack",0,0,0,0);
	
	// ACS ammo
	ACS_NamedExecuteAlways("AmmoNumbers",0,0,0,0);
	
	// Voice timer
	ACS_NamedExecuteAlways("HandleVoiceTimer",0,0,0,0);
	
	// Controls weapon wheel mouse movement
	ACS_NamedExecuteAlways("WheelClient",0,0,0,0);
	
	// Apply voice
	ACS_NamedExecuteAlways("FindMyVoice", 0);
	
	tradePlayers[PlayerNumber()] = 0;
	Thing_ChangeTID(0,p_BaseTID+PlayerNumber());
	TakeInventory("InTrader",1);
	// ACS_NamedExecuteAlways("HUDCircle",0,0,0,0); 		// HUD display code
	ACS_NamedExecuteAlways("HudController",0,0,0,0); 	// HUD display code
	ACS_ExecuteAlways(76,0,0,0,0); 						// Gives health charge every 10 tics
	ACS_ExecuteAlways(77,0,0,0,0); 						// Debug script for zombie pool
	
	ACS_ExecuteAlways(565,0,0,0,0);						// Low health script
	
	PreparePerkSystem();
	
	//--SHOW THE MAP INFORMATION------------------------------
	if (GameType () != GAME_TITLE_MAP && !map_sandbox)
		DisplayTrackInfo(-1, 0);
}

void GiveSpawnInventory()
{
	// Reset the player's inventory to keep them from keeping items throughout maps
	ClearInventory();
	GiveInventory("Machete",1);
	GiveInventory("Bare Hands",1);
	GiveInventory("OrderSecondary",50);
	GiveInventory("Syringe",1);
	GiveInventory("Order Pistol",1);
	GiveInventory("Syringe", 1);
	GiveInventory("Money",startMoney);
	
	TakeInventory("OrderPrimary",999);	
	TakeInventory("IsReloading", 1);
	
	GiveInventory("Action_Block", 1);
	GiveInventory("Action_TertiaryFire", 1);
	GiveInventory("Action_FuckOff", 1);
	GiveInventory("Action_QuickGrenade", 1);
	GiveInventory("Action_SwitchAmmo", 1);
	GiveInventory("Action_Pay", 1);

	// Select the pistol
	SetWeapon("Order Pistol");
}

int alive[MAXPLAYERS];

// Total players in the game
int AlivePlayers(void) { return PlayerCount(); }

// YOUR SQUAD WAS WIPED OUT
void SquadDied(void) { SetActivator(-1); ACS_NamedExecute("SquadDied",0,0,0,0); }
script "SquadDied" (void)
{
	if (GameType() == GAME_TITLE_MAP || SandboxMode())
		terminate;
		
	gameState = STATE_DIED;
		
	SetFont("BIGFONT");
	PrintBold(s:"\cKYOUR SQUAD WAS WIPED OUT");
	
	int cnt;
	int goal = 35*5;
	while (cnt < goal && PlayerCount() > 0)
	{
		cnt ++;
		delay(1);
	}
	
	if (cnt >= goal)
		SquadEnd();
	else
		ResetMap();
}

// Squad successfully died	
void SquadEnd()
{
	Exit_Normal(0);
}

// Update alive players, someone died or left
void UpdateAlive(void)
{
	int aliveCount = 0;
	for (int l=0; l<MAXPLAYERS; l++)
	{
		int TID = PlayerTID(l);
		if (PlayerInGame(l) && PlayerIsSpectator(l) == 0 && (ClassifyActor(TID) & ACTOR_ALIVE))
		{
			alive[l] = true;
			aliveCount ++;
		}
		else
			alive[l] = false;
	}
	
	// Looks like everyone is DEAD!!!
	if (aliveCount <= 0)
		SquadDied();
}

// Respawn all dead players
void RevivePlayers(void)
{
	for (int l=0; l<MAXPLAYERS; l++)
	{
		// This player is DEAD
		if (PlayerIsSpectator(l) == 2 || !StoredInfo[l].joined)
		{
			alive[l] = true;
			SetDeadSpectator(l, false);
			SetPlayerLivesLeft(l, 4);
		}
	}
}

script "RevivePlayers" (void)
{
	RevivePlayers();
}

// How many players are ALIVE?
// Server-sided only
int LivingPlayers(void)
{
	int count = 0;
	for (int l=0; l<MAXPLAYERS; l++)
	{
		if (alive[l])
			count ++;
	}
	
	return count;
}

// Handlers for different things, this keeps the "alive" count intact
script "SurvivalDis" (int gone) DISCONNECT 
{ 
	ClearPerks(gone);
	UpdateAlive(); 
}

script "SurvivalDied" DEATH 
{ 
	SpitWeapons();
	
	// Dying in trader is allowed, nowhere else
	if (gameState != STATE_TRADER)
	{
		// Players who "fake join" should NOT store perks
		if (CheckInventory("UsedPerkMenuOnce"))
			StorePerks();
			
		UpdateAlive();
		SetDeadSpectator(PlayerNumber(), 1);
	}
}

// Show the perk menu if this is our first time joining
void CheckPerkMenu()
{
	int PN = PlayerNumber();
	
	StoredInfo[PN].joined = true;
	
	if (!StoredInfo[PN].pickedPerk)
		PerkMenuControl(PM_SHOW);
}

script "SurvivalJoin" ENTER 
{ 
	// Can we join?
	if (GameJoinable())
	{
		GiveSpawnInventory();
		SetupPlayer();
		CheckPerkMenu();
		UpdateAlive(); 
	}
	
	// Not allowed
	else
	{
		ACS_NamedExecuteAlways("JoinLateMessage", 0);
		SetDeadSpectator(PlayerNumber(), 1);
		UpdateAlive();
	}
	
	// FIRST PLAYER TO EVER JOIN
	if (gameState == STATE_LOBBY)
		gameState = STATE_WAVE;
}

script "SurvivalRespawn" RESPAWN 
{ 
	int PN = PlayerNumber();
	
	GiveSpawnInventory();
	SetupPlayer();
	RestorePerks();
	CheckPerkMenu();
	UpdateAlive(); 
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Clientsided
script "JoinLateMessage" CLIENTSIDE
{
	if (PlayerNumber() != ConsolePlayerNumber())
		terminate;
		
	delay(5);
	SetFont("TFNTAB");
	
	int NX = Half(ScreenWidth() * 1.0);
	int NY = (ScreenHeight() * 1.0) - 120.0;
	
	ZTHudMessage(NX, NY, LID_CANTSTAY, "You will respawn when the round is over!", ALIGN_CENTER, ALIGN_BOTTOM, 6.0);
	
	SetFont("TFNNAME");
	ZTHudMessage(NX, NY-24.0, LID_CANTSTAY-1, "\cDHey, sit tight!", ALIGN_CENTER, ALIGN_BOTTOM, 6.0);
}
