//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
//          Z O M B I E   T O R R E N T   T R A D E R   M E N U 
//        Super fancy magical menu has been re-coded a SECOND TIME
//               [ BCS magic by Zedek the Plague Doctor ]
// 
//      Steal this and a cranus will come crashing into your window
//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

#include "Trader/Materials.h"
#include "Trader/Elements.h"
#include "Trader/Scrollbars.h"
#include "Trader/SellList.h"
#include "Trader/Mode_Guns.h"
#include "Trader/Mode_Perks.h"
#include "Trader/Mode_Upgrades.h"
#include "Trader/Tooltip.h"

// Costs for armor and ammo
int GetArmorCost() { return 200; }
int GetRefillCost() { return 500; }

#define TMODE_OPEN						0
#define TMODE_CLOSE						1
#define SCR_TRADEROPEN					250

#define CS_BUYWEP						0
#define	CS_SELLWEP						1
#define CS_BUYUPGRADE					2
#define CS_SELLUPGRADE					3

// PUKED WHEN USING TRADER //
script SCR_TRADEROPEN (int mode)
{
	switch (mode)
	{
		case TMODE_OPEN:
		if (!CheckInventory("MenuActive"))
		{
			FreezeMe(true);
			GiveInventory("TraderDarken", 1);
			GiveInventory("MenuActive",1);
			GiveInventory("MenuWeapon",1);
			SetWeapon("MenuWeapon");
			ACS_NamedExecuteAlways("TraderInterface",0,0,0,0);
		}
		break;

		case TMODE_CLOSE:
			FreezeMe(false);
			TakeInventory("TraderDarken", 1);
			TakeInventory("MenuActive",1);
			TakeInventory("MenuWeapon",1);
			delay(3);
			SwitchToLast();
		break;
	}
}

namespace TraderEnums
{
	enum TraderStateT
	{
		TS_PERKS,
		TS_GUNS,
		TS_UPGRADES,
	};
}

// Various handshake types
#define HS_BUYGUN					0
#define HS_SELLGUN					1
#define HS_BUYARMOR					2
#define HS_BUYAMMO					3
#define HS_UPGRADEGUN				4

namespace TraderUI
{
	using TraderEnums;
	using TElem;
	
	// We're executing a puke request
	bool Handshaking;
	
	// TraderStateT TraderState = TS_PERKS;
	TraderStateT TraderState = TS_PERKS;
	
	// Which perk are we currently filtering by?
	str FilterPerk = "Perk_Commando";
	
	// Which weapon are we upgrading?
	int UpgradeGun = -1;
	
	// Holding down left mouse?
	private bool MouseHeld;
	
	// Player's input
	int PlayerInput, PlayerInputOld;
	
	void DoHandshake(int type)
	{
		ACS_NamedExecuteAlways("Handshake",0,type);
	}
	
	// Server -> Client
	// Tells us that we successfully made a request
	script "Handshake" (int type) CLIENTSIDE
	{
		switch (type)
		{
			case HS_BUYGUN:
				TElem::ModeGuns::M_Handshook();
			break;
			
			case HS_SELLGUN:
				TElem::SellList::M_Handshook();
			break;
			
			case HS_BUYARMOR:
				TElem::M_BoughtArmor();
			break;
			
			case HS_BUYAMMO:
				TElem::M_BoughtAmmo();
			break;
			
			case HS_UPGRADEGUN:
				TElem::ModeUpgrade::M_Handshook();
			break;
		}
	}
	
	// Opened the trader
	void TraderOpened()
	{
		LocalAmbientSound("trader/hello", 127);
		TraderState = TS_PERKS;
	}
	
	// Closed the trader
	void TraderClosed()
	{
		TE_Cleanup();
	}
	
	// Switch trader states
	void SwapTraderState(TraderStateT newState, bool FromUpgrade = false)
	{
		TE_Cleanup(true, FromUpgrade);
		TraderState = newState;
	}
	
	// Actual main trader script
	script "TraderInterface" (void) CLIENTSIDE
	{
		if (ConsolePlayerNumber() != PlayerNumber())
			terminate;
			
		void ControlInput()
		{
			PlayerInput = GetPlayerInput(-1, INPUT_BUTTONS);
			PlayerInputOld = GetPlayerInput(-1, INPUT_OLDBUTTONS);
			
			// Pressed left click for the first time
			if (ButtonPressed(BT_ATTACK, PlayerInput, PlayerInputOld))
				TE_MouseClick();
			
			// Released it
			else if (ButtonReleased(BT_ATTACK, PlayerInput, PlayerInputOld))
				TE_MouseRelease();
				
			// Pressed right click for the first time
			else if (ButtonPressed(BT_ZOOM, PlayerInput, PlayerInputOld))
				TE_MouseRightClick();
				
			// Strafed
			else if (ButtonPressed(BT_MOVELEFT, PlayerInput, PlayerInputOld))
			{
				// Perks <- Guns / Perks <- Upgrades
				if (TraderState == TS_GUNS || TraderState == TS_UPGRADES)
				{
					bool FU = false;
					
					if (TraderState == TS_UPGRADES)
						FU = true;
						
					LocalAmbientSound("trader/back", 127);
					SwapTraderState(TS_PERKS, FU);
				}
				// Exit <- Menu
				else
					NamedRequestScriptPuke("TraderAbort");
			}
		}
		
		// THIS HAPPENS EVERY SINGLE TIC
		void ControlTrader()
		{
			ControlInput();
			UpdateCursor();
			DrawCursor("TRADCURS");
			TE_Tic();
		}
		
		// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
		
		TraderOpened();
		
		while (CheckInventory("MenuActive") )
		{
			FullHUD();
			ControlTrader();
			RestoreHUD();
			delay(1);
		}
		
		TraderClosed();
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// -- TELEPORT ALL OF THE PLAYERS OUT OF THE TRADER -----------
function void teleportPlayers(void)
{
	// Loop through all of the players!
	for (int x; x < MAXPLAYERS; ++x)
	{
		if (!PlayerInGame(x))
			continue;
			
		SetActivator(x+p_baseTID);
		
		// Do we have the trader inventory item?
		if (CheckInventory("InTrader"))
		{
			// DON'T USE HMUS, THIS IS SERVER-SIDED!!!

			SetFont("TFNNAME");
			ZTHudMessage(Half(ScreenWidth() * 1.0), (ScreenHeight() * 1.0) - 70.0, LID_CANTSTAY, "\cJYou can't stay in the shop after it closes!", ALIGN_CENTER, ALIGN_CENTER, 3.0);
			
			// Only one TID, since players shouldn't collide with each other anyway
			Teleport(TELEPORT_TID, 0, 0);
			
			ACS_ExecuteAlways(SCR_TRADEROPEN, 0, TMODE_CLOSE);
		}
	}
}

// -- PLAYER WALKS IN OR OUT OF THE TRADER --------------------
//script and function for teleporting players at trader [OL]
script 20 (int spot)
{
	// Determine inventory based on side, walking in or out
	InvEnsure("InTrader", (LineSide() == SIDE_FRONT));

	// Update our InTrader state
	ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_INTRADER, CheckInventory("InTrader"), PlayerNumber());
}

//---------------------------------------------------------------------
//
// T R A D E R   V O T I N G
// ALLOWS PLAYERS TO VOTE TO END TRADERTIME
//
//---------------------------------------------------------------------

// Vote to end trader time
script "EndTraderVote" (void) NET
{
	if (!trading)
		terminate;
	
	// We voted
	if (CheckInventory("TraderReady") > 0)
	{
		Print(s:"\cJYou have already voted!");
		terminate;
	}
	
	GiveInventory("TraderReady", 1);
	Print(s:"\cJYou have voted to end trader time.");
	
	if (CheckTraderVotes())
	{
		AmbientSound("slot/failed", 127);
		SetFont("BIGFONT");
		HudMessage(s:"\cDPlayers ended trader time!"; 1, 450, CR_WHITE, 0.5, 0.85, 2.0,0.25);
		SetFont("SMALLFONT");
		ACS_Terminate(SCR_TRADERTIMER,0);
		
		delay(35);
		
		EndTraderTime();
	}
}

// Have all players voted?
bool CheckTraderVotes(void)
{
	int voterCount = AlivePlayers();
	int voteCount = 0;
	
	// Get all players who have voted
	for (int l=0; l<MAXPLAYERS; l++)
	{
		if (PlayerExists(PLAYER_BASE_TID+l))
		{
			if (CheckActorInventory(PLAYER_BASE_TID+l, "TraderReady") > 0)
				voteCount ++;
		}
	}
	
	return (voteCount >= voterCount);
}

// Trader time actually ended
function void EndTraderTime(void)
{
	trading=0;
	SetMusic("");
	ACS_Execute(62,0,0,0,0);
}