#define ALIGN_CENTER 0.4
#define ALIGN_CENTER_Y 0.0
#define ALIGN_LEFT 0.1
#define ALIGN_TOP 0.1
#define ALIGN_RIGHT 0.2
#define ALIGN_BOTTOM 0.2

#define FOREVER				0

int ScreenWidth(void)
{
	if (!IsClientOnly())
		return CheckInventory("ResolutionX");
		
	return GetScreenWidth();
}

int ScreenHeight(void)
{	
	if (!IsClientOnly())
		return CheckInventory("ResolutionY");
		
	return GetScreenHeight();
}

struct Picture
{
	int W, H;
	str Tex;
};

struct Rect
{
	int X, Y;
	int W, H;
};

void SetRect(struct Rect& Box, int X, int Y, int W, int H)
{
	Box.X = X;
	Box.Y = Y;
	Box.W = W;
	Box.H = H;
}

// Mouse is within this rectangle
bool Mousing(struct Rect& Box)
{
	int MX = GetMouseX();
	int MY = GetMouseY();
	
	if (MX >= Box.X && MX <= Box.X+Box.W && MY >= Box.Y && MY <= Box.Y+Box.H)
		return true;
		
	return false;
}

// Get an image based on whether or not this region is being hovered over
str HoverImg(struct Rect& Box, str noHover, str yesHover)
{
	return StrEval( Mousing(Box), noHover, yesHover );
}

//------------------------------------------------
// H U D   M E S S A G E
//------------------------------------------------

// Draw a vertical segmented image
// This uses a top, mid, and low pic
int ZTHudSegment(int x, int y, int h, int ID, struct Picture& TopPic, struct Picture& MidPic, struct Picture& LowPic, int duration = 0.50, bool remove = false)
{
	int segY;
	str text = "a";
	
	if (remove)
		text = "";
	
	// Middle pictures FIRST
	SetFont(MidPic.Tex);
	for (segY = TopPic.H; segY < h-MidPic.H; segY += MidPic.H)
	{
		ZTHudMessage(x, y+segY, ID, text, ALIGN_LEFT, ALIGN_TOP, duration);
		ID --;
	}
	
	// Top picture next!
	SetFont(TopPic.Tex);
	ZTHudMessage(x, y, ID, text, ALIGN_LEFT, ALIGN_TOP, duration);
	ID --;
	
	SetFont(LowPic.Tex);
	ZTHudMessage(x, (y+h) - LowPic.H, ID, text, ALIGN_LEFT, ALIGN_TOP, duration);
	ID --;
	
	return ID;
}

// Brand new super cool HMUSRedux function
void ZTImgMessage(str tex, int x, int y, int id, str text, int alignX = ALIGN_CENTER, int alignY = ALIGN_CENTER, int showTime = 0.50, int fadeTime = 0.25, int resX = -1, int resY = -1)
{
	SetFont(tex);
	ZTHudMessage(x, y, id, text, alignX, alignY, showTime, fadeTime, resX, resY);
}

void ZTHudMessage (int x, int y, int id, str text, int alignX = ALIGN_CENTER, int alignY = ALIGN_CENTER, int showTime = 0.50, int fadeTime = 0.25, int resX = -1, int resY = -1)
{
	x = Shave(x);
	y = Shave(y);
	
	if (resX == -1 && resY == -1)
	{
		// DO NOT DO THIS IN SINGLEPLAYER
		if (!IsClientOnly())
		{
			resX = CheckInventory("ResolutionX") * 1.0;
			resY = CheckInventory("ResolutionY") * 1.0;
		}
		else
		{
			resX = ScreenWidth() * 1.0;
			resY = ScreenHeight() * 1.0;
		}
	}
	
	int finalX, finalY;

	if (y < 0.0) { finalY = resY + y; }
	else { finalY = y; }

	if (x < 0.0) { finalX = resX + x; }
	else { finalX = x; }

	// Typewriter message
	bool typewriter = false;

	int type = HUDMSG_PLAIN | HUDMSG_FADEOUT;
	if (showTime <= 0.0)
		type = HUDMSG_PLAIN;
	else if (showTime >= 100.0 && showTime < 999.0)
	{
		showTime -= 100.0;
		typewriter = true;
		type = HUDMSG_TYPEON;
	}

	SetHUDSize(ShaveInt(resX), ShaveInt(resY), false);
	
	if (!typewriter)
		HudMessage(s:text; type, id, CR_UNTRANSLATED, finalX + alignX, finalY + alignY, showTime, fadeTime);
	else
		HudMessage(s:text; type, id, CR_UNTRANSLATED, finalX + alignX, finalY + alignY, showTime, 0.03, fadeTime);
		
	SetHUDSize(0, 0, false);
}

// May or may not be used in the future
// Tiles a single image across the entire screen
void ScreenTile(str pic, int w, int h, int baseID)
{
	int SW = ScreenWidth();
	int SH = ScreenHeight();
	int cnt = 0;
	
	w = ShaveInt(w);
	h = ShaveInt(h);
	
	SetFont(pic);
	for (int x=0; x<=SW; x+=w)
	{
		for (int y=0; y<=SH; y+=h)
		{
			ZTHudMessage(x*1.0, y*1.0, baseID - cnt, "a", ALIGN_LEFT, ALIGN_TOP);
			cnt ++;
		}
	}
} 

//------------------------------------------------
// C U R S O R
// Used for controlling various AI bits, specifically for trader
//------------------------------------------------

// TODO: MAKE THIS A CVAR
#define CURSOR_BASEMULT			1.25

struct CursorData
{
	int X, Y;
};

struct CursorData Cursor;

bool MouseInRect(int x1, int y1, int x2, int y2)
{
	if (GetMouseX() >= x1 && GetMouseY() >= y1 && GetMouseX() <= x2 && GetMouseY() <= y2)
		return true;
		
	return false;
}

int GetMouseX (void)
{
	return FixedMul(Cursor.X, ScreenWidth() * 1.0);
}

int GetMouseY (void)
{
	return FixedMul(Cursor.Y, ScreenHeight() * 1.0);
}

// -- UPDATE THE CURSOR: CONTROLS SPEED, ETC.

void UpdateCursor(bool drawAfter = false)
{
	int speedX, speedY;

	// How much to scale our mouse movement
	speedX = FixedDiv(1.0, FixedMul(GetCVar("m_yaw"), GetCVar("mouse_sensitivity")) );
	speedY = FixedDiv(1.0, FixedMul(GetCVar("m_pitch"), GetCVar("mouse_sensitivity")) );
	
	// Scale them up a bit to make them "normal"
	speedX = FixedMul(2.5, speedX);
	speedY = FixedMul(5.0, speedY);
	
	// Plain speed just isn't fast enough
	speedX = FixedMul(speedX, CURSOR_BASEMULT);
	speedY = FixedMul(speedY, CURSOR_BASEMULT);
	
	// Delta speed
	int dx = -FixedMul(GetPlayerInput(-1, INPUT_YAW), speedX);
	int dy = -FixedMul(GetPlayerInput(-1, INPUT_PITCH), speedY);
	
	Cursor.X += dx;
	Cursor.Y += dy;
	
	Cursor.X = clamp(Cursor.X, 0.0, 1.0);
	Cursor.Y = clamp(Cursor.Y, 0.0, 1.0);
	
	if (drawAfter) { DrawCursor(); }
}

void DrawCursor(str pic = "cursorp3")
{
	if (pic == "")
	{
		BlankMessage(LID_CURSOR);
		return;
	}
	
	SetFont(pic);
	ZTHudMessage(GetMouseX(), GetMouseY(), LID_CURSOR, "a", ALIGN_LEFT, ALIGN_LEFT);
}

//-----------------------------------------------
// Send resolution to server
//-----------------------------------------------
script "ResSendServer" (int resX, int resY) NET
{
	InvEnsure("ResolutionX", resX);
	InvEnsure("ResolutionY", resY);
}

// Clientsided script - responsible for resolution
script "ResSendClient" (void) CLIENTSIDE
{
	int LastResX, LastResY;
	
	while (ClassifyActor(0) & ACTOR_ALIVE)
	{
		if (LastResX != GetScreenWidth() || LastResY != GetScreenHeight())
		{
			LastResX = GetScreenWidth();
			LastResY = GetScreenHeight();
			
			NamedRequestScriptPuke("ResSendServer",LastResX,LastResY,0);
		}
		
		delay(5);
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

enum BioModeType
{
	BM_NULL,
	BM_WAVE,
	BM_TRADER,
};

enum UpdateType
{
	UT_POOL,
	UT_TIME,
	UT_COUNTDOWN,
	UT_VOTES,
	UT_INTRADER,
};

struct HUDInfoT
{
	int ZombiePool;
	int TraderTime;
	int BioMode;
	int Countdown;
	int Wave;
	int Voters, Votes;
	bool InTrader;
	bool Dirty;
	bool ReceivedInfo;
	bool ShowedMapInfo;
};

HUDInfoT			HUDInfo;

// CLIENT
// Called when the map loads locally
script "HCOpen" OPEN CLIENTSIDE
{
	// Request HUD info from the server every 2 seconds
	// until we successfully update something
	while (!HUDInfo.ReceivedInfo)
	{
		ACS_NamedExecuteAlways("HCOpenServer", 0, ConsolePlayerNumber());
		delay(35*2);
	}
}

// CLIENT -> SERVER
// Requests HUD info from the server, seems to send it to everyone
script "HCOpenServer" (int CPN) NET
{
	// Wave countdown, just send seconds
	if (wavestarting || currentwave <= 0)
		ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_COUNTDOWN, starttime + (currentwave * 100), CPN);
	
	// Send wave-related things
	else if (isfighting)
		ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_POOL, zombiepool, CPN);
	
	// Send trader-related things
	else
		ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_TIME, tradertime, CPN);
		
	// Send voter info
	ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_VOTES, VoteCount() + (VoterCount() * 100), CPN);
	
	// Don't worry about sending "in trader" because we won't be in there at this point
}

// SERVER -> CLIENT
// Update a specific variable
script "HC_UpdateVar" (int type, int cntr, int CPN) CLIENTSIDE
{
	// Specific player ONLY
	if (CPN != -666 && ConsolePlayerNumber() != CPN)
		terminate;
				
	HUDInfo.ReceivedInfo = true;
	
	switch (type)
	{
		// Zombie pool
		case UT_POOL:
			if (HUDInfo.ZombiePool != cntr)
				HUDInfo.Dirty = true;
			if (HUDInfo.BioMode != BM_WAVE)
				HUDInfo.Dirty = true;
				
			HUDInfo.ZombiePool = cntr;
			HUDInfo.BioMode = BM_WAVE;
		break;
		
		// Trader time
		case UT_TIME:
			if (HUDInfo.TraderTime != cntr)
				HUDInfo.Dirty = true;
			if (HUDInfo.BioMode != BM_TRADER)
				HUDInfo.Dirty = true;
				
			HUDInfo.TraderTime = cntr;
			HUDInfo.BioMode = BM_TRADER;
		break;
		
		// Wave countdown
		// HACK: This is combined with wave number
		case UT_COUNTDOWN:
			int waveNum = ShaveInt(FixedDiv(cntr*1.0,100.0));
			int timer = cntr - (100*waveNum);
			
			if (HUDInfo.Wave != waveNum)
				HUDInfo.Dirty = true;
			if (HUDInfo.Countdown != timer)
				HUDInfo.Dirty = true;
			if (HUDInfo.BioMode != BM_NULL)
				HUDInfo.Dirty = true;
				
			HUDInfo.Wave = waveNum;
			HUDInfo.Countdown = timer;
			HUDInfo.BioMode = BM_NULL;
		break;
		
		// Vote information
		// HACK: This is combined with voter count
		case UT_VOTES:
			int voters = ShaveInt(FixedDiv(cntr*1.0,100.0));
			int votes = cntr - (100*voters);
			
			if (HUDInfo.Voters != voters)
				HUDInfo.Dirty = true;
			if (HUDInfo.Votes != votes)
				HUDInfo.Dirty = true;
				
			HUDInfo.Voters = voters;
			HUDInfo.Votes = votes;
		break;
		
		// In the trader?
		case UT_INTRADER:
			if (HUDInfo.InTrader != cntr)
				HUDInfo.Dirty = true;
				
			HUDInfo.InTrader = cntr;
		break;
	}
}

// Runs on the server, controls things constantly
script "HudControllerServer" OPEN
{
	int StoredPool = -1;
	int StoredTime = -1;
	int StoredStarting = -1;
	int StoredCount = -1;
	int StoredVoters = -1;
	int StoredVotes = -1;
	
	SetActivator(-1);
	
	while (true)
	{
		// Wave has started or ended
		if (StoredStarting != wavestarting)
		{
			StoredStarting = wavestarting;
			
			// Tell clients that our wave is counting down, only if it starts
			if (StoredStarting)
				ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_COUNTDOWN, starttime + (currentwave * 100), -666);
			else
				ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_POOL, zombiepool, -666);
		}
		
		// Game is going!
		if (!wavestarting)
		{
			// Zombie count
			if (StoredPool != zombiepool && isfighting)
			{
				StoredPool = zombiepool;
				ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_POOL, zombiepool, -666);
			}
			
			// Trader time
			if (StoredTime != tradertime && !isfighting)
			{
				StoredTime = tradertime;
				ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_TIME, tradertime, -666);
			}
			
			// Votes
			if (!isfighting)
			{
				int voters = VoterCount();
				int votes = VoteCount();
				
				if (StoredVotes != votes || StoredVoters != voters)
				{
					StoredVotes = votes;
					StoredVoters = voters;
					ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_VOTES, votes + (voters * 100), -666);
				}
			}
		}
		
		// Check for countdown if we're ticking down
		if (StoredCount != starttime)
		{
			StoredCount = starttime;
			ACS_NamedExecuteAlways("HC_UpdateVar", 0, UT_COUNTDOWN, starttime + (currentwave * 100), -666);
		}
		
		delay(4);
	}
}

int VoteCount()
{
	// Reset it
	int totalVotes = 0;
	
	for (int l=0; l<MAXPLAYERS; l++)
	{
		if (PlayerExists(PLAYER_BASE_TID+l))
		{
			if (CheckActorInventory(PLAYER_BASE_TID+l, "TraderReady") > 0)
				totalVotes ++;
		}
	}
	
	return totalVotes;
}

int VoterCount() { return AlivePlayers(); }

// Update some HUD information
// Serves as a middleman between server and client
script "HudController" (void)
{
	InvEnsure("HUDDummy_CurrentSeconds", starttime);
	InvEnsure("HUDDummy_HasBegan", game_hasbegan);
	InvEnsure("HUDDummy_WaveStarting", wavestarting);
	InvEnsure("HUDDummy_Wave", waveno);
	InvEnsure("HUDDummy_CurrentWave", currentwave);
	
	// Is calling this every tic bad? Who knows
	delay(4);
	restart;
}

void CleanCircle()
{
	for (int l=0; l<5; l++)
		BlankMessage(HID_BIO-l);
}

// Controls the circle on the top-right
// This is called locally when the map opens, EVEN FOR SPECTATORS
script "HUDCircle" OPEN CLIENTSIDE
{		
	int pooldraw, imagetochoose;
	str tradertext, extra;

	HUDInfo.BioMode = BM_NULL;

	// Stored BioMode
	// This is updated AFTER we draw everything
	int storedMode;
	
	// Stored trader state
	// Changes if we enter / leave trader
	int storedTrader;
	
	// Hud ID to draw on
	int BID;
	
	bool drawHUD = (!GetCvar("cl_disablezthud") && !CheckInventory("MenuActive"));

	// No circle on title
	if (GameType() == GAME_TITLE_MAP)
		terminate;

	while (true)
	{
		if (storedMode != HUDInfo.BioMode || storedTrader != HUDInfo.InTrader || HUDInfo.Dirty)
		{
			CleanCircle();
			
			BID = HID_BIO;
			
			// Which mode are we in?
			switch (HUDInfo.BioMode)
			{
				// IN THE MIDDLE OF A WAVE
				case BM_WAVE:
					ZTImgMessage(StrEval(HUDInfo.Wave >= 10, "BIOTIME2", "BIOTIME3"), -100.0, 100.0, BID, "a", ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					BID --;

					// Plain ol' wave
					if (HUDInfo.Wave < 10)
					{
						pooldraw = clamp(HUDInfo.ZombiePool, 0, 9999);
						SetFont("BIGWRITR");
						ZTHudMessage(-100.0, 90.0, BID, StrParam(d:pooldraw), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
						BID --;
						SetFont("TYPEWRIT");
						ZTHudMessage(-100.0, 125.0, BID, StrParam(d:HUDInfo.Wave,s:" / 10"), ALIGN_CENTER, ALIGN_CENTER, FOREVER);
						BID --;
					}
					else
					{
						SetFont("BIGWRITR");
						ZTHudMessage(-100.0, 90.0, BID, "\cKBOSS", ALIGN_CENTER, ALIGN_CENTER, FOREVER);
						BID --;
					}
				break;

				// TRADER TIME
				case BM_TRADER:
					// Main circle
					ZTImgMessage(StrEval(HUDInfo.InTrader, "BIOTIME1", "BIOTIME4"), -100.0, 100.0, BID, "a", ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					BID --;
					
					// Time
					tradertext = StrParam( s:"\cK", s:StrEval(HUDInfo.TraderTime >= 0, "00:00", TimeString(HUDInfo.TraderTime)) );
					SetFont("TYPEWRIT");
					ZTHudMessage(-100.0, 100.0, BID, tradertext, ALIGN_CENTER, ALIGN_CENTER, FOREVER);
					BID --;
						
					// Players who HAVE voted
					int voteCount = HUDInfo.Votes;
					// Number of voters who are in the game
					int voterCount = HUDInfo.Voters;
					
					str readyString = "";
					
					// All players are ready!
					if (voteCount >= voterCount)
						readyString = "\cDAll players ready!";
					else
					{
						str playerString = "Player";
						if (CheckInventory("HUDDummy_TotalVotes") > 1)
							playerString = "Players";
							
						readyString = StrParam(d:voteCount, s:"/", d:voterCount, s:" ", s:playerString, s:" Ready");
					}
					
					SetFont("BIGFONT");
					ZTHudMessage(-100.0, 200.0, BID, readyString, ALIGN_CENTER, ALIGN_TOP, FOREVER);
				break;
			}
			
			storedTrader = HUDInfo.InTrader;
			storedMode = HUDInfo.BioMode;
			HUDInfo.Dirty = false;
		}
		
		delay(2);
	}
}

//-----------------------------------------------
// Show some song / map info
//-----------------------------------------------
void DisplayTrackInfo(int trackIndex, bool trader)
{
	// This executes for all players
	ACS_NamedExecuteAlways("DisplayTrackInfo", 0, trackIndex, trader);
}

// (str song, str artist)
script "DisplayTrackInfo" (int index, int trader) CLIENTSIDE
{
	int TempX, TempY, StartY, SS;
	
	// If index is -1, this is for THIS CLIENT ONLY
	if (index == -1)
	{	
		delay(35);
		
		if (WrongClient())
			terminate;
			
		// We showed map info already!
		if (HUDInfo.ShowedMapInfo)
			terminate;
			
		HUDInfo.ShowedMapInfo = true;
	}
	
	// Pick the artist and song from the index
	str artist, song;
	
	// MAP INFORMATION: INDEX -1
	if (index == -1)
	{
		artist = map_author;
		song = map_title;
	}
	
	// ARE WE IN THE TRADER?
	else if (trader)
	{
		if (!MusicList.CustomCalm)
		{
			song = MusicList.CalmTracks[index].Song;
			artist = MusicList.CalmTracks[index].Artist;
		}
		else
		{
			song = MusicList.CCalmTracks[index].Song;
			artist = MusicList.CCalmTracks[index].Artist;
		}
	}
	
	// NORMAL MUSIC
	else
	{
		if (!MusicList.CustomCombat)
		{
			song = MusicList.CombatTracks[index].Song;
			artist = MusicList.CombatTracks[index].Artist;
		}
		else
		{
			song = MusicList.CCombatTracks[index].Song;
			artist = MusicList.CCombatTracks[index].Artist;
		}
	}
	
	SS = FixedDiv(ScreenHeight() << 16, 1080.0);
	
	TempX = 0.0;
	TempY = ScreenHeight() * 0.5;
	
	// Fading background
	SetFont("MUSIFADE");
	HMUSReduxExtended(TempX, TempY, "a", 0, 0, FADE_ID, 0.1, SIDE_CLIENT, 1.0, 0);
	
	// Which picture to use?
	str bloodPic = "MUSIBACK";
	if (trader || index == -1)
		bloodPic = "MUSIBLUE";
	
	// Blood
	SetFont(bloodPic);
	HMUSReduxExtended(TempX, TempY, "a", 0, 0, BACKGROUND_ID, 4.0, SIDE_CLIENT, 2.0, 0);
	
	// Move to the right a bit more to make space
	TempX += 24.0;
	
	// Figure out how far down we need to move
	TempY += 64.0;
	StartY = TempY;
	
	// Song info
	TempY = StartY - 1.0;
	SetFont("TYPEWRIT");
	HMUSReduxExtended(TempX, TempY, song, 0, 2, SONG_ID, 5.0, SIDE_CLIENT, 2.0, 0);
	
	TempX += 8.0;
	
	// Artist info
	TempY = StartY + 1.0;
	SetFont("BIGFONT");
	HMUSReduxExtended(TempX, TempY, artist, 0, 0, ARTIST_ID, 5.0, SIDE_CLIENT, 2.0, 0);
}
