///////////////////////////////////////////////////////////////////////////////////////
// -- VOICE SCRIPT AND CLASS SWITCH SCRIPTS, WILL BE ADDED SOON -- //
///////////////////////////////////////////////////////////////////////////////////////

// -- DIFFERENT SOUND VALUES -- //
enum VoiceType
{
	VS_RELOAD,			// 0
	VS_TAUNT,			// 1
	VS_PAIN,			// 2
	
	VS_GRUNT,			// 3
	VS_HEAL,			// 4
	VS_CASHTOSS,		// 5
	VS_WEAPONBIG,		// 6
	VS_WEAPON,			// 7
	VS_BOSS,			// 8
	VS_IDLE,			// 9
	VS_TEAMDEATH,		// 10
	VS_RESPAWN,			// 11
	VS_OOF,				// 12
	VS_BIGGRUNT,		// 13
	
	// Was going to make this dynamic, but oh well
	VS_KILL,			// 14
	VS_KILLANNOY,		// 15
	VS_KILLGIB,			// 16
	VS_KILLBLUNT,		// 17
	VS_KILLHEAD			// 18
};

// Which type of kill was this?
int GetKillType()
{
	if (CheckInventory("Kill_Gib"))
		return VS_KILLGIB;
	else if (CheckInventory("Kill_Headshot"))
		return VS_KILLHEAD;
	else if (CheckInventory("Kill_Blunt"))
		return VS_KILLBLUNT;
	
	return VS_KILL;
}

///////////////////////////////////
// - VOICE CHANGE SCRIPT ------- //
/////////////////////////////////////////////////////////////
// - Credit to TMC and ZzZombo for helping with this //
/////////////////////////////////////////////////////////////

#define VOICE_COUNT 		5
#define MAX_VOICES 			120
#define MAX_VOICESOUNDS		32

// An individual voice line
struct VoiceSound
{
	str Sound;
	int Duration;
	int Chance;
};

// A whole voice set
struct VoiceEntry
{
	str DisplayName;
	struct VoiceSound Lines[MAX_VOICESOUNDS];
};

int voicePackCount;
struct VoiceEntry VoiceList[MAX_VOICES];

// -- VOICE SYSTEM FUNCTIONS -----------------------------------------------------

// Add a new voice
// Name, Reload Sound, Taunt Sound, Pain Sound, Taunt Time
void AddVoicepack(str vName)
{
	for (int l=0; l<MAX_VOICESOUNDS; l++)
		VoiceList[VoicePackCount].Lines[l].Sound = "";

	VoiceList[VoicePackCount].DisplayName = vName;
	voicePackCount ++;
}

void SetVoiceSound(int snd, str val, int lng = 1.0, int chn = -1) 
{ 
	VoiceList[VoicePackCount-1].Lines[snd].Sound = val; 
	
	// Calculate length in TICS
	if (snd == VS_PAIN)
		lng = 0.5;
		
	// Calculate default chances
	if (chn == -1)
	{
		switch (snd)
		{
			default:
				chn = 0;
			break;
			
			case VS_GRUNT:
				chn = 6;
			break;
			
			case VS_BIGGRUNT:
				chn = 12;
			break;
			
			case VS_KILL:
			case VS_KILLANNOY:
			case VS_KILLGIB:
			case VS_KILLBLUNT:
			case VS_KILLHEAD:
				chn = 10;
			break;
		}
	}
	
		
	int ticTime = ShaveInt(35 * lng);
	
	VoiceList[VoicePackCount-1].Lines[snd].Duration = ticTime; 
	VoiceList[VoicePackCount-1].Lines[snd].Chance = chn; 
}

// Remove all voicepacks
// This should ONLY be used for core gameplay
void ClearVoicepacks(void) { voicePackCount = 0; }

// Does this voicepack have a valid sound entry for this line?
bool ValidLine(int VP, int LN)
{
	if (StrLen(VoiceList[VP].Lines[LN].Sound) > 0)
		return true;
		
	return false;
}

// Return a voice fallback
int VoiceFallback(int VP, int Mode)
{
	switch (Mode)
	{
		case VS_KILLANNOY:
		case VS_KILLGIB:
		case VS_KILLBLUNT:
		case VS_KILLHEAD:
			if (!ValidLine(VP, Mode))
				return VS_KILL;
		break;
	}
	
	return Mode;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// How many voices on either side of the selection to display
#define SECTION_PADDING		2

// Compiles the Available Voicepacks section
void CompileSection(int Selected)
{
	// str sectionString = "\cDAvailable Voicepacks:\n\n";
	str color = "\cM";
	str pic = "VOICEBLK";
	int startX, startY;
	int sectionHeight;
	int counter;
	
	int adder, totalHeight;
	
	totalHeight = (SECTION_PADDING*2)+1;
	
	counter = 0;
	
	startX = 0.0;
	startY = FixedMul(CheckInventory("ResolutionY")*1.0,0.25);
	
	sectionHeight = 26.0 * totalHeight;
	startY = startY - FixedDiv(sectionHeight,2.0);
	
	sectionHeight = 26.0;
	
	if (Selected == 0)
		adder = totalHeight-1;
	else if (Selected == 1)
		adder = totalHeight-2;
	else
		adder = SECTION_PADDING;
	
	for (int l=Selected-SECTION_PADDING; l<=Selected+adder; l++)
	{
		SetFont(pic);
		HMUSRedux(startX, startY+(sectionHeight*counter), "", 0, 1, SECTION_BG_ID+counter, 2.0, 0);
		SetFont("BIGFONT");
		HMUSRedux(startX+16.0, startY+(sectionHeight*counter), "", 0, 1, SECTION_ID+counter, 2.0, 0);
		SetFont("SMALLFONT");
		
		if (l >= voicePackCount)
			break;
		
		if (l < 0)
			continue;
			
		if (l == Selected)
		{
			color = "\cK";
			pic = "VOICERED";
		}
		else
		{
			color = "\cM";
			pic = "VOICEBLK";
		}
		
		SetFont(pic);
		HMUSRedux(startX, startY+(sectionHeight*counter), "a", 0, 1, SECTION_BG_ID+counter, 2.0, 0);
		SetFont("BIGFONT");
		HMUSRedux(startX+16.0, startY+(sectionHeight*counter), StrParam(s:color,d:l,s:".) ",s:VoiceList[l].DisplayName), 0, 1, SECTION_ID+counter, 2.0, 0);
		SetFont("SMALLFONT");
		
		counter ++;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Wow, we should probably get the player's stored voice eh?
// That might be a good idea you know
//
// (Called from ZT2Survival.h in SetupPlayer)
// This is server-sided, we CAN get their CVAR from this end!

void FindMyVoice(void)
{
	str StoredVoice = GetUserCVarString(PlayerNumber(),"cl_zt2voice");
	
	for (int l=0; l<voicePackCount; l++)
	{
		if (VoiceList[l].DisplayName == StoredVoice)
			InvEnsure("PlayerVoice", l);
	}
}

// Requests a taunt
// Client -> Server
script "ZT2Taunt" (void) NET 
{ 
	PlayVoiceSound(VS_TAUNT, -1);
}

// We selected a specific voice, let's show information about it
// Client or serverside, who knows, who cares
void SetVoicePreference(int pref)
{
	LocalAmbientSound(VoiceList[CheckInventory("PlayerVoice")].Lines[VS_RELOAD].Sound,127);
	
	// Show a message
	SetFont("BIGWRITR");
	HudMessage(s:"\cKVoice changed:\n\cD",s:VoiceList[CheckInventory("PlayerVoice")].DisplayName;HUDMSG_FADEOUT,VOICE_ID,CR_WHITE,1.5,0.8,2.0,0.5);
	SetFont("BIGFONT");
	
	CompileSection(CheckInventory("PlayerVoice"));
}

// -- CHANGE VOICES --
// Client -> Server
script "ChangeVoice" (void) NET
{
	// Actually control the junk
	if (CheckInventory("PlayerVoice") < voicePackCount-1)
		GiveInventory("PlayerVoice",1);
	else
		TakeInventory("PlayerVoice",99);
		
	// Update our preference accordingly
	SetUserCVarString(PlayerNumber(),"cl_zt2voice",VoiceList[CheckInventory("PlayerVoice")].DisplayName);
	SetVoicePreference(CheckInventory("PlayerVoice"));
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Voice timers for ALL players
// If this is > 0, they cannot play any voice sounds (with exceptions)
int voiceTimers[MAXPLAYERS];

// This controls our voice timer
script "HandleVoiceTimer" (void)
{
	int voiceTick;
	
	while (ClassifyActor(0) & ACTOR_ALIVE)
	{
		int PN = PlayerNumber();
		if (voiceTimers[PN] > 0)
			voiceTimers[PN] --;
			
		// Check our voice every second or so
		voiceTick ++;
		if (voiceTick >= 35)
		{
			voiceTick = 0;
			FindMyVoice();
		}
			
		delay(1);
	}
}

// -- ACTUAL CORE SOUND SCRIPT -- //
// This is done server-side, via DECORATE
script "Vocalize" (int snd, int chance) { Vocalize(snd, chance); }
void Vocalize(int snd, int chance = 0, int PN = -1)
{
	// -1 means the activator
	if (PN == -1)
		PN = PlayerNumber();
		
	// -2 means ALL players
	if (PN == -2)
	{
		for (int l=0; l<MAXPLAYERS; l++)
		{
			if (PlayerInGame(l))
				PlayVoiceSound(snd, l);
		}
	}
	else
		PlayVoiceSound(snd, PN);
}

// Convenience scripts
script "VocalSwingLight" (void) { Vocalize(VS_GRUNT, 6); }
script "VocalSwingHeavy" (void) { Vocalize(VS_GRUNT, 15); }
script "VocalHeal" (void) { delay(15); Vocalize(VS_HEAL, 0); }
script "VocalKill" (int vt) { delay(15); Vocalize(vt, 0); }
script "VocalPay" (void) { Vocalize(VS_CASHTOSS, 0); }

//-- PLAY THE ACTUAL SOUND --
// Server-sided
void PlayVoiceSound (int soundmode, int PN)
{
	// -1 means activator
	if (PN == -1)
		PN = PlayerNumber();
	
	// They're not in the game
	if (!PlayerInGame(PN))
		return;
		
	int originalPlayer = PlayerTID(PN);
	int voice = CheckActorInventory(originalPlayer, "PlayerVoice");
	
	// Fallback, if we don't have specific lines
	soundmode = VoiceFallback(voice, soundmode);
	
	// No sound at all!
	if (!ValidLine(voice, soundmode))
		return;
		
	// No chance at all
	int chn = VoiceList[voice].Lines[soundmode].Chance;
	if (chn > 0 && Random(0,100) > chn)
		return;
		
	// Certain sounds have restrictions
	int canPlay = true;
	
	// Voice timer is still active
	if (voiceTimers[PN] > 0)
		canPlay = false;
	
	switch (soundmode)
	{
		//-- RELOAD --
		case VS_RELOAD:
			if (GetUserCVar(PlayerNumber(), "cl_noreloadsound"))
				canPlay = false;
		break;
		
		//-- PAIN --
		case VS_PAIN:
			canPlay = true;
		break;
	}
	
	// We can't play this
	if (!canPlay)
		return;

	voiceTimers[PN] = VoiceList[voice].Lines[soundmode].Duration;
	ThingSound(originalPlayer,VoiceList[voice].Lines[soundmode].Sound,127);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Add the actual voices
void ZT2Voices_Init()
{
	// BO2 Merc
	AddVoicepack("CoD:BO2 (Merc)");
	SetVoiceSound(VS_RELOAD, "merc/reload");
	SetVoiceSound(VS_TAUNT, "merc/taunt", 2.0);
	SetVoiceSound(VS_PAIN, "merc/pain");
	
	// KF Male
	AddVoicepack("Killing Floor (Human)");
	SetVoiceSound(VS_RELOAD, "kfmale/reload");
	SetVoiceSound(VS_TAUNT, "kfmale/taunt", 3.0);
	SetVoiceSound(VS_PAIN, "kfmale/pain");
	SetVoiceSound(VS_GRUNT,"kfmale/grunt", 0.5);
	SetVoiceSound(VS_CASHTOSS, "kfmale/cash", 2.0);
	// SetVoiceSound(VS_HEAL,"kfmale/heal");

	// KF D.A.R.
	AddVoicepack("Killing Floor (D.A.R.)");
	SetVoiceSound(VS_RELOAD, "dar/reload");
	SetVoiceSound(VS_TAUNT, "dar/taunt", 4.0);
	SetVoiceSound(VS_PAIN, "dar/pain");
	SetVoiceSound(VS_GRUNT, "dar/grunt", 0.5);
	SetVoiceSound(VS_CASHTOSS, "dar/cash", 2.0);
	
	// Cod:MW American
	AddVoicepack("CoD:MW (American)");
	SetVoiceSound(VS_RELOAD, "usa/reload");
	SetVoiceSound(VS_TAUNT, "usa/taunt", 2.0);
	SetVoiceSound(VS_PAIN, "rga/pain");
	
	// CoD:WAW German
	AddVoicepack("CoD:WAW (German)");
	SetVoiceSound(VS_RELOAD, "german/reload");
	SetVoiceSound(VS_TAUNT, "german/taunt", 2.0);
	SetVoiceSound(VS_PAIN, "german/pain");
}

//---------------------------------------------
// LANDING SOUND HACK
//---------------------------------------------
#define AIR_HEIGHT 16
#define MAX_FALLSPEED 12.0

script "LandHack" (void) CLIENTSIDE
{
	int mySpeed;
	bool inAir;
	
	// Not the player who activated the server-side script
	if (ConsolePlayerNumber() != PlayerNumber())
		terminate;
	
	while (ClassifyActor(0) && ACTOR_ALIVE)
	{
		int gap = abs((GetActorZ(0) >> 16) - (GetActorFloorZ(0) >> 16));

		// Are we in the air?
		if (gap >= AIR_HEIGHT)
		{
			mySpeed = GetActorVelZ(0);
			if (!inAir)
				inAir = true;
		}
		
		// Must be on the ground
		else if (gap < 1 && inAir)
		{
			inAir = false;
			
			// Must be FALLING
			if (mySpeed < 0.0)
			{
				// Determine how loud we should play it
				int vol = clamp(FixedDiv(abs(mySpeed), MAX_FALLSPEED), 0.0, 1.0);
				PlaySound(0, "player/land", CHAN_VOICE, vol, false, ATTN_NORM);
			}
		}
		
		delay(2);
	}
}