//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
//
// A L L   W E A P O N S
//
//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

// Crosshairs
enum XHair {
	XH_SHOTGUN = 13,
	XH_SNIPER = 14,
	XH_CIRCLESMALL = 15,
	XH_BLUR = 16,
	XH_QUADBIG = 17,
	XH_QUADSMALL = 18,
	XH_LMG = 19,
	XH_LMGSMALL = 21,
	XH_NOTHING = 20,
};

// Various fire modes
str FireModeNames[99] = {
	"Semi-Automatic",
	"Fully-Automatic",
	"",
	"Blunt Melee",
	"Grenade",
	"Semi-Automatic",
	"Fully-Automatic",
	"",
	"",
	"Burst Fire",
};

enum FireModeType {
	FM_SINGLE,
	FM_AUTO,
	FM_SINGLEAUTO,
	FM_MELEE,
	FM_GRENADE,
	FM_SHOTSINGLE,
	FM_SHOTAUTO,
	FM_SHOTSINGLEAUTO,
	FM_SINGLEBURST,
	FM_BURST,
};

#define XHAIR_COUNT			6

// Maximum number of different ammo types that a weapon can have
// Keep it somewhat reasonable
#define AMMOTYPE_COUNT		6

struct AmmoType
{
	str Class;
	str Display;
	str Prefix;
	str Icon;
	str BG;
};

//-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 

#define SPREAD_CROUCH			0.80
#define SPREAD_ADS				0.80
#define SPREAD_SEMI				0.80

#define MAX_UPGRADES 8

// Weapon upgrade struct!
// Contains information about a weapon's upgrades
struct WeaponUpgrade
{
	int Hash;				// Used for purchasing a specific upgrade
	str Name;				// Display name
	str Description;		// The description shown for this upgrade
	int Price;				// How much does it cost?
	str Item;				// The particular item to give, this is APPENDED to the wep name
	str Pic;				// Picture to show for this upgrade
};

#define PERKBONUS_MAX			8

// Main weapon struct, contains lots of info
struct WeaponInfo {
	str class;
	str ammoClassA, ammoClassB;
	int ammoCapA, ammoCapB;
	int fireMode;
	str fireModeItem;
	int crosshairs[XHAIR_COUNT];
	int headshotMult;
	bool undroppable;

	// Wheel info
	bool useWheel;
	str wheelScript;
	str wheelClass;
	
	int ammoTypeCount;
	struct AmmoType ammoTypes[AMMOTYPE_COUNT];
	
	// List of perks that get bonuses on this weapon
	str Perks[PERKBONUS_MAX];
	int PerkSize;
	
	// For dualies
	str DualVariant, SingleVariant;
	
	// -- TRADER NONSENSE -- //
	str DisplayName;			// Actual name to show
	str Pic;					// Buypic
	str Description;			// Description
	str Perk;					// Perk ACTOR this wep belongs to
	int Price;					// Base price for this item
	bool Hidden;				// Don't show this in the trader
	
	// This weapon was registered on the client end
	bool IsLocal;
	
	// This is generated on both client and server
	// Puke requests can refer to this number
	bool Hash;
	
	// Spread-related variables
	int spreadX, spreadY;
	int spreadMultCrouch;
	int spreadMultSemi;
	int spreadMultIS;
	
	// For upgrades
	struct WeaponUpgrade Upgrades[MAX_UPGRADES];
	int UpgradeCount;
};

#define MAX_WEPS 120
int WeaponCount;
struct WeaponInfo WeaponList[MAX_WEPS];

// Get the weapon INDEX by its hash
int WeaponByHash(int Hash)
{
	for (int l=0; l<WeaponCount; l++)
	{
		if (WeaponList[l].Hash == Hash)
			return l;
	}
	
	return -1;
}

// Find a weapon UPGRADE by its hash
int UpgradeByHash(int WI, int Hash)
{
	for (int l=0; l<WeaponList[WI].UpgradeCount; l++)
	{
		if (WeaponList[WI].Upgrades[l].Hash == Hash)
			return l;
	}
	
	return -1;
}

// Do we have an upgrade?
bool HasUpgrade(int WI, int UI)
{
	str FinalName = StrParam(s:WeaponList[WI].Class, s:" ", s:WeaponList[WI].Upgrades[UI].Item);
	return CheckInventory(FinalName);
}

// Get the weapon INDEX for a specific type
int WeaponByType(str class)
{
	for (int l=0; l<WeaponCount; l++)
	{
		if (!StrICmp(WeaponList[l].class, class))
			return l;
	}
	
	return -1;
}

// Find this weapon's reserve ammo
str GetReserveClass(int WI)
{
	if (WeaponList[WI].ammoTypeCount > 0)
	{
		// Find out the CURRENT mode we're in
		int AM = CheckInventory(WeaponList[WI].wheelClass);
		return WeaponList[WI].ammoTypes[AM].Class;
	}
	
	return WeaponList[WI].ammoClassB;
}

// Number of crosshairs this particular weapon has
int CrosshairCount(int index)
{
	int count = 0;
	
	for (int l=0; l<XHAIR_COUNT; l++)
	{
		if (WeaponList[index].crosshairs[l] > 0)
			count ++;
	}
	
	return count;
}
// Throw out weapons on death!
// This should be called at a point where the owner cannot grab them
#define SPIT_RADIUS 		8.0
#define SPIT_STRENGTHMIN 		2.0
#define SPIT_STRENGTHMAX 		4.0

script "SpitWeapons" (void) { SpitWeapons(); }
void SpitWeapons()
{
	int velx, vely, velz, myx, myy, myz;
	
	myx = GetActorX(0);
	myy = GetActorY(0);
	myz = GetActorZ(0) + 16;
	
	for (int l=0; l<WeaponCount; l++)
	{
		// Cannot be dropped
		if (WeaponList[l].undroppable)
			continue;
			
		// We have the wep, throw it
		if (CheckInventory(WeaponList[l].class))
		{
			int ang = Random(0, 1.0);
			int spd = Random(SPIT_STRENGTHMIN, SPIT_STRENGTHMAX);
			
			velz = Random(4.0, 8.0);
			velx = FixedMul(cos(ang), spd);
			vely = FixedMul(sin(ang), spd);
			
			int wepTID = UniqueTID();
			SpawnForced(WeaponList[l].class, myx, myy, myz, wepTID);
			SetActorVelocity(wepTID, velx, vely, velz, false, false);
			Thing_ChangeTID(wepTID, 0);

			InvEnsure(WeaponList[l].class, 0);
		}
	}
}

void RegisterWeapon(str class, str magA, str magB, int capA, int capB, str perk = "Perk_Commando", bool hidden = false)
{
	WeaponList[WeaponCount].DualVariant = "";
	WeaponList[WeaponCount].SingleVariant = "";
	WeaponList[WeaponCount].class = class;
	WeaponList[WeaponCount].ammoClassA = magA;
	WeaponList[WeaponCount].ammoClassB = magB;
	WeaponList[WeaponCount].ammoCapA = capA;
	WeaponList[WeaponCount].ammoCapB = capB;
	WeaponList[WeaponCount].fireMode = FM_SINGLE;
	WeaponList[WeaponCount].fireModeItem = "";
	WeaponList[WeaponCount].headshotMult = 1.0;
	WeaponList[WeaponCount].useWheel = false;
	WeaponList[WeaponCount].Hidden = hidden;
	
	// No bonuses
	WeaponList[WeaponCount].PerkSize = 0;
	
	WeaponList[WeaponCount].IsLocal = IsClient();
	
	WeaponList[WeaponCount].spreadX = -1.0;
	WeaponList[WeaponCount].spreadY = -1.0;
	WeaponList[WeaponCount].spreadMultCrouch = SPREAD_CROUCH;
	WeaponList[WeaponCount].spreadMultSemi = SPREAD_SEMI;
	WeaponList[WeaponCount].spreadMultIS = SPREAD_ADS;
	
	// Trader
	WeaponList[WeaponCount].DisplayName = "Sample Weapon";
	WeaponList[WeaponCount].Price = 200;
	WeaponList[WeaponCount].Description = "This is a gun!";
	WeaponList[WeaponCount].Pic = "BUYPIC03";
	WeaponList[WeaponCount].Perk = perk;
	
	// Make unique identifier hash
	WeaponList[WeaponCount].Hash = FakeHash(StrParam(s:class,s:magA));
	
	// PrintBold(s:class, s:": ", d:WeaponList[WeaponCount].Hash);
	
	WeaponCount ++;
	
	// Register the appropriate magazine
	NewMagazine(magA, magB, capA);
}

// -- STATUS FUNCTIONS
// These must be called AFTER RegisterWeapon

void WeaponSetFM(int FM, str item)
{
	WeaponList[WeaponCount-1].fireMode = FM;
	WeaponList[WeaponCount-1].fireModeItem = item;
}

// TRADER INFORMATION
void WeaponSetTrader(str DN = "DUMMY ITEM", int P = 50, str BP = "BUYPIC03", str Desc = "DUMMY TEXT")
{
	WeaponList[WeaponCount-1].DisplayName = DN;
	WeaponList[WeaponCount-1].Description = Desc;
	WeaponList[WeaponCount-1].Price = P;
	WeaponList[WeaponCount-1].Pic = BP;
}

// Spread-related
void WeaponSetSpread(int SX = -1.0, int SY = -1.0, int SMC = SPREAD_CROUCH, int SMS = SPREAD_SEMI, int SMA = SPREAD_ADS)
{
	WeaponList[WeaponCount-1].spreadX = SX;
	WeaponList[WeaponCount-1].spreadY = SY;
	WeaponList[WeaponCount-1].spreadMultCrouch = SMC;
	WeaponList[WeaponCount-1].spreadMultSemi = SMS;
	WeaponList[WeaponCount-1].spreadMultIS = SMA;
}

void WeaponSetCrosshair(int index, int ID) { WeaponList[WeaponCount-1].crosshairs[index] = ID; }
void WeaponSetHeadshot(int mult) { WeaponList[WeaponCount-1].headshotMult = mult; }
void WeaponSetUndroppable(bool ud) { WeaponList[WeaponCount-1].undroppable = ud; }
void WeaponSetWheel(str WS, str modeClass) 
{ 
	WeaponList[WeaponCount-1].useWheel = true; 
	WeaponList[WeaponCount-1].wheelScript = WS; 
	WeaponList[WeaponCount-1].wheelClass = modeClass; 
}

// Set perks for this weapon
void WeaponSetPerked(str PerkClass, int index = -1)
{
	if (index == -1)
		index = WeaponCount-1;
		
	int PS = WeaponList[index].PerkSize;
	WeaponList[index].Perks[PS] = PerkClass;
	WeaponList[index].PerkSize ++;
}

// Set ammo type for this weapon
void WeaponAddAmmoType(str type, str display, str prefix, str icon, str bg) 
{ 
	int index = WeaponList[WeaponCount-1].ammoTypeCount;
	WeaponList[WeaponCount-1].ammoTypes[index].Class = type; 
	WeaponList[WeaponCount-1].ammoTypes[index].Display = display; 
	WeaponList[WeaponCount-1].ammoTypes[index].Prefix = prefix; 
	WeaponList[WeaponCount-1].ammoTypes[index].Icon = icon; 
	
	if (StrLen(bg) > 0)
		WeaponList[WeaponCount-1].ammoTypes[index].BG = bg; 
	else
		WeaponList[WeaponCount-1].ammoTypes[index].BG = "HUDSHBX1"; 
	
	WeaponList[WeaponCount-1].ammoTypeCount ++;
}

// Add an upgrade for this weapon!
void WeaponAddUpgrade(str Name, str Item, str Description, str Pic, int Price, int ID = -1)
{
	if (ID == -1)
		ID = WeaponCount-1;
		
	int UC = WeaponList[ID].UpgradeCount;
	WeaponList[ID].Upgrades[UC].Hash = FakeHash(Name);
	WeaponList[ID].Upgrades[UC].Name = Name;
	WeaponList[ID].Upgrades[UC].Item = Item;
	WeaponList[ID].Upgrades[UC].Description = Description;
	WeaponList[ID].Upgrades[UC].Pic = Pic;
	WeaponList[ID].Upgrades[UC].Price = Price;
	
	WeaponList[ID].UpgradeCount ++;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Pretty generic iron-sight multiplier for full-auto weps
#define TYPICAL_IS				0.50

// Register ALL in-game weapons
void ZT2WeaponList_Init(bool BeClient = false)
{
	// Prevents double-register in singleplayer
	if (!IsClient() && BeClient)
		return;
		
	// Order Pistol
	RegisterWeapon("Order Pistol", "OrderPrimary", "OrderSecondary", 10, -1, "Perk_Rifleman", true);
	WeaponSetUndroppable(true);
	WeaponSetCrosshair(0, XH_SHOTGUN);
	WeaponSetTrader("Order Pistol", 200, "BUYPIC28", "The Order Pistol, an essential sidearm for any zombie killing machine. Not that great alone, but packs a punch when dual-wielded.");
	WeaponSetSpread(2.0, 2.0, SPREAD_CROUCH, 0.0, 0.70);
	WeaponSetPerked("Perk_Rifleman");
	
	// Sten MK II
	RegisterWeapon("Sten MK II", "StenPrimary", "StenSecondary", 32, -1, "Perk_Commando");
	WeaponSetFM(FM_SINGLEAUTO, "StenFireMode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetTrader("Sten MK II", 700, "BUYPIC01", "The STEN was a family of British 9mm submachine guns used extensively by British and Commonwealth forces throughout World War II and the Korean War, with the Mark II being the most common.");
	WeaponSetSpread(3.0, 3.0, SPREAD_CROUCH, TYPICAL_IS);
	WeaponSetPerked("Perk_Commando");
	
	// Pulverizer
	RegisterWeapon("Pulverizer", "PulvPrimary", "PulvSecondary", 5, -1, "Perk_Demo");
	WeaponSetFM(FM_MELEE, "");
	WeaponSetCrosshair(0, XH_BLUR);
	WeaponSetPerked("Perk_Demo");
	WeaponSetTrader("Pulverizer", 2500, "BUYPIC26", "The Pulverizer is a combination of shotgun shells and raw sledgehammer madness. Delivers a nasty pummeling, or an explosive impact! Your call.\n\n\
	- Alt-fire to perform an explosive swing\n\
	- Explosive swings require ammunition");
	
	// Type-97 Nade
	RegisterWeapon("Type-97 Grenade", "Type97Primary", "", 1, -1, "Perk_Demo", true);
	WeaponSetFM(FM_GRENADE, "");
	WeaponSetCrosshair(0, XH_BLUR);
	WeaponSetUndroppable(true);
	WeaponSetPerked("Perk_Demo");
	WeaponSetTrader("Type-97 Grenade", 100, "BUYPIC27", "Your essential frag grenade. Great for taking out groups of enemies!");
	
	// PPSH
	RegisterWeapon("PPSH-41", "PPSHPrimary", "PPSHSecondary", 70, -1, "Perk_Commando");
	WeaponSetFM(FM_SINGLEAUTO, "PPSHFireMode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetPerked("Perk_Commando");
	WeaponSetTrader("PPSH-41", 600, "BUYPIC00", "The PPSh-41 is a Soviet submachine gun designed by Georgi Shpagin as a cheap, simplified alternative to the PPD-40.");
	WeaponSetSpread(3.0, 3.0, SPREAD_CROUCH, TYPICAL_IS);
	
	// M16
	RegisterWeapon("M16A4 Rifle", "M16Primary", "M16Secondary", 30, -1, "Perk_Commando");
	WeaponSetFM(FM_SINGLEBURST, "M16FireMode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetHeadshot(1.25);
	WeaponSetPerked("Perk_Commando");
	WeaponSetTrader("M16A4 Rifle", 2400, "BUYPIC07", "The M16A4 Rifle is one of the more well-known military assault rifles in existence. A stylish one, at that! Great for taking out single targets with ease.");
	WeaponSetSpread(3.0, 3.0, SPREAD_CROUCH, TYPICAL_IS);
	
	// MP5
	RegisterWeapon("MP5 Submachine Gun", "MP5Primary", "MP5Secondary", 32, -1, "Perk_Commando");
	WeaponSetFM(FM_SINGLEAUTO, "MP5FireMode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetPerked("Perk_Commando");
	WeaponSetSpread(3.0, 3.0, SPREAD_CROUCH, TYPICAL_IS);
	WeaponSetTrader("MP5 Submachine Gun", 800, "BUYPIC03", "The Heckler & Koch MP5 is one \
of the most widely used submachine guns in the world, having been adopted by 40 \
nations and numerous military, law enforcement, intelligence, and security organizations.");
	
	// MP7
	RegisterWeapon("MP7", "MP7Primary", "MP7Secondary", 20, -1, "Perk_Commando");
	WeaponSetFM(FM_SINGLEAUTO, "MP7FireMode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetPerked("Perk_Commando");
	WeaponSetSpread(4.0, 4.0, SPREAD_CROUCH, TYPICAL_IS);
	WeaponSetTrader("H&K MP7", 2000, "BUYPIC16", "Supposedly not fit for civilian use due to the fact that it \
fills anything you shoot at with holes. If you can afford to buy magazines in bulk, then the MP7 is a perfect weapon for you!");
	
	// MP40
	RegisterWeapon("MP40", "MP40Primary", "MP40Secondary", 32, -1, "Perk_Commando");
	WeaponSetFM(FM_SINGLEAUTO, "MP40FireMode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetPerked("Perk_Commando");
	WeaponSetTrader("MP40", 1500, "BUYPIC11", "The Maschinenpistole 40 is an SMG of German design, small yet \
powerful for its size. It is a common weapon amongst German \
forces and is also popular around the world.");
	WeaponSetSpread(3.0, 3.0, SPREAD_CROUCH, TYPICAL_IS);
	
	// M60
	RegisterWeapon("SS-2 Ravager", "RavagerPrimary", "RavagerSecondary", 100, -1, "Perk_Commando");
	WeaponSetFM(FM_SINGLEAUTO, "M60FireMode");
	WeaponSetCrosshair(0, XH_LMG);
	WeaponSetCrosshair(1, XH_LMGSMALL);
	WeaponSetHeadshot(2.00);
	WeaponSetPerked("Perk_Commando");
	WeaponSetTrader("SS-2 Ravager", 4000, "BUYPIC17", "One of the most satisfying LMGs known to man, the SS-2 \
Ravager is your one-stop shop for spraying and praying needs. With a \
standard capacity of 100 bullets, the SS2 is hell in a gun!");
	WeaponSetSpread(4.0, 4.0, 0.50, TYPICAL_IS);
	
	// AA12
	RegisterWeapon("AA12 Shotgun", "AA12Primary", "AA12Secondary", 20, -1, "Perk_Juggernaut");
	WeaponSetFM(FM_SHOTSINGLEAUTO, "AA12FireMode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetHeadshot(2.00);
	WeaponSetPerked("Perk_Juggernaut");
	WeaponSetSpread(6.0, 6.0, SPREAD_CROUCH);
	WeaponSetTrader("AA12 Shotgun", 3400, "BUYPIC06", "One of the most well-known and powerful shotguns on the \
market today, the AA12 is great for clearing out rooms of \
hostile terrorists. Or zombies.");
	
	// Garand - PING
	RegisterWeapon("M1Garand", "M1Primary", "M1Secondary", 8, -1, "Perk_Rifleman");
	WeaponSetCrosshair(0, XH_SNIPER);
	WeaponSetHeadshot(3.00);
	WeaponSetSpread(1.0, 1.0, 0.50, 0.25);
	WeaponSetPerked("Perk_Rifleman");
	WeaponSetTrader("M1 Garand", 2250, "BUYPIC15", "Listen to that ping. The M1 Garand is a classic rifle, doing \
everything from close-range to long-range combat. Although it can only \
hold 8 rounds, the M1 continues to stand the test of time.");
	
	// Lobotomizer
	RegisterWeapon("Zed's Lobotomizer", "ZedPrimary", "ZedSecondary", 7, -1, "Perk_Rifleman");
	WeaponSetCrosshair(0, XH_SNIPER);
	WeaponSetHeadshot(4.00);
	WeaponSetSpread(1.0, 1.0, 0.50, 0.25);
	WeaponSetPerked("Perk_Rifleman");
	WeaponSetTrader("Zed's Lobotomizer", 1800, "BUYPIC05", "The Lobotomizer, perfect for the sniper who dreams of becoming a doctor.\n\n\cGPerfect for brain surgery.");
	
	// C4 Kit
	RegisterWeapon("C4 Kit", "C4Primary", "", 1, -1, "Perk_Demo");
	WeaponSetFM(FM_GRENADE, "");
	WeaponSetCrosshair(0, XH_BLUR);
	WeaponSetPerked("Perk_Demo");
	WeaponSetTrader("C4 Kit", 2000, "BUYPIC09", "C4 is always a fun thing to have around. The Demo's C4 \
kit is a great tool for making explosions from a distance, \
or setting traps. Throw, click, boom.\n\n\
- Alt-fire to detonate C4 mines\n\
- Mines stay until detonated");
	
	// Sticky Bomb
	RegisterWeapon("Sticky Bomb", "StickyPrimary", "", 1, -1, "Perk_Demo");
	WeaponSetFM(FM_GRENADE, "");
	WeaponSetCrosshair(0, XH_BLUR);
	WeaponSetPerked("Perk_Demo");
	WeaponSetTrader("Sticky Bomb", 3500, "BUYPIC23", "It's a box of sticky mines, what more could you want? Flick, stick, boom. Perfect for demos, perfect for fun!\n\n\
- Alt-fire to toggle sticky mode\n\
- Mines explode in proximity to enemies");
	
	// RPG-7
	RegisterWeapon("RPG7", "RPGPrimary", "RPGSecondary", 1, -1, "Perk_Demo");
	WeaponSetFM(FM_GRENADE, "");
	WeaponSetCrosshair(0, XH_BLUR);
	WeaponSetPerked("Perk_Demo");
	WeaponSetTrader("RPG-7", 8000, "BUYPIC19", "Become a dangerous threat to society today, get an RPG-7. It might have to reload after every shot, but the sheer power that it has between each shot makes it more than worth it.");
	
	// Pump Shotgun
	RegisterWeapon("Pump Shotgun", "DukePrimary", "DukeSecondary", 8, -1, "Perk_Juggernaut");
	WeaponSetFM(FM_SHOTSINGLE, "");
	WeaponSetCrosshair(0, XH_SHOTGUN);
	WeaponSetSpread(6.0, 6.0, SPREAD_CROUCH);
	WeaponSetPerked("Perk_Juggernaut");
	WeaponSetWheel("PumpShotgunSetup", "DukeAmmoMode");
		WeaponAddAmmoType("DukeSecondary", "\cD12 Gauge Shells", "", "HUDSHEL1", "HUDSHBX2");
		WeaponAddAmmoType("DukeDragonSecondary", "\cIDragon's Breath Shells", "Dragon", "HUDSHEL2", "HUDSHBX3");
	WeaponSetTrader("Pump Shotgun", 1300, "BUYPIC04", "A standard-issue pump shotgun, equipped \
with a side holder for extra shells. It \
looks like one wielded by a well-known badass.");
	
		// PUMP SHOTGUN UPGRADE: DRAGON'S BREATH
		WeaponAddUpgrade("Dragon's Breath", "Dragon", "Set zombies alight with the power of Dragon's Breath rounds! Can be selected as an alternate ammo type.", "ZTSCREN4", 500);
		
	// M79 Thumper
	RegisterWeapon("M79 Grenade Launcher", "M79Primary", "M79Secondary", 1, -1, "Perk_Demo");
	WeaponSetFM(FM_GRENADE, "");
	WeaponSetCrosshair(0, XH_SHOTGUN);
	WeaponSetPerked("Perk_Demo");
	WeaponSetTrader("M79 Grenade Launcher", 5600, "BUYPIC08", "\
The M79 Grenade Launcher, or Thumper, is a classic single-shot grenade \
launcher that has been around for ages. It not only looks nice, \
but packs a punch too!");

	WeaponSetWheel("M79WheelSetup", "M79Mode");
		WeaponAddAmmoType("M79Secondary", "\cK40mm Explosives", "", "HUDNADE1", "HUDSHBX1");
		WeaponAddAmmoType("M79FlameSecondary", "\cI40mm Dragon's Breath", "Incendiary", "HUDNADE2", "HUDSHBX3");
		WeaponAddAmmoType("M79ShotgunSecondary", "\cD40mm Buckshot", "Shotgun", "HUDNADE3", "HUDSHBX2");
		
		// M79 UPGRADE: DRAGON'S BREATH
		WeaponAddUpgrade("40mm Dragon's Breath", "Incendiary", "Set zombies alight with the power of Dragon's Breath grenades! Can be selected as an alternate ammo type.", "ZTSCREN5", 500);
		WeaponAddUpgrade("40mm Buckshot", "Shotgun", "Get up close and personal with these pellet filled 40mm grenade rounds! Great for close quarter combat.", "ZTSCREN6", 500);
		
	// Franchi SPAS-12
	RegisterWeapon("SPAS-12 Shotgun", "SpasPrimary", "SpasSecondary", 8, -1, "Perk_Juggernaut");
	WeaponSetFM(FM_SHOTSINGLE, "");
	WeaponSetHeadshot(2.00);
	WeaponSetCrosshair(0, XH_SHOTGUN);
	WeaponSetSpread(6.0, 6.0, SPREAD_CROUCH);
	WeaponSetPerked("Perk_Juggernaut");
	WeaponSetTrader("SPAS-12", 2400, "BUYPIC21", "The Franchi SPAS-12 is another of the most well-known shotguns \
in existence. Making its appearance in many popular movies and \
video games, its intimidating appearance is bound to put \
any opponent in a state of unease.");
	
	// Flamethrower
	RegisterWeapon("ZTFlamethrower", "FlamePrimary", "", 300, -1, "Perk_Pyro");
	WeaponSetFM(FM_GRENADE, "");
	WeaponSetCrosshair(0, XH_BLUR);
	WeaponSetPerked("Perk_Pyro");
	WeaponSetTrader("M2 Flamethrower", 3500, "BUYPIC10", "Whether or not owning a flamethrower is legal, the time of zombie apocalypses calls for an obligatory M2 Flamethrower in your arsenal.\n\n\cIStart a disco inferno!");
	
	// Circulo-Cooker
	RegisterWeapon("CirculoCooker", "CirculoPrimary", "CirculoSecondary", 100, -1, "Perk_Pyro");
	WeaponSetFM(FM_GRENADE, "");
	WeaponSetCrosshair(0, XH_NOTHING);
	WeaponSetPerked("Perk_Pyro");
	WeaponSetTrader("Circulo-Cooker", 4000, "BUYPICZZ", "Cook 'em up! The Circulo Cooker uses state of the art microwave technology to heat zombies from the inside out. How explosive!\n\n\
	- Cooked zombies explode after puffing up");
	
	// MAC-10
	RegisterWeapon("MAC-10", "MAC10Primary", "MAC10Secondary", 30, -1, "Perk_Pyro");
	WeaponSetFM(FM_SINGLEAUTO, "MAC10Mode");
	WeaponSetCrosshair(0, XH_QUADBIG);
	WeaponSetCrosshair(1, XH_QUADSMALL);
	WeaponSetPerked("Perk_Pyro");
	WeaponSetTrader("MAC-10", 3000, "BUYPIC25", "Kind of a pistol, but not really! The MAC-10 is a compact SMG, perfect for drive-bys and filling a group of opponents with holes.\n\n\
- Incendiary ammunition for Pyromaniacs!");
	  
	ZT2PerkSystem_AmmoSetup();
}

// HELP! WHAT IS MY WEAPON?
script "ImHolding" (void) NET
{
	Print(s:"SERVER - ", s:GetWeapon());
	ACS_NamedExecuteAlways("ImHoldingClient",0);
}

script "ImHoldingClient" (void) CLIENTSIDE
{
	if (WrongClient())
		terminate;
		
	Print(s:"CLIENT - ", s:GetWeapon());
}