
#define WHEELSTAYTIME 0.1
// How much to offset it from the center
#define CENTERNUMBER 2000
// The minimum sensitivity we must reach to "select" a wheel number
#define SENSCAP 1000
// Mostly for display stuff
#define WHEELSIZE 45.0
// The default script for wheel checking
#define BASESCRIPT "WheelCheck"

// This is the resolution the wheel is "meant" to be at
#define BASERES_X 640
#define BASERES_Y 480

// Holds information about the current wheel
#define WHEEL_MAXSIDES 16

struct WepWheel {
	int wheelSides;
	int wheelAngleMin[WHEEL_MAXSIDES];
	int wheelAngleMax[WHEEL_MAXSIDES];
	str wheelPics[WHEEL_MAXSIDES];
	str wheelDenied[WHEEL_MAXSIDES];
	str wheelRequires[WHEEL_MAXSIDES];
	str wheelBackground;
	int wheelSelected;
};

struct WepWheel CurrentWheel;

//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

// Returns fixed point value
int WWfloor(int fixedNumber)
{
	return fixedNumber & 0xFFFF0000;
}

// absolute value
int WWabs(int numnum)
{
	if (numnum >= 0) {return numnum;}
	return -numnum;
}

// Initialize the wheel
void initializeWheel(int sides, str BG)
{
	CurrentWheel.wheelSides = sides;
	CurrentWheel.wheelBackground = BG;
}

// Set data for a wheel part
void setWheelPart(int part, str BG, int angleMin, int angleMax, str requires, str denyPic)
{
	CurrentWheel.wheelAngleMin[part] = angleMin;
	CurrentWheel.wheelAngleMax[part] = angleMax;
	CurrentWheel.wheelPics[part] = BG;
	CurrentWheel.wheelRequires[part] = requires;
	CurrentWheel.wheelDenied[part] = denyPic;
}

//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
//--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

// -- REDUX OF THE PREVIOUS SCRIPTS --
void WheelMessage (int xoffset, int yoffset, str text, int alignx, int aligny, int id, int showtime)
{
	int vidres_x;
	int vidres_y;
	
	vidres_x = GetCVar("vid_defwidth");
	vidres_y = GetCVar("vid_defheight");
	
	int vidres_xFixed = vidres_x * 1.0;
	int vidres_yFixed = vidres_y * 1.0;
	int alignvar_x;
	int alignvar_y;

	int finaloffset_x;
	int finaloffset_y;

	if (yoffset < 0) { finaloffset_y = vidres_yFixed + yoffset; }
	else { finaloffset_y = yoffset; }

	if (xoffset < 0) { finaloffset_x = vidres_xFixed + xoffset; }
	else { finaloffset_x = xoffset; }

	switch (alignx)
	{
		case 0:
		alignvar_x = ALIGN_LEFT;
		break;

		case 1:
		alignvar_x = ALIGN_CENTER;
		break;

		case 2:
		alignvar_x = ALIGN_RIGHT;
		break;
	}

	switch (aligny)
	{
		case 0:
		alignvar_y = ALIGN_LEFT;
		break;

		case 1:
		alignvar_y = ALIGN_CENTER_Y;
		break;

		case 2:
		alignvar_y = ALIGN_RIGHT;
		break;
	}

	SetHUDSize(vidres_x, vidres_y, false);
	HudMessage(s:text; HUDMSG_PLAIN | HUDMSG_FADEOUT, id, CR_UNTRANSLATED, finaloffset_x + alignvar_x, finaloffset_y + alignvar_y, showtime, 0.1);
	SetHUDSize(0, 0, false);
}

// -- ABILITY WHEEL: USE THE MOUSE -- //
// -- This is entirely clientsided -- //
void ControlMouseMovement(void)
{
	// These are raw pitch / yaw inputs
	int mousex_var;
	int mousey_var;
	mousex_var = GetPlayerInput(-1, INPUT_YAW) * -3;
	mousey_var = GetPlayerInput(-1, INPUT_PITCH) * -3;

	// We have to move the mouse fairly vigorously to affect it
	// We don't want the user nudging their mouse, that's twitchy
	if ((mousex_var >= SENSCAP || mousex_var <= -SENSCAP) || (mousey_var >= SENSCAP || mousey_var <= -SENSCAP))
	{
		// Reset the "mouse" position
		TakeInventory("MouseX",99999);
		TakeInventory("MouseY",99999);
		TakeInventory("MouseNegativeX",1);
		TakeInventory("MouseNegativeY",1);
		
		// Store the mouse values in our inventory
		GiveInventory("MouseX",WWabs(mousex_var));
		GiveInventory("MouseY",WWabs(mousey_var));
		
		// We can't have a negative amount of inventory items, so use these
		if (mousex_var < 0)
			GiveInventory("MouseNegativeX",1);
		if (mousey_var < 0)
			GiveInventory("MouseNegativeY",1);
	}
	
	// These should correspond to the user's resolution
	int resX = GetCVar("vid_defwidth");
	int resY = GetCVar("vid_defheight");
	
	// Get the mouse value based on our inventory items
	int MX = CheckInventory("MouseX");
	int MY = CheckInventory("MouseY");
	if (CheckInventory("MouseNegativeX") > 0)
		MX *= -1;
	if (CheckInventory("MouseNegativeY") > 0)
		MY *= -1;
	
	// This is a vector angle based on X and Y, good for circles
	int vecAng = VectorAngle(MX,MY);
	int finalX = FixedMul(cos(vecAng),WHEELSIZE);
	int finalY = FixedMul(sin(vecAng),WHEELSIZE);
	
	// An "understandable" angle, based on 360 degrees
	int finalWheelAngle = WWfloor(FixedMul(vecAng,360.0));
	finalWheelAngle = 360.0-finalWheelAngle;
	
	// Based on the angle, decide which part we're hovering over
	CurrentWheel.wheelSelected = -1;
	for (int l=0; l<CurrentWheel.wheelSides; l++)
	{
		if (finalWheelAngle >= CurrentWheel.wheelAngleMin[l]*1.0 && finalWheelAngle <= CurrentWheel.wheelAngleMax[l]*1.0)
			CurrentWheel.wheelSelected = l;
	}
	
	// Draw the actual wheel image
	str wheelFont = CurrentWheel.wheelBackground;
	if (CurrentWheel.wheelSelected != -1)
		wheelFont = CurrentWheel.wheelPics[CurrentWheel.wheelSelected];
	SetFont(wheelFont);
	WheelMessage(WWfloor(resX*0.5),WWfloor(resY*0.5),"A",0,0,WHEELID+1,WHEELSTAYTIME);
	
	// Draw the "over" messages
	for (int m=0; m<CurrentWheel.wheelSides; m++)
	{
		if ((CheckInventory(CurrentWheel.wheelRequires[m]) <= 0) && (CurrentWheel.wheelRequires[m] != ""))
		{
			SetFont(CurrentWheel.wheelDenied[m]);
			WheelMessage(WWfloor(resX*0.5),WWfloor(resY*0.5),"A",0,0,WHEELID-m,WHEELSTAYTIME);
		}
		else
			WheelMessage(WWfloor(resX*0.5),WWfloor(resY*0.5),"",0,0,WHEELID-m,WHEELSTAYTIME);
	}
	
	// Remember: X and Y have to be ints not floats, hence the WWfloor
	SetFont("CONFONT");
	WheelMessage(WWfloor((resX*0.5)+finalX),WWfloor((resY*0.5)+finalY),"\cKX",1,1,WHEELID-CurrentWheel.wheelSides,WHEELSTAYTIME);
}

// -- DEFAULT SCRIPT USED FOR WHEEL CHECKING -- //
script "WheelCheck" (int selection) NET
{
	PrintBold(s:"You picked selection number ",d:selection,s:"!");
}

// -- CALLED SERVERSIDE: Trigger wheel selection -- //
script "WheelStart" (void) NET
{
	if (!CheckInventory("UsingWheel") && CheckInventory("CanUseWheel"))
	{
		GiveInventory("UsingWheel",1);
		// Make the player invulnerable and other things
		SetActorProperty(0,APROP_Invulnerable,1);
		SetPlayerProperty(0,1,4);
		SetPlayerProperty(0,1,1);
		SetPlayerProperty(0,1,7);
	}
}

// -- CALLED CLIENTSIDE: We need to exit the menu -- //
script "WheelStop" (void) NET CLIENTSIDE
{
	if (CheckInventory("UsingWheel"))
	{
		// Actually pull us out of the wheel
		NamedRequestScriptPuke("WheelStopServer",0,0,0,0);
		// Do something based on the wheel icon we're hovering over
		NamedRequestScriptPuke("WheelProcess",CurrentWheel.wheelSelected,0,0,0);
	}
}

// -- CALLED SERVERSIDE: Trigger wheel selection -- //
script "WheelStopServer" (void) NET
{
	if (CheckInventory("UsingWheel"))
	{
		TakeInventory("UsingWheel",1);
		// Make the player vulnerable again
		SetActorProperty(0,APROP_Invulnerable,0);
		SetPlayerProperty(0,0,4);
		SetPlayerProperty(0,0,1);
		SetPlayerProperty(0,0,7);
	}
}

// -- EXECUTES CONSTANTLY AS LONG AS THE PLAYER IS ALIVE -- //
// -- (Controls clientsided wheel mumbo jumbo) -- //
script "WheelClient" (void) CLIENTSIDE
{
	while (true)
	{
		if (CheckInventory("UsingWheel"))
			ControlMouseMovement();
			
		delay(1);
	}
}

// -- == -- == -- == -- == -- == -- == -- == -- == -- == -- == -- == -- == -- == 

// Are we allowed to UNLOAD alternate ammo from this gun?
script "AltUnloadAllowed" (void) { SetResultValue(AltUnloadAllowed()); }
bool AltUnloadAllowed()
{
	int WI = WeaponByType(GetWeapon());
	if (WI == -1)
		return false;
	
	if (CheckInventory(WeaponList[WI].ammoClassA) > 0)
		return true;
		
	return false;
}

// Actually PERFORM an alternate unload
script "PerformAltUnload" (void) { PerformAltUnload(); }
void PerformAltUnload()
{
	int WI = WeaponByType(GetWeapon());
	if (WI == -1)
		return;
		
	// This is the OLD unload mode we're currently in
	// We haven't switched to the new one yet
	int UM = CheckInventory(WeaponList[WI].wheelClass);
		
	str primary = WeaponList[WI].ammoClassA;
	str secondary = WeaponList[WI].ammoTypes[UM].Class;
	
	TakeInventory(primary, 1);
	GiveInventory(secondary, 1);
}

// Switch from our OLD unload mode to our NEW unload mode
script "UpdateUnloadMode" (int forceunload)
{
	int WI = WeaponByType(GetWeapon());
	if (WI == -1)
		terminate;
		
	// This unloads AND updates our new mode
	if (forceunload)
		PerformAltUnload();
		
	InvEnsure( WeaponList[WI].wheelClass, CheckInventory("UnloadMode") );
}

// Something happens with the shell wheel
script "WheelProcess" (int selection) NET { WheelProcess(selection); }
void WheelProcess(int selection)
{
	str WN = GetWeapon();
	int WI = WeaponByType(GetWeapon());
	int AM = CheckInventory(WeaponList[WI].wheelClass);
	
	// More ammo types than we have
	if (selection >= WeaponList[WI].ammoTypeCount)
		return;
		
	// Already using this mode
	if (AM == selection)
		return;
		
	// Do we have the required item?
	if (StrLen(WeaponList[WI].ammoTypes[selection].Prefix) > 0)
	{
		if (!CheckInventory( StrParam(s:WN, s:" ", s:WeaponList[WI].ammoTypes[selection].Prefix) ))
			return;
	}
		
	// Do we have enough of each ammo type?
	str primary = WeaponList[WI].ammoClassA;
	str secondary = WeaponList[WI].ammoTypes[selection].Class;
	
	if (/*CheckInventory(primary) > 0 &&*/ CheckInventory(secondary) > 0)
	{
		InvEnsure("UnloadMode", selection);
		GiveInventory("UnloadPending", 1);
		
		Print(s:"\cJ", s:WeaponList[WI].ammoTypes[selection].Display, s:" \cJselected.");
	}
}