// -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
// I N T E R N A L   Z O M B I E   L I S T
// Stores info about EVERY zombie class
// -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

#define ZLIST_SIZE					128
#define BOUNTY_TYPES				4
enum BountyType
{
	PRICE_MAIN,						// 0
	PRICE_HEADSHOT,					// 1
	PRICE_GIB,						// 2
	PRICE_CRIPPLE,					// 3
	PRICE_BURNED,					// 4
};

// Zombie's body shape
// This is used to spawn proper zombies in appropriate spots
enum ZombieBuild
{
	BUILD_HUMAN,			// Plain human build (imp, zombieman, etc.)
	BUILD_MIDGET,			// Dogs, anything that crawls on the ground
	BUILD_LANKY,			// Revenant, facemaw, anything that's tall
	BUILD_BULKY,			// Chaingunners, anything really bulky and wide
	BUILD_HEAD,				// Flying heads, or anything skull-sized
	
	// Used by spawners
	BUILD_ALL,				// Any zombie can spawn here
};

struct ZombieEntry
{
	str Class, CrispyClass;
	int Bounty[BOUNTY_TYPES];
	int XP[BOUNTY_TYPES];
	int Build;
	bool headless;
	int headStrength;
	
	// Can we crawl out of floor / ceiling?
	bool canFloor, canCeiling;
	
	// Minimum wave this zombie can spawn on
	int MinWave;
};

int ZombieListCount;
struct ZombieEntry ZombieList[ZLIST_SIZE];

// Add a brand new zombie
void RegisterZombie(str class, int price, int xp, int minWave = 1)
{
	ZombieList[ZombieListCount].Class = class;
	ZombieList[ZombieListCount].headStrength = 1.0;
	ZombieList[ZombieListCount].MinWave = minWave;
	ZombieList[ZombieListCount].canFloor = false;
	ZombieList[ZombieListCount].canCeiling = false;
	ZombieList[ZombieListCount].Build = BUILD_HUMAN;
	ZombieList[ZombieListCount].Bounty[PRICE_MAIN] = price;
	ZombieList[ZombieListCount].Bounty[PRICE_HEADSHOT] = 30;
	ZombieList[ZombieListCount].Bounty[PRICE_CRIPPLE] = 20;
	ZombieList[ZombieListCount].Bounty[PRICE_GIB] = 50;
	ZombieList[ZombieListCount].XP[PRICE_MAIN] = xp;
	ZombieList[ZombieListCount].XP[PRICE_HEADSHOT] = 10;
	ZombieList[ZombieListCount].XP[PRICE_CRIPPLE] = 5;
	ZombieList[ZombieListCount].XP[PRICE_GIB] = 10;
	
	ZombieListCount ++;
}

void SetZombieBounty(int ind, int val) { ZombieList[ZombieListCount-1].Bounty[ind] = val; }
void SetZombieBuild(int BLD) { ZombieList[ZombieListCount-1].Build = BLD; }
void SetZombieAnims(bool fromFloor, bool fromCeiling) 
{ 
	ZombieList[ZombieListCount-1].canFloor = fromFloor;
	ZombieList[ZombieListCount-1].canCeiling = fromCeiling;
}

void SetZombieHeadless() { ZombieList[ZombieListCount-1].headless = true; }
void SetZombieHeadStrength(int st) { ZombieList[ZombieListCount-1].headStrength = st; }

// Returns the INDEX of a zombie by class
int ZombieByClass(str class)
{
	if (StrLen(class) <= 0)
		class = GetActorClass(0);
		
	for (int l=0; l<ZombieListCount; l++)
	{
		if (!StrICmp(ZombieList[l].Class, class))
			return l;
	}
	
	return -1;
}

void ZT2Zombies_Init()
{
	RegisterZombie("BaseZombie", 30, 10);
	SetZombieAnims(true, false);
	
	RegisterZombie("VaginaZombie", 50, 20);
	SetZombieHeadStrength(1.10);
	SetZombieBuild(BUILD_LANKY);
	
	RegisterZombie("DogZombie", 30, 10);
	SetZombieHeadless();
	SetZombieBuild(BUILD_MIDGET);
	
	RegisterZombie("Eviscerator",50, 70, 5);
	SetZombieHeadStrength(0.40);
	SetZombieBuild(BUILD_BULKY);
	
	RegisterZombie("Cranus", 50, 100, 7);
	SetZombieBuild(BUILD_HEAD);
}

// -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

// Dog sprint
script "DogSprint" (int sprinting)
{
	InvEnsure("Sprinting", sprinting);
}

// -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

// Gets the number of zombies currently alive
function int GetZombieCount (void)
{
	int x;
	
	for (int l=0; l<ZombieListCount; l++)
		x += ThingCountName(ZombieList[l].Class, 0);

	return x;
}

// Called when a zombie dies, subtracts from the zombie pool and gives the player money
// DEPRECATED
script 72 (int argue, int money, int XP) { ACS_NamedExecuteAlways("ZombieDeath", 0); }

// If "instant" then money is calculated IMMEDIATELY
// Things like explosions shouldn't test for headshots, etc.
script "ZombieDeath" (int instant, int deathMode)
{
	if (CheckInventory("HasDied"))
		terminate;
		
	GiveInventory("HasDied", 1);
	
	// Instantly decrease the pool, that's an absolute
	zombiepool --;
	zombieskilled++;
	
	if (!instant)
		delay(2);
		
	// Figure out what this is
	ACS_Execute(801,0,0,0,0);

	// LOOP THROUGH SETACTIVATORTOTARGET UNTIL WE FIND A PLAYER
	// Failsafe for giving XP
	int attempts = 0;
	int success = 0;
	
	int xp, money;
	int extraXP, extraMoney;

	// How much money should we give?
	int ZI = ZombieByClass("");
	if (ZI != -1)
	{
		money = ZombieList[ZI].Bounty[PRICE_MAIN];
		xp = ZombieList[ZI].XP[PRICE_MAIN];
		
		// Type of death we experienced
		// We forced a mode
		if (deathMode > 0)
		{
			extraMoney = ZombieList[ZI].Bounty[deathMode];
			extraXP = ZombieList[ZI].XP[deathMode];
		}
		// HEADSHOT - Extra money!
		else if (CheckInventory("IsDecapitated"))
		{
			extraMoney = ZombieList[ZI].Bounty[PRICE_HEADSHOT];
			extraXP = ZombieList[ZI].XP[PRICE_HEADSHOT];
		}
	}
	
	int killType = GetKillType();

	while (PlayerNumber() < 0 && attempts < 5 && !success)
	{
		SetActivatorToTarget(0);

		if (PlayerNumber() >= 0)
		{
			success = 1;
			GiveInventory("Money",money+extraMoney);
			GiveInventory("SlotKills",1);
			ACS_NamedExecuteAlways("GiveXP",0,xp+extraXP,0,0);
			
			ACS_NamedExecuteAlways("VocalKill",0,killType);
		}
		
		attempts++;
		
		delay(1);
	}
}

// Used to scale the monster's health and speed based on skill
// This also gives items that indicate our "real" height
// (Used for headshots)

// TODO: Make all scripts use the named version, numbers are obsolete
script "ZombieSetup" (void) { ACS_ExecuteAlways(49,0,0,0,0); }
script 49 (void)
{
	GiveInventory("WasSetup", 1);
	GiveInventory("RealHeight", (GetActorProperty(0, APROP_HEIGHT) >> 16));

	int skill = GameSkill();
	int health = GetActorProperty(0, AProp_SpawnHealth);
	
	// Check zombie list
	int ZI = ZombieByClass("");
	if (ZI != -1 && ZombieList[ZI].headless)
		GiveInventory("IgnoreHeadshot", 1);

	switch (skill)
	{
		case 0:
		SetActorProperty(0, AProp_Health, (health * 0.55)>>16);
		SetActorProperty(0, AProp_Speed, FixedMul(GetActorProperty(0,AProp_Speed),0.85));
		break;

		case 1:
		SetActorProperty(0, AProp_Health, (health * 0.75)>>16);
		SetActorProperty(0, AProp_Speed, FixedMul(GetActorProperty(0,AProp_Speed),0.93));
		break;

		case 2:
		SetActorProperty(0, AProp_Health, (health * 1.0)>>16);
		break;

		case 3:
		SetActorProperty(0, AProp_Health, (health * 1.15)>>16);
		SetActorProperty(0, AProp_Speed, FixedMul(GetActorProperty(0,AProp_Speed),1.15));
		break;

		case 4:
		SetActorProperty(0, AProp_Health, (health * 1.50)>>16);
		SetActorProperty(0, AProp_Speed, FixedMul(GetActorProperty(0,AProp_Speed),1.50));
		break;
	}
	
	ACS_NamedExecuteAlways("HatePlayer",0,0,0,0);
	ACS_NamedExecuteAlways("SightChecker",0,0,0,0);
}

// Hate the player that's closest to us, done when spawning
script "HatePlayer" (void)
{
	int storedPlayerTID;
	int lastDistance = 99999;
	
	for (int l=0; l<MAXPLAYERS; l++)
	{
		if (!PlayerInGame(l))
			continue;
			
		// Does this player exist?
		if (ThingCountName("ZTPlayer", PLAYER_BASE_TID+l) > 0)
		{
			// They are closer to us than the last player
			int dist = distance(0, PLAYER_BASE_TID+l);
			if (dist < lastDistance)
			{
				storedPlayerTID = PLAYER_BASE_TID+l;
				lastDistance = dist;
			}
		}
	}
	
	// If it's 0 then we probably didn't find anyone
	if (storedPlayerTID > 0)
		Thing_Hate(0, storedPlayerTID);
}


// Facemaw melee damage
// TODO: GENERALIZE THIS
script "FacemawDamage" (void)
{
	switch (GameSkill())
	{
		case 0:
			SetResultValue(2 * Random(1,4));
		break;
		
		default:
		case 1:
			SetResultValue(2 * Random(3,8));
		break;
		
		case 2:
		case 3:
			SetResultValue(2 * Random(6, 13));
		break;
		
		case 4:
			SetResultvalue(4 * random(8, 13));
		break;
	}
}

//---------------------------------------------------------------------
//
// S P A W N E R   S Y S T E M
// HANDLES ZOMBIE SPAWNERS
//
//---------------------------------------------------------------------

// Locations for various spawners, added dynamically
// These share TIDs, so checking this way is easier
#define MAXSPAWNERS 64

// Maximum radius for spawners to be functional
// This might need to be changed, test this to be sure!
#define SPAWNER_RADIUS			1000

// Maximum builds a spawner can have
#define MAXBUILDS 5

enum SpawnerAnim
{
	ANIM_NONE,
	ANIM_FLOOR,
};

struct SpawnSpotT
{
	int Wave;					// MINIMUM wave this spawner is activated on
	int TID;					// TID of the spawner, just in case
	int X, Y, Z;				// Position of the spawner
	int Builds[MAXBUILDS];		// Type of zombies that should spawn here
	
	// SPECIAL ANIMATION
	int Animation;
	
	// Backend
	int BuildCount;
};

// For backend
struct SpawnDecide
{
	int possibles[64];
	int possibleCount;
	
	int radials[32];
	int radialCount;
};

struct SpawnDecide SpawnDecision;

int spawnSpotCap = 0;
SpawnSpotT spawnSpots[MAXSPAWNERS];

// Zombie classes
#define ZOMBIE_SPAWNERCOUNT 8
#define ZSPAWN_CLASS 0

// Different types of zombies that this spawner supports
#define ZSPAWN_TYPECOUNT 4
str SpawnerTypes[ZOMBIE_SPAWNERCOUNT][1+ZSPAWN_TYPECOUNT] = {

	// Base zombie
	{"ZombieSpawner_Base", "BaseZombie", "BaseZombie", "BaseZombie", "BaseZombie"},
	
	// Facemaw
	{"ZombieSpawner_Vag", "VaginaZombie", "VaginaZombie", "VaginaZombie", "VaginaZombie"},
	
	// Randomized
	{"ZombieSpawner_Random", "BaseZombie", "VaginaZombie", "DogZombie", "SavageImp"},
	
	// Eviscerator
	{"ZombieSpawner_Eviscerator", "Eviscerator", "Eviscerator", "Eviscerator", "Eviscerator"},
	
	// Cranus
	{"ZombieSpawner_Cranus", "Cranus", "Cranus", "Cranus", "Cranus"},
	
	// Dog
	{"ZombieSpawner_Dog", "DogZombie", "DogZombie", "DogZombie", "DogZombie"},
	
	// Blue Dog
	{"ZombieSpawner_BlueDog", "DogZombie", "DogZombie", "DogZombie", "DogZombie"}
};

// Check if we can spawn zombies yet or we are over the pool already
int HasFreeSpots(void)
{
	// How many zombies exist on the map at any one time
	int x = GetZombieCount();
	int y;
	
	if (x >= ZombiePool || x >= GetCVar("sv_maxzombies")) 
		y = 0;
	else 
	{
		y = 1; 
		ACS_Execute(66,0,0,0);
	}
	
	// Tell DECORATE if we can spawn or not
	return y;
}

// We can spawn!
void EnableSpawning(void) { canSpawnZombies = 1; }
void DisableSpawning(void) { canSpawnZombies = 0; }

// Get the delay between spawning waves of zombies
// TODO: LET A CVAR INFLUENCE THIS
int GetSpawnDelay(void)
{
	return 35 * ShaveInt( Random(spawnTimeMin, spawnTimeMax) );
}

// Get the INDEX of a desirable spawn spot
// This checks player distance and such
int DesiredSpawn(void)
{
	// Most desirable spawner
	// COULD THIS BREAK THINGS?
	int storedSpawnerID = -1;
	int minSpawnerDist = 99999;
	
	// Radial spawner cap
	SpawnDecision.radialCount = 0;
	
	// Loop through all the spots we've added
	for (int l=0; l<spawnSpotCap; l++)
	{
		// ONLY if this is within the desired wave
		int spawnerWave = spawnSpots[l].Wave;
		
		// Haven't reached this wave yet
		if (waveno < spawnerWave)
			continue;
			
		int X = spawnSpots[l].X;
		int Y = spawnSpots[l].Y;
		int Z = spawnSpots[l].Z;
		
		int minDistance = 99999;
		int isRadial = 0;
		
		// Check each player and find the closest one
		for (int m=0; m<MAXPLAYERS; m++)
		{
			// Does this player exist?
			if (PlayerInGame(m) && ThingCountName("ZTPlayer", PLAYER_BASE_TID+m) > 0)
			{
				// They are closer to us than the last player
				int dist = distancePoint(X, Y, Z, PLAYER_BASE_TID+m);
				if (dist < minDistance)
					minDistance = dist;
					
				// Are they within a certain distance?
				if (dist <= SPAWNER_RADIUS)
					isRadial = 1;
			}
		}
		
		// Now we have the closest player to THIS PARTICULAR SPAWNER
		// Is this player closer than the other spawners?
		// If so, we'll use this spawner as a fallback if radial fails
		if (minDistance < minSpawnerDist)
		{
			minSpawnerDist = minDistance;
			storedSpawnerID = l;
		}
		
		// Was a player close enough for this to be a radial spawner?
		// These are prioritized on top of our fallback spawner
		// Radial spawners are stored in a list, and a random is chosen from them
		if (isRadial)
		{
			SpawnDecision.radials[SpawnDecision.radialCount] = l;
			SpawnDecision.radialCount ++;
		}
	}
	
	// We have radial spawners to use, so use them!
	if (SpawnDecision.radialCount > 0)
	{
		int spawnerChoice = Random(0, SpawnDecision.radialCount-1);
		return SpawnDecision.radials[spawnerChoice];
	}
	
	// No radial spawners, so just use the one closest to us whatever it might be
	return storedSpawnerID;
}

// Called when the map opens, this is the actual script
// responsible for spawning!
script "SpawnHandler" OPEN
{
	// Always
	while (true)
	{
		// As long as we can spawn zombies
		while (canSpawnZombies)
		{
			// Only if we haven't hit the maximum zombie count
			if (HasFreeSpots())
			{
				// Get the index we want to spawn at
				int SID = DesiredSpawn();
				
				// Spawn a zombie!
				// Just basic shit for now
				DoSpawn(SID);
			}
			
			delay( GetSpawnDelay() );
		}
		
		// Wait a bit, we can't spawn yet
		delay(35);
	}
}

// Called in the Spawn state of each spawner
// Adds the spawner to our global ACS array of spawners
script "AddSpawner" { AddSpawner(); }

int AddSpawner(int AID = -1, int wave = -1)
{
	if (AID == -1)
		AID = 0;
			
	spawnSpots[spawnSpotCap].TID = AID;
	spawnSpots[spawnSpotCap].X = GetActorX(AID);
	spawnSpots[spawnSpotCap].Y = GetActorY(AID);
	spawnSpots[spawnSpotCap].Z = GetActorZ(AID);
	
	str AC = GetActorClass(AID);
	
	// Dedicated floor spawner
	if (AC == "ZombieSpawner_Floor")
		spawnSpots[spawnSpotCap].Animation = ANIM_FLOOR;
	else
		spawnSpots[spawnSpotCap].Animation = ANIM_NONE;
	
	// If our TID is >= 200, use that as our wave
	if (wave == -1 && AID > 200)
		spawnSpots[spawnSpotCap].Wave = AID-200;
	else if (wave >= 0)
		spawnSpots[spawnSpotCap].Wave = wave;
	
	// Types of builds this spawner is allowed to spawn
	spawnSpots[spawnSpotCap].Builds[0] = BUILD_ALL;
	spawnSpots[spawnSpotCap].BuildCount = 1;
	
	spawnSpots[spawnSpotCap].Wave = 1;
	
	SetActorState(AID, "Static", 0);
	
	spawnSpotCap ++;
	
	return spawnSpotCap-1;
}

// Allow a certain build type from this spawner
void SpawnerAllow(int BuildType, int SN = -1)
{
	// Fallback, we didn't specify an ID
	if (SN == -1)
		SN = spawnSpotCap-1;
		
	// We haven't restricted anything yet
	if (spawnSpots[SN].Builds[0] == BUILD_ALL)
		spawnSpots[SN].BuildCount = 0;
		
	spawnSpots[SN].Builds[ spawnSpots[SN].BuildCount ] = BuildType;
	spawnSpots[SN].BuildCount ++;
}

// Set spawner to a ceiling / floor spawner
void SpawnerSpecialize(bool isCeiling, int SN = -1)
{
	// Fallback, we didn't specify an ID
	if (SN == -1)
		SN = spawnSpotCap-1;
		
	// TODO
	if (isCeiling)
		spawnSpots[SN].Animation = ANIM_NONE;
	else
		spawnSpots[SN].Animation = ANIM_FLOOR;
}

// Find the INDEX of a zombie to spawn
int ZombieForSpawner(int SI)
{
	// Wave to search for
	int searchWave = spawnSpots[SI].Wave;
	
	// Requires floor
	bool needFloor = (spawnSpots[SI].Animation == ANIM_FLOOR);
	
	// Requires ceiling
	bool needCeiling = false;
	
	SpawnDecision.possibleCount = 0;
	
	for (int l=0; l<ZombieListCount; l++)
	{
		// Check animation requirements
		if ((needFloor & !ZombieList[l].canFloor) || (needCeiling && !zombieList[l].canCeiling))
			continue;
	
		if (currentwave >= ZombieList[l].MinWave && ZombieList[l].MinWave >= searchWave)
		{
			// Does this zombie match one of our builds?
			bool buildMatch = false;
			
			if (spawnSpots[SI].Builds[0] == BUILD_ALL)
				buildMatch = true;
			else
			{
				// Try to find a build that matches
				for (int m=0; m<spawnSpots[SI].BuildCount; m++)
				{
					if (ZombieList[l].Build == spawnSpots[SI].Builds[m]) { buildMatch = true; break; }
				}
			}
			
			// Does this build match?
			if (buildMatch)
			{
				SpawnDecision.possibles[SpawnDecision.possibleCount] = l;
				SpawnDecision.possibleCount ++;
			}
		}
	}
	
	// NO INDEX
	if (SpawnDecision.possibleCount <= 0)
		return -1;
	
	int desired = Random(0, SpawnDecision.possibleCount-1);
	
	return SpawnDecision.possibles[desired];
}

script "SpawnerTest" (void)
{
	for (int l=0; l<64; l++) { ZombieForSpawner(0); }
}

// Spawn a spot from that spawner!
void DoSpawn(int spawnerIndex)
{
	int ZI = ZombieForSpawner(spawnerIndex);
	if (ZI == -1)
		return;
	
	str zombieClass = ZombieList[ZI].Class;

	int UID = UniqueTID();
	Spawn(zombieClass, spawnSpots[spawnerIndex].X, spawnSpots[spawnerIndex].Y, spawnSpots[spawnerIndex].Z, UID, 0);
	
	// Ceiling / floor
	if (spawnSpots[spawnerIndex].Animation == ANIM_FLOOR)
		GiveActorInventory(UID, "Spawned_Floor", 1);
		
	Thing_ChangeTID(UID, 0);
}

//--------------------------------------------------------
// HANDLER FOR "LOST" ZOMBIES
//--------------------------------------------------------

// Interval is time in SECONDS to check for players
#define LOST_INTERVAL			5
#define LOST_RANGE 				512

// Are we in a condition to teleport to a spawn spot?
bool IsLost(void)
{
	bool lost = true;
	
	// Check ALL players
	for (int l=0; l<MAXPLAYERS; l++)
	{
		int TID = PlayerTID(l);
		if (PlayerInGame(l))
		{
			// Calculate distance
			int dist = distance(ActivatorTID(), TID);
			if (dist <= LOST_RANGE)
			{
				lost = false;
				break;
			}
				
			// Calculate line of sight
			if (CheckSight(ActivatorTID(), TID, 0))
			{
				lost = false;
				break;
			}
		}
	}
	
	return lost;
}

script "SightChecker" (void)
{
	int cTick = 0;
	
	while (ClassifyActor(0) & ACTOR_ALIVE)
	{
		cTick ++;
		
		if (cTick >= 35*LOST_INTERVAL)
		{
			cTick = 0;
			int lost = IsLost();
			
			if (lost)
			{
				int SS = DesiredSpawn();
				if (SS != -1)
				{
					int x = spawnSpots[SS].X;
					int y = spawnSpots[SS].Y;
					int z = spawnSpots[SS].Z;
					SetActorPosition(0, x, y, z, 0);
				}
			}
		}
		
		delay(1);
	}
}

// A_SetSize doesn't exist in Zan 3.0
script "Corpsify" (int ht, int hl)
{
	SetActorProperty(0, APROP_Height, ht * 1.0);
	SetActorProperty(0, APROP_Health, hl);
}

// Our fake corpse was killed
// This should only be done for things that ordinarily kill corpses
script "CorpseKilled" (void)
{
	if (SetActivator(0, AAPTR_MASTER))
		SetActorState(0, "CorpseDeath_Explode", true);
}

script "CorpseSpawn" (void)
{
	int AID = ActivatorTID();
	int UAID = UniqueTID();
	int UCID = UniqueTID();
	
	Thing_ChangeTID(0, UAID);
	SpawnForced("InvisCorpse", GetActorX(0), GetActorY(0), GetActorZ(0), UCID);
	
	SetActivator(UCID);
	SetPointer(AAPTR_MASTER, UAID);

	Thing_ChangeTID(0, 0);
	
	SetActivator(UAID);
	Thing_ChangeTID(0, AID);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// B U R N   M E C H A N I C S

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Called from the PUFF when it hits an actor
// SERVERSIDED
//
// 0: Hit a wall
// 1: Hit an actor
script "FlamePuff" (void)
{
	str Me = GetActorClass(0);
	int MyDam = GetActorProperty(0, APROP_Damage);
	
	// When we hit a monster, they're set as our target
	if (SetActivatorToTarget(0))
	{
		str Them = GetActorClass(0);
		
		if (Them != Me)
		{
			FlameHit(true, MyDam*2);
			SetResultValue(1);
		}
		else
			SetResultValue(0);
	}
	else
		SetResultValue(0);
}

// Called from the projectile when it hits an actor
// SERVERSIDED
script "FlameHit"() { FlameHit(); }

// Called from FlameForce, we were hit with a radial DoT thing
// SERVERSIDED
script "FlameRadial"()
{
	delay(1);
	FlameHit(true, CheckInventory("FlameForceDamage") * 2);
	InvEnsure("FlameForceDamage", 0);
}

void FlameHit(bool sameTarget = false, int Dam = 0)
{
	// We didn't specify damage
	if (Dam <= 0)
		Dam = GetActorProperty(0, APROP_Damage);

	// Set the target to the zombie
	if (!sameTarget)
		SetActivatorToTarget(0);
		
	// No head!
	if (CheckInventory("IsDecapitated"))
		return;
		
	// Not alive
	if (!Living())
		return;
	
	// WasSetup is ONLY given to zombies, no other type of actors
	if (!CheckInventory("WasSetup"))
		return;
		
	// Increase our DoT damage, regardless
	int DOTD = Dam / 2;
	int DOTD_Goal = clamp(CheckInventory("DoTDamage") + DOTD, 0, DOT_CAP);
	InvEnsure("DoTDamage", DOTD_Goal);
	
	// Reset DoT timer when we do damage to them
	InvEnsure("DoTReset", 1);
		
	// Are we already on fire?
	// TODO: INCREASE BURN TIME
	if (CheckInventory("Burning"))
		return;
		
	GiveInventory("Burning", 1);
	
	ACS_NamedExecuteAlways("FlameDoT", 0);
}

#define DOT_INTERVALS			5
#define DOT_DELAY				35
#define DOT_CAP					30

// Damage over Time for flame things
// This is called from the zombie itself
// SERVERSIDED
script "FlameDoT" (void)
{
	// Number of times we did damage
	int times = 0;
	
	// How many times should we do damage?
	int goal = DOT_INTERVALS;
	
	// Spawn a flame CLINGER
	ACS_NamedExecuteAlways("FlameFXClient",0,1);
	
	while (Living() && CheckInventory("Burning") && times < goal)
	{
		// Do we have a DoT Reset item?
		if (CheckInventory("DoTReset"))
		{
			times = 0;
			InvEnsure("DoTReset", 0);
		}
		
		delay(DOT_DELAY);
		
		// Actually do damage
		int Dam = CheckInventory("DoTDamage");
		Thing_Damage2(0, Dam, "Fire");
		
		// CVAR check can get costly, do your debugging in singleplayer
		if (SinglePlayer())
		{
			if (GetUserCVar(0,"cl_zombiedebugger"))
				PrintBold(s:"OUCH: ", d:Dam, s:" ", d:GetActorProperty(0, APROP_HEALTH));
		}
		
		times ++;
	}
	
	// Stop burning!
	TakeInventory("Burning", 1);
	ACS_NamedExecuteAlways("FlameFXClient",0,0);
}

// Spawned from the monster itself
// SERVER -> CLIENT
script "FlameFXClient" (int burner) CLIENTSIDE
{
	if (burner > 0)
	{
		InvEnsure("BurnFX", burner);
		// Spawn a flame CLINGER
		int FID = UniqueTID();
		int MID = UniqueTID();
		int AID = ActivatorTID();
		
		Thing_ChangeTID(0, MID);
		
		SpawnForced("FlameClinger", GetActorX(0), GetActorY(0), GetActorZ(0) + Half(GetActorProperty(0, APROP_Height)), FID);
		SetActivator(FID);
		SetPointer(AAPTR_TARGET, MID);
		SetActivator(MID);
		
		Thing_ChangeTID(MID, AID);
		Thing_ChangeTID(FID, 0);
	}
	else
		InvEnsure("BurnFX", 0);
}

// Sets height and width appropriately
// Called from clinger in DECORATE
// CLIENT
script "FlameLatch" (void)
{
	int UID = UniqueTID();
	Thing_ChangeTID(0, UID);
	
	if (SetActivatorToTarget(0))
	{
		SetActorProperty(UID, APROP_ScaleX, GetActorProperty(0, APROP_Radius));
		SetActorProperty(UID, APROP_ScaleY, GetActorProperty(0, APROP_Height));
	}
	
	Thing_ChangeTID(UID, 0);
}

// Should a clinger continue spawning flames?
// Called from clinger in DECORATE
// CLIENT
script "FlameContinue" (void)
{
	// Query the zombie we're attached to
	if (SetActivatorToTarget(0))
	{
		bool alive = false;
		int BFX = CheckInventory("BurnFX");
		
		if (BFX == 2)
			alive = true;
		else if (BFX == 1 && Living())
			alive = true;
		
		if (alive)
			SetResultValue(1);
		else
			SetResultValue(0);
	}
	else
		SetResultValue(0);
}

// Hurt self, since A_DamageSelf isn't in zan 3.0
script "FlameOuch" (void) { Thing_Damage2(0, 40, "None"); }